/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.addressing;

import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.exception.ArgumentException;
import dk.gov.oiosi.exception.NullArgumentException;

/**
 * Represents an URL endpoint address.
 */
public class EndpointAddressSMTP extends EndpointAddress {

    /**
     * Endpoint.
     */
    private String endpointSmtp;


    /**
     * Constructs and EndpointAddressSMTP from an URI formatted email address,
     * e.g. "mailto:test@mail.com".
     * 
     * @param uri
     *            an URI formatted email address, e.g. "mailto:test@mail.com"
     */
    public EndpointAddressSMTP(Uri uri) {
        if (uri == null) {
            throw new NullArgumentException("Email address was null");
        }
        
        String address = uri.getAbsoluteUri();

        if (!address.toLowerCase().startsWith("mailto:") || address.length() < 12) {
            throw new ArgumentException("Email format must be in the format 'mailto:test@mail.com': " + endpointSmtp);
        }

        // Axis uses whole url as endpoint address
        endpointSmtp = address;
    }

    /**
     * Gets key as string.
     * 
     * @return mailto url
     */
    public String getKeyAsString() {
        return endpointSmtp;
    }

    /**
     * Gets keytype as string.
     * 
     * @return the key type as a string
     */
    public String getKeyTypeAsString() {
        return "email";
    }

    /**
     * Return the Microsoft.Uddi.UrlType as a sting.
     * 
     * @return the url
     */
    public String getUrlTypeAsString() {
        return "mailto";
    }
}
