/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.addressing;

import dk.gov.oiosi.exception.NullOrEmptyArgumentException;

/**
 * EAN Identifier.
 */
public class IdentifierEan implements IIdentifier {

    
    /**
     * ean representation of Identifier.
     */
    private String eanNumber;

    /**
     * EAN Identifier.
     * 
     * @param value
     *            ean representation of Identifier
     */
    public IdentifierEan(final String value) {
        set(value);
    }

    /* (non-Javadoc)
     * @see dk.gov.oiosi.addressing.IIdentifier#set(java.lang.String)
     */
    public void set(final String value) {
        if (value == null || value.length() == 0) {
            throw new NullOrEmptyArgumentException("eanNumber");
        }
        if (value.length() != 13) {
            throw new IncorrectBusinessIdentifierException("Not a valid ean number [" + value + "], length is not 13.");
        }
        try {
            Long.parseLong(value);
        } catch (NumberFormatException e) {
            throw new IncorrectBusinessIdentifierException("Not a valid ean number [" + value + "], contains non digits.");
        }
        eanNumber = value;
    }

    /**
     * Return the String representation of Identifier.
     * 
     * @return the EAN number as a string
     */
    public String getAsString() {
        return eanNumber;
    }

    /**
     * Compare two Identifiers.
     * 
     * @param other
     *            Identifier to compare
     * @return true if equal
     */
    public boolean equals(IdentifierEan other) {
        if (eanNumber != null) {
            return eanNumber.equals(other.getAsString());
        }
        return false;
    }
    
    /* (non-Javadoc)
     * @see dk.gov.oiosi.addressing.IIdentifier#getType()
     */
    public String getType() {
        return "EAN";
    }

}
