/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.axis2.module.customheaders;

import java.util.List;

import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.policy.OiosiPolicyBean;

/**
 * Add custom headers to outgoing messages.
 */
public class CustomHeaderOutHandler extends RaspHandler {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(CustomHeaderOutHandler.class);

    /**
     * Name.
     */
    public static final String NAME = "CustomHeaderOutHandler";

    /**
     * Description.
     */
    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("deafult Handler");

    /**
     * Description.
     */
    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public CustomHeaderOutHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#init(org.apache.axis2.description.HandlerDescription)
     */
    public void init(HandlerDescription handlerdesc) {
        this.handlerDesc = handlerdesc;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#invoke(org.apache.axis2.context.MessageContext)
     */
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {

        log.debug("CustomHeaderOutHandler : invoke " + msgContext.getSoapAction());

        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
            log.debug("Rasp module not engaged - skip");
            return InvocationResponse.CONTINUE;
        }

//        if (msgContext.isServerSide()) {
//            // Serverside :: SKIP FOR NOW!" );
//            return InvocationResponse.CONTINUE;
//        }

        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);
        if (!oiosiPolicyBean.doCustomHeaders()) {
            log.debug(" - policy defined skip...");
            return InvocationResponse.CONTINUE;
        }

        Object transferedList = msgContext.getProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_LIST);
        Object transferedMessageId = msgContext.getProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER);
        log.warn("CustomHeader List : " + transferedList);
        log.warn("MessageIdentifier : " + transferedMessageId);
        if (transferedList == null && transferedMessageId == null) {
            log.debug("No Custom Headers to set");
            return InvocationResponse.CONTINUE;
        }

        SOAPEnvelope envelope = msgContext.getEnvelope();
        SOAPHeader header = envelope.getHeader();

        //
        List headerList = (List) transferedList;

        boolean rmMessage = RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction());

        for (int i = 0; i < headerList.size(); i++) {
            SOAPHeaderBlock shb = (SOAPHeaderBlock) headerList.get(i);

            if (dk.gov.oiosi.Constants.RASP_MESSAGE_IDENTIFIER.equals(shb.getLocalName()) && rmMessage) {
                // SKIP MessageIdentifier for RM
            } else {
                log.debug("Add : " + shb.getLocalName());
                header.addChild(shb);
            }
        }

        if (!rmMessage && transferedMessageId != null) {
            log.debug("Add : MessageIdentifier" + transferedMessageId);
            header.addChild((SOAPHeaderBlock) transferedMessageId);
        }

        return InvocationResponse.CONTINUE;

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#flowComplete(org.apache.axis2.context.MessageContext)
     */
    public void flowComplete(MessageContext arg0) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getName()
     */
    public String getName() {
        return NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getParameter(java.lang.String)
     */
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getHandlerDesc()
     */
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }

}
