/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.common;

import org.apache.axiom.om.util.UUIDGenerator;

/**
 * Wrap Axiom Guid.
 */
public class Guid {

    /**
     * Value of GUID.
     */
    private String guid;

    /**
     * Construct new GUID with new random uid.
     */
    public Guid() {
        guid = UUIDGenerator.getUUID();
    }

    /**
     * Construct new GUID based on argument.
     * 
     * @param argument
     *            GUID value
     */
    public Guid(final String argument) {
        guid = argument;
    }

    /**
     * GUID.
     * 
     * @return value of GUID
     */
    public final String getGuid() {
        return guid;
    }

    /**
     * GUID Factory.
     * 
     * @return new Guid
     */
    public static Guid newGuid() {
        return new Guid();
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public final boolean equals(final Object object) {
        if (object == null) {
            return false;
        }
        if (!(object instanceof Guid)) {
            return false;
        }
        Guid other = (Guid) object;

        if (guid.equals(other.guid)) {
            return true;
        }

        return false;
    }

}
