/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.common;

/**
 * TimeSpan.
 */
public class TimeSpan {

    /**
     * span in millis.
     */
    private long span;

    /**
     * Default Constructor.
     */
    public TimeSpan() {
        span = 0;
    }

    /**
     * Init TimeSpan to millis.
     * 
     * @param spanMillis
     *            sets the timespan.
     */
    public TimeSpan(final long spanMillis) {
        span = spanMillis;
    }

    /**
     * Return timespan.
     * 
     * @return timespan in millis.
     */
    public final long getSpanMillis() {
        return span;
    }

    /**
     * Set the timespan.
     * 
     * @param millis
     *            timespan.
     * @return timespan in millis.
     */
    public final TimeSpan setMillis(final long millis) {
        span = millis;
        return this;
    }

    /**
     * Set the timespan in seconds. eg. new TimeSpan().setSeconds( 42 );
     * 
     * @param seconds
     *            timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setSeconds(final long seconds) {
        span = seconds * 1000;
        return this;
    }

    /**
     * Set the timespan in minuttes. eg. new TimeSpan().setMinuttes( 42 );
     * 
     * @param minuttes
     *            timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setMinuttes(final long minuttes) {
        span = minuttes * 60 * 1000;
        return this;
    }

    /**
     * Set the timespan in hours. eg. new TimeSpan().setHours( 42 );
     * 
     * @param hours
     *            timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setHours(final long hours) {
        span = hours * 60 * 60 * 1000;
        return this;
    }
}
