/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.common.cache;

/**
 * Interface that represents a timed cache.
 */
public interface ITimedCache {

    /**
     * Adds the key-value pair to the cache.
     * 
     * @param key
     *            key for cached item
     * @param value
     *            item to cache
     */
    void add(Object key, Object value);

    /**
     * Lookup whether a given key is represented in the cache.
     * 
     * @param key
     *            key to search for
     * @return true if key exists
     */
    boolean containsKey(Object key);

    /**
     * Removes a given key-value pair from the cache, based on the key.
     * 
     * @param key
     *            remove item for key
     */
    void remove(Object key);

    /**
     * Try to get the value from a given key.
     * 
     * @param key
     *            search for item with key
     * @return value or null if unknown
     */
    Object tryGetValue(Object key);
    
}
