/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication;

import dk.gov.oiosi.security.oces.OcesX509Certificate;

/**
 * Representing RASP credentials.
 * 
 * NOTE: In Java/Axis - or now only Server/Receiver party certificate is
 * specified. Sender/Self certificate/key could in future be specified as a
 * keylabel for Rampart.
 */
public class Credentials {
    /**
     * The server/receiver certificate.
     */
    private OcesX509Certificate serverCertificate;

    /**
     * Constructor.
     * 
     * @param server
     *            the server(receiver) certificate
     */
    public Credentials(final OcesX509Certificate server) {
        serverCertificate = server;
    }

    /**
     * Get the server/receiver certificate.
     * @return server certificate
     */
    public final OcesX509Certificate getServerCertificate() {
        return serverCertificate;
    }
}
