/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication;

import org.apache.axiom.om.OMElement;

import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.xml.XmlDocument;

/**
 * Response gotten from a RASP http or email endpoint.
 * TODO: Implement properties.
 */
public class Response implements IResponse {
    /**
     * The actual response message.
     */
    private OiosiMessage responseMessage;
    /**
     * Response Uri.
     */
    private Uri responseUri = null;
	
    /**
     * Constructor that takes the OMElement, the message id, to and from as parameter.
     * 
     * @param e response message
     * @param msgId received messageid
     * @param rTo reply to
     * @param from from
     */
    public Response(OMElement e, String msgId, String rTo, String from) {
        // Get the xml body
        XmlDocument messageXml = new XmlDocument(e);

        // Check if the WCF Message.MessageId is a guid, if not, use a new one
        Guid messageId;
        if (msgId != null) {
            messageId = new Guid(msgId);
        } else {
            messageId = Guid.newGuid();
        }
        // Check if the WCF Message.RelatesTo is a guid, if not, use a new one
        Guid relatesTo;
        if (rTo != null) {
            relatesTo = new Guid(rTo);
            responseMessage = new OiosiMessage(messageXml, messageId, relatesTo);
        } else {
            responseMessage = new OiosiMessage(messageXml, messageId);
        }
        
        // Who was the response from?
        if (from != null) {
            responseUri = new Uri(from);
        }
    }

    /**
     * The actual response message.
     * @return received message
     */
    public OiosiMessage getResponseMessage() {
        return responseMessage;
    }

    /**
     * Gets the response URI.
     * @return the response uri
     */
    public Uri getResponseUri() {
        return responseUri;
    }
}
