/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication;

/**
 * Representing send policies.
 */
public class SendPolicy {
    private RMPolicy mailPolicy = new RMPolicy(2, 60000, false);
    private RMPolicy httpPolicy = new RMPolicy(5, 5000, true);
    /**
     * Name of WSS policy file.
     */
    private String policyFileName = "./conf/wss_policy.xml";
    /**
     * Axis configuration file.
     */
    private String axisConfigurationFile = "./conf/client_axis2.xml";
    /**
     * Axis repository.
     */
    private String axisRepsitoryDir = "./repository";
    /**
     * perform client side XML transformation. 
     */
    private boolean xmlTransformation = false;

    /**
     * perform client side Schematron test. 
     */
    private boolean xmlSchematron = false;

    /**
     * perform client side Schema validation. 
     */
    private boolean xmlValidation = false;

    /**
     * client side custom header check.
     */
    private boolean customHeaders = false;
    
    /**
     * Gets the mail reliable messaging policy
     * 
     * @return the mail reliable messaging policy
     */
    public RMPolicy getMailRMPolicy() {
        return mailPolicy;
    }

    /**
     * Sets the mail reliable messaging policy
     * 
     * @param policy
     */
    public void setMailRMPolicy(RMPolicy policy) {
    	mailPolicy = policy;
    }
    
    /**
     * Gets the HTTP reliable messaging policy
     * 
     * @return the HTTP reliable messaging policy
     */
    public RMPolicy getHttpRMPolicy() {
        return httpPolicy;
    }
    
    /**
     * Sets the HTTP reliable messaging policy
     * 
     * @param policy
     */
    public void setHttpRMPolicy(RMPolicy policy) {
        httpPolicy = policy;
    }
    
    /**
     * Sets the name of WSS policy file.
     * 
     * @param v
     *            policy file
     */
    public void setWSSPolicyFileName(String v) {
        policyFileName = v;
    }

    /**
     * Gets the name of WSS policy file.
     * 
     * @return policy file
     */
    public String getWSSPolicyFileName() {
        return policyFileName;
    }

    /**
     * Gets the Axis configuration file.
     * 
     * @return axis configuration file
     */
    public String getAxisConfigurationFile() {
        return axisConfigurationFile;
    }

    /**
     * Sets the Axis configuration file.
     * 
     * @param value
     *            axis configuration file
     */
    public void setAxisConfigurationFile(String value) {
        axisConfigurationFile = value;
    }

    /**
     * Gets the Axis repository.
     * 
     * @return Axis repository
     */
    public String getAxisRepositoryDir() {
        return axisRepsitoryDir;
    }

    /**
     * Sets the Axis repository.
     * 
     * @param value
     *            Axis repository
     */
    public void setAxisRepositoryDir(String value) {
        axisRepsitoryDir = value;
    }

    /**
     * Get whether to perform client side XML transformation. 
     * @return perform client XML transformation
     */
    public boolean doXmlTransformation() {
        return xmlTransformation;
    }

    /**
     * Sets whether to perform client side XML transformation. 
     * @param value
     */
    public void setXmlTransformation(boolean value) {
        this.xmlTransformation = value;
    }

    /**
     * Gets whether to perform client side Schematron test. 
     * @return perform client schematron test
     */
    public boolean doXmlSchematron() {
        return xmlSchematron;
    }

    /**
     * Sets whether to perform client side Schematron test. 
     * @param value
     */
    public void setXmlSchematron(boolean value) {
        this.xmlSchematron = value;
    }

    /**
     * Gets whether to perform client side Schema validation. 
     * @return boolean whether to do XML validation
     */
    public boolean doXmlValidation() {
        return xmlValidation;
    }

    /**
     * Sets whether to perform client side Schema validation. 
     * @param value
     */
    public void setXmlValidation(boolean value) {
        this.xmlValidation = value;
    }

    /**
     * Gets whether to do client side custom header check.
     * @return boolean whether to do custom header check
     */
    public boolean doCustomHeaders() {
        return customHeaders;
    }

    /**
     * Sets whether to do client side custom header check.
     * @param value
     */
    public void setCustomHeaders(boolean value) {
        this.customHeaders = value;
    }
}
