/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication.client;

import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.SOAPFault;
import org.apache.axiom.soap.SOAPFaultSubCode;
import org.apache.axiom.soap.SOAPFaultValue;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.client.async.AxisCallback;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.deployment.FileSystemConfigurator;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.engine.AxisConfigurator;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.transport.http.HttpTransportProperties.ProxyProperties;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyEngine;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityEngineResult;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerResult;
import org.wso2.mercury.callback.MercuryErrorCallback;
import org.wso2.mercury.client.MercuryClient;
import org.wso2.mercury.exception.RMException;
import org.wso2.mercury.util.MercuryClientConstants;
import org.wso2.mercury.util.RMUtil;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.communication.FaultReturnedException;
import dk.gov.oiosi.communication.RMPolicy;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.SignatureValidationProof;
import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentHashMap;

/**
 * Low level Dispatcher. Sets up Axis2 Stack.
 */
public class AxisMercuryDispatcher implements IAxisDispatcher {

    private Log log = LogFactory.getLog(AxisMercuryDispatcher.class);
    private boolean useRM = true;
    private boolean useWSS = true;
    
    public void setUseRM(boolean v) {
        useRM = v;
    }

    public void setUseWSS(boolean v) {
        useWSS = v;
    }

    private RMPolicy httpRMPolicy = null;
    private RMPolicy mailRMPolicy = null;

    public void setHttpRMPolicy(RMPolicy policy) {
        httpRMPolicy = policy;
    }
    public void setMailRMPolicy(RMPolicy policy) {
        mailRMPolicy = policy;
    }
    
    
    /**
     *	@deprecated Doesn't do anything, and will be removed in next release 
     */
    public void cleanUp() {
    }
    

    /**
     * Resolve resource file name.
     */
    private String getFileName(String f) {
        try {
            URL url = AxisMercuryDispatcher.class.getClassLoader().getResource(f);
            return url.getFile();
        } catch (Exception e) {
            return f;
        }
    }

    private static String configContextKey = null;
    private static ConfigurationContext configContext = null;
    private static ConcurrentHashMap wssPolicyMap = new ConcurrentHashMap();
    private Policy wssPolicy = null;

    public static void registerConfigurationContext(String axisXml, String axisRepository, ConfigurationContext configurationContext) {
        configContextKey = "" + axisXml + "::" + axisRepository;
        configContext = configurationContext;
    }

    public AxisMercuryDispatcher(String axisXml, String axisRepository, String wssPolicyFile) throws Exception {
        String key = "" + axisXml + "::" + axisRepository;
        synchronized (AxisMercuryDispatcher.class) {
            // init Axis2 configuration context
            if (configContext == null) {
                configContextKey = "" + axisXml + "::" + axisRepository;

                log.info("Initialize Axis Configuration - axis2.xml : " + axisXml + " - repository : " + axisRepository);
                AxisConfigurator axisConf = new FileSystemConfigurator(getFileName(axisRepository), getFileName(axisXml));
                configContext = ConfigurationContextFactory.createConfigurationContext(axisConf);
            } else {
                log.debug("Reuse Axis Configuration - axis2.xml : " + axisXml + " - repository : " + axisRepository);
                if (!configContextKey.equals(key)) {
                    log.warn("This Implementation only supports one configuration context");
                }
            }
        }

        // init wss policy
        wssPolicy = (Policy) wssPolicyMap.get(wssPolicyFile);
        if (wssPolicy == null) {
            log.info("Initialize WSS Policy from : " + wssPolicyFile);
            wssPolicy = loadPolicy(wssPolicyFile);
            wssPolicyMap.put(wssPolicyFile, wssPolicy);
        } else {
            log.debug("Reuse WSS Policy : " + wssPolicyFile);
        }

        
        // TODO: Shouldn't these be read from config?
        httpRMPolicy = new RMPolicy(5, 5000, true);
        mailRMPolicy = new RMPolicy(2, 60000, false);
    }

    ProxyProperties proxy = null;

    /**
     * Sets a proxy, through which the service call will be made. Useful for SOAP tracing.
     */
    public void setProxy(String proxyHost, int proxyPort) {
        proxy = new ProxyProperties();
        proxy.setProxyName(proxyHost);
        proxy.setProxyPort(proxyPort);
    }

    /**
     * @deprecated use sendRaspRequest(...) instead, to get SignatureValidationProof
     */
    public OMElement sendRaspRequest(OMElement doc, String endpointAddress, String action, X509Certificate cert,
            Hashtable customHeaders, Hashtable properties) throws Exception {
    	
    	return sendRaspRequest(doc, endpointAddress, action, cert, customHeaders, properties, new SignatureValidationProof[1]);
    }
    	
   
    /**
     * @param doc The XML document to be sent
     * @param endpointAddress The receiver's address
     * @param action The SOAP action (operation) to be used
     * @param cert The receivers public X509 certificate
     * @param customHeaders A list of custom SOAP headers to use
     * @return The XML reply
     * @throws Exception
     */
    public OMElement sendRaspRequest(OMElement doc, String endpointAddress, String action, X509Certificate cert,
                Hashtable customHeaders, Hashtable properties, SignatureValidationProof[] signatureValidationProof) throws Exception {

        String toEPR = endpointAddress;
        log.debug("Invoking against endpoint:" + toEPR);

        // Check whether this is a mail or http sending
        boolean mailtransport = toEPR.startsWith("mail");

        ServiceClient serviceClient = new ServiceClient(configContext, null);

        log.debug("Engaging modules..:");
        serviceClient.engageModule(org.apache.axis2.Constants.MODULE_ADDRESSING);
        serviceClient.engageModule("rasp");

        Options clientOptions = new Options();

        // Should we send through a proxy?
        if (proxy != null) {
            clientOptions.setProperty(HTTPConstants.PROXY, proxy);
        }

        // Do Basic Setup
        clientOptions.setSoapVersionURI(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI);
        clientOptions.setProperty(AddressingConstants.WS_ADDRESSING_VERSION, AddressingConstants.Final.WSA_NAMESPACE);

        // sync transfer
        clientOptions.setUseSeparateListener(false);

        // anonymous sender
        String anonymous_reply_to_address = AddressingConstants.Final.WSA_ANONYMOUS_URL;
        clientOptions.setReplyTo(new EndpointReference(anonymous_reply_to_address));

        // set action
        clientOptions.setAction(action);

        // SET ENDPOINT
        clientOptions.setTo(new EndpointReference(toEPR));

        // SETUP RM
        String offeredSequenceId = "N/A";
        MercuryClient mc = null;

        if (useRM) {
            log.debug("Engaging Mercury .. : " + useRM);
            serviceClient.engageModule("Mercury");
            offeredSequenceId = RMUtil.getUUID();
            clientOptions.setProperty(MercuryClientConstants.INTERNAL_KEY, offeredSequenceId);

            RMPolicy rmPolicy;
            // transport specific setup..
            if (mailtransport) {
                rmPolicy = mailRMPolicy;
                clientOptions.setTransportInProtocol(org.apache.axis2.Constants.TRANSPORT_MAIL);
                clientOptions.setTimeOutInMilliSeconds(1000 * 60 * 5);
            } else {
                rmPolicy = httpRMPolicy;
            }
            mc = setupRM(serviceClient, rmPolicy);

        } else {
            serviceClient.disengageModule("Mercury");
        }

        // Setup Security
        if (useWSS) {
            // TODO : Find out how this works after Axis2 1.2
            log.debug("Engaging rampart module..:");
            serviceClient.engageModule("rampart");
            setupWSS(cert, serviceClient, clientOptions);
        } else {
            serviceClient.disengageModule("rampart");
        }

        setCustomHeaders(customHeaders, serviceClient, clientOptions);

        copyProperties(properties, clientOptions);

        // TODO: Check this out
        clientOptions.setExceptionToBeThrownOnSOAPFault(false);

        
        OMElement reply = null;
        DispatchCallback callback = new DispatchCallback(useRM, offeredSequenceId);

        clientOptions.setProperty(MercuryClientConstants.ERROR_CALLBACK, callback);

        // set options
        serviceClient.setOptions(clientOptions);

        long invokeTimeStart = System.currentTimeMillis();
        long invokeTimeStop = -1;
        try {
            log.debug("Creating callback and sending request...");
            serviceClient.sendReceiveNonBlocking(doc, callback);
            log.debug("- message dispatched ... ");

            while (!callback.hasCompleted() && !callback.hasTimeout()) {
                // log.error(":: " + callback.hasCompleted() + " : " + callback.hasTimeout() );
                try {
                    synchronized (callback) {
                        callback.wait();
                        log.error("Callback Notified " + callback.hasCompleted());
                    }
                    // Thread.sleep(200);
                    // System.out.print(".");
                } catch (InterruptedException e) {
                    log.error("Error receiving reply : " + e, e);
                    throw new AxisFault(e.getMessage());
                }
            }
            invokeTimeStop = System.currentTimeMillis();
            log.debug("Dispatch time : " + (invokeTimeStop - invokeTimeStart));

            if (useRM) {
                mc.terminateSequence(offeredSequenceId);
            } else {
                log.debug("NO RM ");
            }

            reply = callback.getReply();
            signatureValidationProof[0] = callback.getSignatureProof();
            

        } catch (AxisFault e) {
            log.error("An server error has occured! " + e.getMessage());
            throw e;
        } catch (Exception e) {
            log.fatal("An server error has occured! " + e.getMessage());
            throw e;
        } finally {

            log.debug("Cleanup ServiceClient !!");
            serviceClient.cleanup();
        }

        // TODO: check this...
        if (callback.hasError()) {
            log.warn("An error has occured in the request : " + callback.getError().getMessage());
            throw callback.getError();
        }
        if (callback.hasTimeout()) {
            throw new Exception("RM Operation timed out ");
        }

        return reply;
    }

	private void copyProperties(Hashtable properties, Options clientOptions) {
		// Move Properites...
        if (properties != null && properties.size() > 0) {
            Enumeration enumeration = properties.keys();
            while (enumeration.hasMoreElements()) {
                String key = enumeration.nextElement().toString();
                Object object = properties.get(key);
                clientOptions.setProperty(key, object);
            }
        }
	}

	private void setCustomHeaders(Hashtable customHeaders,
			ServiceClient serviceClient, Options clientOptions) {
		// Custom headers
        serviceClient.removeHeaders();
        if (customHeaders != null) {
            log.debug("set Custom Headers");
            SOAPFactory fac = OMAbstractFactory.getSOAP12Factory();

            OMNamespace ns = fac.createOMNamespace(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_NS,
                    dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_PREFIX);

            List headerList = new ArrayList();
            Iterator keys = customHeaders.keySet().iterator();
            while (keys.hasNext()) {
                String key = keys.next().toString();
                Object o = customHeaders.get(key);

                SOAPHeaderBlock shb = null;
                if (o instanceof SOAPHeaderBlock) {
                    shb = (SOAPHeaderBlock) o;
                } else {
                    shb = fac.createSOAPHeaderBlock(key, ns);
                    if (o instanceof IIdentifier) {
                        IIdentifier identifier = (IIdentifier) o;

                        shb.setText(identifier.getAsString());
                        shb.addAttribute(dk.gov.oiosi.Constants.PARTY_IDENTIFIER_TYPE, identifier.getType(), ns);
                    } else {
                        shb.setText(o.toString());
                    }

                }
                if (dk.gov.oiosi.Constants.RASP_MESSAGE_IDENTIFIER.equals(key)) {
                    clientOptions.setProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER, shb);
                    // DONOT add to headerlist as we only want MessageIdentifer on Payload
                    
                    //NOTE: no mustUnderstand for 1.0.1 - .Net cannot support.
                    //shb.setMustUnderstand(true);
                } else {
                    if (dk.gov.oiosi.Constants.RASP_SENDER_PARTY_IDENTIFIER.equals(key)
                            || dk.gov.oiosi.Constants.RASP_RECEIVER_PARTY_IDENTIFIER.equals(key)) {
                        //NOTE: no mustUnderstand for 1.0.1 - .Net cannot support.
                        //shb.setMustUnderstand(true);
                    }
                    // For now 'RASP_CUSTOM_HEADER_MANDATORY' is only used serverside - add to generel list...
                    headerList.add(shb);
                }
            }
            // Add property with headers for RM Messages
            clientOptions.setProperty(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_LIST, headerList);

        } else {
            log.debug("No custom header");
        }
	}

	private void setupWSS(X509Certificate cert, ServiceClient serviceClient,
			Options clientOptions) {
		// set policies
		AxisService axisService = serviceClient.getServiceContext().getAxisService();
		axisService.getPolicySubject().attachPolicy(wssPolicy);
      
		// set certificate to be used for encryption
		Vector results = new Vector();
		WSSecurityEngineResult wsser = new WSSecurityEngineResult(WSConstants.SIGN, null, cert, null, new byte[0]);
		results.add(wsser);
		WSHandlerResult wshr = new WSHandlerResult("STRING", results);
		Vector resultObj = new Vector();
		resultObj.add(wshr);
		clientOptions.setProperty(WSHandlerConstants.RECV_RESULTS, resultObj);
		log.debug("Receiver Certificate : " + cert.getSubjectDN() + " : " + cert.getIssuerDN());
	}

	private MercuryClient setupRM(ServiceClient serviceClient, RMPolicy rmPolicy)
			throws AxisFault {
		MercuryClient mc;
		/*
		 * TODO: Decide which params to use
		 * 
		 * Mercury Parameters <!-- RMS sequence parameters all are in milliseconds--> <parameter name="rmsSequenceTimeout"
		 * locked="false">600000</parameter> <parameter name="rmsSequenceRetransmitTime" locked="false">15000</parameter>
		 * <parameter name="rmsSequenceWorkerSleepTime" locked="false">5000</parameter>
		 * 
		 * <!-- RMD sequence parameters all are in milliseconds--> <parameter name="rmdSequenceTimeout" locked="false">600000</parameter>
		 * <parameter name="rmdSequenceRetransmitTime" locked="false">15000</parameter> <parameter
		 * name="rmdSequenceWorkerSleepTime" locked="false">5000</parameter>
		 * 
		 * <!-- Invoker parameters all are in milliseconds--> <parameter name="invokerTimeout" locked="false">600000</parameter>
		 * <parameter name="invokerWorkerSleepTime" locked="false">4000</parameter>
		 * 
		 * 
		 * <parameter name="sequenceRemovalWorkerSleepTime" locked="false">30000</parameter>
		 */

		mc = new MercuryClient(serviceClient);
		mc.setRMSSequenceRetransmitTime(rmPolicy.getRetransmissionInterval());
		mc.setRMSMaximumRetransmitCount(rmPolicy.getMaximumRetransmissionCount());
		
		// TODO: This is no longer available in Mercury v 0.91... should we configure the use of exponential backoff in another way?
		//mc.setRMSExponentialBackoff(rmPolicy.isExponentialBackoff());

		long timeoutValue = 0;
		if (rmPolicy.isExponentialBackoff()) {
		    timeoutValue = rmPolicy.getRetransmissionInterval() << (rmPolicy.getMaximumRetransmissionCount() + 1);

		} else {
		    timeoutValue = rmPolicy.getRetransmissionInterval()
		            * (rmPolicy.getMaximumRetransmissionCount() + 2);
		}

		// TODO: Find out about sleeper values
		mc.setRMSSequenceTimeout(timeoutValue);
		mc.setRMDSequenceTimeout(timeoutValue);
		mc.setInvokerTimeout(timeoutValue);

		log.debug("RM retransmission interval : " + rmPolicy.getRetransmissionInterval() + " - count : "
		        + rmPolicy.getMaximumRetransmissionCount() + " - exponentialBackup : " + rmPolicy.isExponentialBackoff() + " - session timeout : " + timeoutValue);
		log.error("RM retransmission interval : " + rmPolicy.getRetransmissionInterval() + " - count : "
		        + rmPolicy.getMaximumRetransmissionCount() + " - exponentialBackup : " + rmPolicy.isExponentialBackoff() + " - session timeout : " + timeoutValue);
		return mc;
	}

    static int callbackCount = 0;

    
    private class DispatchCallback implements AxisCallback, MercuryErrorCallback {

        private Log log = LogFactory.getLog(DispatchCallback.class);
        private boolean isCompleted = false;
        private OMElement reply;
        private SignatureValidationProof signatureProof;
        private Exception error;
        private boolean timeout = false;
        private boolean useRM = false;
        private String offeredSequence = null;
        public boolean faultTimeout = false;

        public DispatchCallback(boolean useRM, String offer) {
            this.useRM = useRM;
            if (useRM) {
                offeredSequence = offer;
            } else {
                offeredSequence = "NonRM-Request-" + (++callbackCount);
            }
        }

        
        /**
         * The SOAP reply message
         */
        public OMElement getReply() {
            return reply;
        }

        /**
         * A signature validation proof for the delivery
         */
        public SignatureValidationProof getSignatureProof() {
            return signatureProof;
        }
        
        /**
         * Has the call timed out?
         */
        public boolean hasTimeout() {
            return timeout;
        }

        /**
         * Has the call completed?
         */
        public boolean hasCompleted() {
            return isCompleted;
        }

        public boolean hasFaultTimeout() {
            return faultTimeout;
        }

        public boolean hasError() {
            return error != null;
        }

        public Exception getError() {
            return error;
        }

        // AXIS METHODS
        public void onComplete() {
            log.debug("Callback.onComplete() : " + offeredSequence);
            isCompleted = true;
            synchronized (this) {
                this.notify();
            }
        }

        public void onMessage(MessageContext ctx) {
            log.debug("Callback.onMessage() : " + offeredSequence);
            SOAPBody body = ctx.getEnvelope().getBody();
            reply = body.getFirstElement();

            if (reply != null) {
                log.debug("Reply has been received:" + reply.toString());
            } else {
                log.debug("Reply has been received Empty Body : null ");
            }

            if (error != null) {
                // cancel errror
                log.warn("Reply has been received - previous error is reset : " + error);
                error = null;
            }
            if (!useRM) {
                log.debug("NO RM - set complete...");
                isCompleted = true;
            } else {
                isCompleted = true;
            }

            signatureProof = extractSignatureProof(ctx);
            
            log.error("Notifiy Waiting MainThread");
            synchronized (this) {
                this.notify();
            }

        }

		private SignatureValidationProof  extractSignatureProof(MessageContext ctx) {
			return (SignatureValidationProof) ctx.getProperty(Constants.RASP_SIGNATURE_PROOF); 
		}

        public void onError(Exception e) {
            log.debug("Callback.onError(Exception) : " + offeredSequence);

            if (e instanceof AxisFault) {
                AxisFault a = (AxisFault) e;
                log.error("Axis2 returned a fault!");
                log.error("Reason - " + a.getReason());
                log.error(" - faultcode : " + a.getFaultCode());

                if (a.getFaultCode() != null) {
                    log.error(" - faultcode : " + a.getFaultCode().getLocalPart());
                    log.error(" - fc element: " + a.getFaultCodeElement());
                    // log.error(" - fc detail : " + a.getFaultDetailElement() );
                    log.error(" - fc node ele:" + a.getFaultNodeElement());

                    isCompleted = true;

                    // boolean

                    MainException me = null;
                    if (a.getFaultCodeElement() != null && a.getFaultCodeElement().getSubCode() != null) {
                        log.error("WE HAVE SUBCODE !!! : " + a.getFaultCodeElement().getSubCode());
                        log.error("WE HAVE SUBCODE !!! : " + a.getFaultCodeElement().getSubCode().getValue());
                        log.error("WE HAVE SUBCODE !!! : "
                                + a.getFaultCodeElement().getSubCode().getValue().getTextAsQName().getLocalPart());

                        /* TODO: Have we gotten all SOAP fault properties already? In that case this code should be deleted
                         * String subCode = a.getFaultCodeElement().getSubCode().getValue().getTextAsQName().getLocalPart(); if(
                         * SchematronValidationFailedException.FAULT_ACTION.equals( subCode) ) { me = new
                         * SchematronValidationFailedException(a.getReason()); } else if(
                         * SchemaValidationFailedException.FAULT_ACTION.equals( subCode) ) { me = new
                         * SchemaValidationFailedException(a.getReason()); } else if(
                         * NoDocumentTypeFoundFromXmlDocumentException.FAULT_ACTION.equals( subCode) ) { me = new
                         * NoDocumentTypeFoundFromXmlDocumentException(a.getReason()); } else { // Fallback to MainException me =
                         * new MainException(a.getReason()); me.setSoapFaultAction( subCode ); }
                         */
                        String subCode = a.getFaultCodeElement().getSubCode().getValue().getTextAsQName().getLocalPart();
                        me = new FaultReturnedException(a.getReason(), e);
                        me.setSoapFaultAction(subCode);

                    } else {
                        MessageContext ctx = a.getFaultMessageContext();
                        if (ctx != null) {
                            log.warn("SoapFault has no Subcode : " + a + " : " + ctx.getEnvelope(), a);
                        } else {
                            log.warn("SoapFault has no Subcode + MessageContext not present : " + a, a);
                        }
                        me = new MainException(a.getReason());
                    }
                    if ("Receiver".equals(a.getFaultCode().getLocalPart())) {
                        me.setReceiverError(true);
                    } else {
                        me.setReceiverError(false);
                    }

                    error = me;
                    synchronized (this) {
                        this.notify();
                    }
                    return;
                } else {
                    System.out.println("AxisFault has no FaultCode : " + e);
                }
            } else {
                System.out.println("This NOT AxisFault! : " + e.getClass().getCanonicalName());
                e.printStackTrace();
            }

            if (e.getMessage() != null) {
                if (e.getMessage().indexOf("!continueSending") != -1) {
                    // we allready have error ...
                    log.error("STOP Listining for RM " + e.getMessage());
                    faultTimeout = true;
                    return;
                } else if ("Sandesha2 sender thread has not received a valid CreateSequnceResponse".equals(e.getMessage())) {
                    log.warn("CreateSeq  problem ?? " + e);
                    if (reply != null) {
                        // we allready have response ??
                        log.warn("Create Seq problem reported - buy reply already received ");
                    } else {
                        faultTimeout = true;
                    }
                } else {
                    log.warn("Yet unknown : CreateSeq ?? " + e);
                    faultTimeout = true;
                    isCompleted = true;
                }
            }
            if (!useRM) {
                isCompleted = true;
            }
            log.error("Callback - Exception reported :" + e.getMessage() + " : " + timeout + " : " + isCompleted + " : "
                    + faultTimeout);
            error = e;

            synchronized (this) {
                this.notify();
            }
        }

        // TODO: CHECK THIS OUT!!! THIS NEVER GETS CALLED
        /*
         * (non-Javadoc)
         * 
         * @see org.apache.axis2.client.async.AxisCallback#onFault(org.apache.axis2.context.MessageContext)
         */
        public void onFault(MessageContext ctx) {
            log.debug("Callback.onFault(MessageContext) " + ctx.getSoapAction() + " : " + offeredSequence);
            isCompleted = true;
            try {
                SOAPFault fault = ctx.getEnvelope().getBody().getFault();
                log.debug("Received Fault : Action " + ctx.getSoapAction() + " : " + fault.getCode() + " : " + fault);
                QName faultCode = fault.getCode().getTextAsQName();
                log.debug(" fault code " + faultCode + " : " + faultCode.getLocalPart() + " : " + faultCode.getNamespaceURI()
                        + " : " + faultCode.getPrefix());

                if (fault.getCode().getSubCode() != null) {
                    SOAPFaultSubCode subCode = fault.getCode().getSubCode();

                    log.debug(" sub code : " + subCode + " : " + subCode.getLocalName());
                    // subCode.getLocalName()
                    // QName subCode = fault.getCode().getSubCode().getTextAsQName();
                    SOAPFaultValue tmp = subCode.getValue();
                    if (tmp != null) {
                        QName subCodeValue = tmp.getTextAsQName();
                        log.debug(" sub code " + subCodeValue.getLocalPart());
                        log.debug(" sub code " + subCodeValue.getNamespaceURI());
                        log.debug(" sub code " + subCodeValue.getPrefix());
                    } else {
                        log.debug(" no subcode - value " + tmp);

                    }

                    // log.debug("Received Fault : " + subCode.getLocalPart() + " : " + subCode.getNamespaceURI() );
                } else {
                    log.debug("Received Fault : " + fault.getCode().getText() + " - no subcode");
                }

                // TODO Construct 'Rasp' Fault based on incoming soap fault
                // for now these cannot be transfered correctly...

                error = new AxisFault(fault);
                if (!useRM) {
                    isCompleted = true;
                } else {
                    faultTimeout = true;
                }

                synchronized (this) {
                    this.notify();
                }

                return;

            } catch (Exception e) {
                log.error("Error constructing fault : " + e, e);
            }
            //
            error = new Exception(ctx.getFailureReason());
            if (!useRM) {
                isCompleted = true;
            }
        }

        public void onError(RMException e) {
            log.error("\n\n### MERCURY RM EXCEPTION : FIND OUT HOW TO HANDLE" + e, e);
            isCompleted = true;
        }
    }

    /**
     * Helper for reading policy from file...
     * 
     * @param xmlPath
     * @return
     * @throws Exception
     */
    private static Policy loadPolicy(String xmlPath) throws Exception {
        StAXOMBuilder builder = new StAXOMBuilder(xmlPath);
        Policy policy = PolicyEngine.getPolicy(builder.getDocumentElement());
        return policy;
    }

}
