/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication.fault;

import java.util.ArrayList;
import java.util.Arrays;

import dk.gov.oiosi.common.RaspLibraryException;

/**
 * Represents a rasp message fault.
 */
public class OiosiMessageFault {

    /**
     * FaultCode.
     */
    private String code;

    /**
     * FaultReaseon.
     */
    private String reason;

    private void setFaultCode(String value) {
        if("Sender".equals(value) || "Reciever".equals(value)) {
            // OK
            code = value;
        } else {
            throw new RaspLibraryException("FaultCode can only be 'Sender' or 'Receiver'");
        }   
    }

    private String[] defaultRaspFault = new String[] {
            // Sender
            "SchemaValidationFault" 
            ,"SchematronValidationFault"
            ,"SignatureNotValidFault"
            ,"UnknownDocumentTypeFault"

            // Receiver
            ,"MessagePersistencyFault"
            ,"XsltTransformationFault"
            ,"InternalSystemFailureFault"
            ,"MissingHeaderFault"
            
            ,"DispatchingNotPossibleFault"
            ,"DispatchingDestinationUnknownFault"
            ,"DispatchingFailedFault"
            ,"DispatchingDeniedFault"
            ,"ValidationFault"

    };
    ArrayList faultList = new ArrayList(Arrays.asList(defaultRaspFault));

    /**
     * Constructor
     * @deprecated Unused constructor
     * 
     * @param e
     *            exception
     * @param code
     *            fault code
     * @param subcode
     *            inner fault code
     * @throws OiosiMessageFaultUnexpectedFaultCode
     */
    public OiosiMessageFault(Exception e, String code, String subcode) { }


    /**
     * Gets whether the fault has details.
     * 
     * @return returns whether the fault has details
     */
    public boolean hasDetail() {
        return false;
    }

}
