/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.oces;

import java.security.cert.X509Certificate;

import dk.gov.oiosi.exception.MainException;

/**
 * TODO: Check method for checking certificate. TODO: Why handle employee cert
 * different. Represents an employee oces x509 certificate.
 */
public class EmployeeOcesX509Certificate extends OcesX509Certificate {

    /**
     * cvr number for company.
     */
    private String cvrNumber;

    /**
     * Constructor that takes an x509 certificate as parameter. If the
     * certificate is not an employee certificate an exception is thrown.
     * 
     * @param certificate
     *            certificate
     * @throws MainException
     */
    public EmployeeOcesX509Certificate(X509Certificate certificate) throws MainException {
        super(certificate);

        if (getOcesCertificateType().getType() != OcesCertificateType.OcesEmployee)
            throw new InvalidOcesEmployeeCertificateException(certificate);
        setCvrNumber();
    }

    /**
     * Constructor that takes an ocescertificate as parameter. If the
     * certificate is not an employee certificate an exception is thrown.
     * 
     * @param certifcate
     *            certificate
     * @throws MainException
     */
    public EmployeeOcesX509Certificate(OcesX509Certificate certifcate) throws MainException {
        this(certifcate.getCertificate());

        //TODO: Why is no code running?
        // if (getOcesCertificateType().getType() !=
        // OcesCertificateType.OcesEmployee)
        // throw new
        // InvalidOcesEmployeeCertificateException(certifcate.getCertificate());
        //
        // setCvrNumber();
    }

    /**
     * Gets the cvr number of the employee certificate.
     * 
     * @return CVR number
     */
    public String getCvrNumber() {
        return cvrNumber;
    }

    /**
     * Extract CVR number.
     * 
     * @throws MainException
     */
    private void setCvrNumber() throws MainException {
        String serialNumber = getSubject().getSerialNumberValue().toLowerCase();
        int cvrix = serialNumber.indexOf("cvr:");
        if (cvrix == -1) {
            throw new NoSubjectCvrNumberException(getCertificate());
        }
        if( serialNumber.indexOf("cvr:",cvrix+1) != -1) {
            throw new AmbigousSubjectCvrNumberException(getCertificate());
        }
        int ix = serialNumber.indexOf("-", cvrix);
        cvrNumber = serialNumber.substring(cvrix + 4, ix);

    }

}
