/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.ocsp;

import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.exception.ArgumentException;

/**
 * OCSP dynamic configuration.
 */
public class OcspConfig {

    /**
     * url.
     */
    private Uri serverUrl = new Uri("http://localhost");

    /**
     * timeout.
     */
    private long defaultTimeoutMsec = 10000;

    /**
     * Default constructor that initializes the OscpConfig with default values.
     */
    public OcspConfig() { }
    
    /**
     * Gets the URL of the OCSP server.
     * 
     * @return ServerUrl
     */
    public String getServerUrl() {
        return serverUrl.toString();
    }

    /**
     * Gets the URL of the OCSP server.
     * 
     * @param value
     *            ServerUrl
     */
    public void setServerUrl(String value) {
        serverUrl = new Uri(value);
    }

    /**
     * Sets the URL of the OCSP server.
     * 
     * @param value
     *            ServerUrl
     */
    public void setServerUrl(Uri value) {
        serverUrl = value;
    }

    /**
     * The default timeout in milliseconds for the OCSP lookup operation.
     * 
     * @return DefaultTimeoutMsec
     */
    public long getDefaultTimeoutMsec() {
        return defaultTimeoutMsec;
    }

    /**
     * The default timeout in milliseconds for the OCSP lookup operation.
     * 
     * @param value
     *            DefaultTimeoutMsec
     */
    public void setDefaultTimeoutMsec(long value) {
        if (defaultTimeoutMsec < 0) {
            throw new ArgumentException("Timeout value < 0 must be greater than -1.");
        }
        defaultTimeoutMsec = value;
    }
}
