/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.ocsp;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;

/**
 * Creates an instance of a class with the IOcspLookup interface.
 */
public class OcspLookupFactory {

    private static Log log = LogFactory.getLog(OcspLookupFactory.class);
    
    private static IOcspLookup lookupClient;

    /**
     * Builds an instance of an IOcspLookup client based on configuration.
     * 
     * @return ocsp lookup
     */
    public IOcspLookup createOcspLookupClient() throws Exception {

        synchronized (OcspLookupFactory.class) {
            if (lookupClient == null) {
                OcspLookupFactoryConfig config = RaspConfigurationHandler.getInstance().getOcspLookupFactoryConfig();

                try {
                    log.debug("Init OcspLookupFactory : " + config.getImplementationNamespaceClass());
                    lookupClient = (IOcspLookup) Class.forName(config.getImplementationNamespaceClass()).newInstance();
                } catch (InstantiationException e) {
                    throw new RaspLibraryException(e);
                } catch (IllegalAccessException e) {
                    throw new RaspLibraryException(e);
                } catch (ClassNotFoundException e) {
                    throw new RaspLibraryException(e);
                }
            }
            return lookupClient;
        }

    }
}
