/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.ocsp;

import java.util.Date;

/**
 * This class is used to store the result of a Ocsp certification validation
 * process.
 */
public class OcspResponse {

    /**
     * This property is used to indicate if the certificate has been revoked
     * (either permanently or temporarily (on hold)).
     */
    private boolean isValid;

    /**
     * This property is used to store the time at or before which newer
     * information will be available about the status of the certificate.
     */
    private Date nextUpdate = new Date();

    /**
     * Gets the next update of the OCSP check of the certificate.
     * 
     * @return NextUpdate
     */
    public Date getNextUpdate() {
        return nextUpdate;
    }

    /**
     * Sets the next update of the OCSP check of the certificate.
     * 
     * @param value
     *            NextUpdate
     */
    public void setNextUpdate(Date value) {
        nextUpdate = value;
    }

    /**
     * Gets whether the certificate is valid.
     * 
     * @return is valid
     */
    public boolean isValid() {
        return isValid;
    }

    /**
     * Sets whether the certificate is valid.
     * 
     * @param value
     *            IsValid
     */
    public void setIsValid(boolean value) {
        isValid = value;
    }
}
