/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

/**
 * Options for the lookup.
 */
public class LookupReturnOption {

    private int value = -1;

    public LookupReturnOption(String v) {
        if("noMoreThanOneSetOrFail".equalsIgnoreCase(v)) {
            value = noMoreThanOneSetOrFail;
        } else if("firstResult".equalsIgnoreCase(v)) {
            value = firstResult;
        } else if("allResults".equalsIgnoreCase(v)) {
            value = allResults;
        }
    }

    public LookupReturnOption(int v) {
        value = v;
    }

    public int getCode() {
        return value;
    }

    /**
     * Throws an exceptions if not either one or zero results were returned.
     */
    public static final int noMoreThanOneSetOrFail = 0;

    /**
     * If multiple results were returned from the UDDI inquiry, return only the
     * first
     */
    public static final int firstResult = 1;

    /**
     * Return all results from the UDDI inquiry
     */
    public static final int allResults = 2;

    public String getValue() {
        switch (value) {
        case noMoreThanOneSetOrFail:
            return "noMoreThanOneSetOrFail";
        case firstResult:
            return "firstResult";
        case allResults:
            return "allResults";
        }
        return null;
    }
    
}
