/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.uddi.ranges.GatewayRange;

/**
 * UDDI configuration for the user.settings file.
 */
public class UddiConfig {

    private String profileConformanceClaim = "";
    private String registrationConformanceClaim = "";
    private String uddiInquireEndpointURL = "";
    private String uddiInquireEndpointURLFallback = "";
    private String uddiPublishEndpointURL = "";
    private String uddiSecurityEndpointURL = "";
    private int fallbackTimeoutMinutes = 0;
    private LookupReturnOption lookupReturnOptions = new LookupReturnOption(LookupReturnOption.allResults);
    private boolean tryOtherHostsOnFailure = true;
    private GatewayRange gatewayRange = null;

    // TODO: ??? IS THIS STILL POSSIBLE ???
    private boolean enableEndpointCaching = false;

    /**
     * Default constructor
     */
    public UddiConfig() {
    }

    /**
     * Gets or sets the default service profile claim used when performing a
     * lookup
     * @return ProfileConformanceClaim
     */
    public String getProfileConformanceClaim() {
        return profileConformanceClaim;
    }

    /**
     * Gets or sets the default service profile claim used when performing a
     * lookup
     * @param value ProfileConformanceClaim
     */
    public void setProfileConformanceClaim(String value) {
        profileConformanceClaim = value;
    }

    /**
     * Gets or sets the default UDDI profile claim used when performing a lookup
     * @return RegistrationConformanceClaim
     */
    public String getRegistrationConformanceClaim() {
        return registrationConformanceClaim;
    }

    /**
     * Gets or sets the default UDDI profile claim used when performing a lookup
     * @param value RegistrationConformanceClaim
     */
    public void setRegistrationConformanceClaim(String value) {
        registrationConformanceClaim = value;
    }

    /**
     * Gets or sets the default UDDI inquiry service endpoint
     * @return UddiInquireEndpointURL
     */
    public String getUddiInquireEndpointURL() {
        return uddiInquireEndpointURL;
    }

    /**
     * Gets or sets the default UDDI inquiry service endpoint
     * @param value UddiInquireEndpointURL
     */
    public void setUddiInquireEndpointURL(String value) {
        uddiInquireEndpointURL = value;
    }

    /**
     * Gets or sets the default UDDI inquiry FALLBACK service endpoint
     * @return UddiInquireEndpointURL
     */
    public String getUddiInquireEndpointURLFallback() {
        return uddiInquireEndpointURLFallback;
    }

    /**
     * Gets or sets the default UDDI inquiry FALLBACK service endpoint
     * @param value UddiInquireEndpointURL
     */
    public void setUddiInquireEndpointURLFallback(String value) {
        uddiInquireEndpointURLFallback = value;
    }

    /**
     * Gets or sets the default UDDI publish service endpoint
     * @return UddiPublishEndpointURL
     */
    public String getUddiPublishEndpointURL() {
        return uddiPublishEndpointURL;
    }

    /**
     * Gets or sets the default UDDI publish service endpoint
     * @param value UddiPublishEndpointURL
     */
    public void setUddiPublishEndpointURL(String value) {
        uddiPublishEndpointURL = value;
    }

    /**
     * Gets or sets the default UDDI security endpoint
     * @return UddiSecurityEndpointURL
     */
    public String getUddiSecurityEndpointURL() {
        return uddiSecurityEndpointURL;
    }

    /**
     * Gets or sets the default UDDI security endpoint
     * @param value UddiSecurityEndpointURL
     */
    public void setUddiSecurityEndpointURL(String value) {
        uddiSecurityEndpointURL = value;
    }


    /**
     * Gets or sets the fallback timeout in minutes.
     * @return fallbackTimeoutMinutes
     */
    public int getFallbackTimeoutMinutes() {
        return fallbackTimeoutMinutes;
    }

    /**
     * Gets or sets the fallback timeout in minutes.
     * @param value fallbackTimeoutMinutes
     */
    public void setFallbackTimeoutMinutes(int value) {
        fallbackTimeoutMinutes = value;
    }

    
    /**
     * Gets or sets the lookup return options
     * @return LookupReturnOptions
     */
    public LookupReturnOption getLookupReturnOptions() {
        return lookupReturnOptions;
    }

    /**
     * Gets or sets the lookup return options
     * @param value LookupReturnOptions
     */
    public void setLookupReturnOptions(LookupReturnOption value) {
        lookupReturnOptions = value;
    }

    /**
     * Gets or sets the 'try other hosts on failure' flag
     * @return TryOtherHostsOnFailure
     */
    public boolean getTryOtherHostsOnFailure() {
        return tryOtherHostsOnFailure;
    }

    /**
     * Gets or sets the 'try other hosts on failure' flag
     * @param value TryOtherHostsOnFailure
     */
    public void setTryOtherHostsOnFailure(boolean value) {
        tryOtherHostsOnFailure = value;
    }


    /**
     * @deprecated
     * Gets or sets the 'enable endpoint caching' flag.
     * @return EnableEndpointCaching
     */
    public boolean getEnableEndpointCaching() {
        return enableEndpointCaching;
    }

    /**
     * @deprecated
     * Gets or sets the 'enable endpoint caching' flag.
     * @param value EnableEndpointCaching
     */
    public void setEnableEndpointCaching(boolean value) {
        enableEndpointCaching = value;
    }

    /**
     * Gets or sets the gateway range configuration
     * @return GatewayRange
     */
    public GatewayRange getGatewayRange() {
        return gatewayRange;
    }

    /**
     * Gets or sets the gateway range configuration
     * @param value GatewayRange
     */
    public void setGatewayRange(GatewayRange value) {
        gatewayRange = value;
    }

}
