/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import org.apache.axiom.om.util.UUIDGenerator;

import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.exception.NullOrEmptyArgumentException;

/**
 * Represents a UDDI type guid.
 */
public class UddiGuidId extends UddiId {

    /**
     * Original guid : private Guid _guid; Guid is Microsoft UUID - represent as
     * string in java/axis
     */
    private String _guid;

    /**
     * Creates a new GUID
     */
    public UddiGuidId() {
        _guid = UUIDGenerator.getUUID();
    }

    /**
     * Tests a guid to see if it is a valid uddi guid
     * 
     * @param guid
     *            guid to test
     * @param isUddiType
     *            indicates if it is a uddi type guid
     * @return Returns true if the guid is valid
     */
    public static boolean isValidGuidId(String guid, boolean isUddiType) {
        if (guid == null || guid.length() == 0)
            return false;

        String guidString = "";

        if (isUddiType) {
            guidString = guid.substring(5);
        } else {
            guidString = guid;
        }

        try {
        	//TODO: Isn't there a smarter test for valid GUIDs?!
            new Guid(guidString);
        } catch (Exception e) {
            return false;
        }

        return true;

    }

    /**
     * Constructor
     * 
     * @param guid
     *            guid
     * @param isUddiType
     *            is it uddi type
     */
    public UddiGuidId(String guid, boolean isUddiType) {
        if (guid == null || guid.length() == 0)
            throw new NullOrEmptyArgumentException("guid");

        String guidString = "";
        if (isUddiType) {
            guidString = guid.substring(5);
        } else {
            guidString = guid;
        }

        _guid = guidString;
    }

    /**
     * Constructor
     * 
     * @param guid
     *            guid
     */
    public UddiGuidId(String guid) throws UddiEmptyGuidException {
        if (guid == null || guid.length() == 0) {
            throw new UddiEmptyGuidException();
        }
        _guid = guid;
    }

    /**
     * Constructor
     * 
     * @param Id
     */
    public UddiGuidId(UddiId Id) throws UddiException {
        this(Id.getID(), true);
    }

    /**
     * Gets the guid
     * 
     */
    public String getID() {
        return "uddi:" + _guid; // .ToString();
    }

    /**
     * Gets as string
     * 
     * @return string
     */
    public String toString() {
        return getID();
    }

}
