/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import dk.gov.oiosi.exception.NullOrEmptyArgumentException;

/**
 * Represents a non-guid UDDI ID (i.e. uddi v.3 ID's apart from ID's of the
 * format 'uddi:d01987d1-ab2e-3013-9be2-2a66eb99d824').
 * 
 */
public class UddiNonGuidId extends UddiId {

    /**
     * The UDDI 3.0 format key
     * 
     */
    protected String pId;

    /**
     * Constructor
     * 
     * @param id
     *            guid
     */
    public UddiNonGuidId(String id) {
        // TODO: could add more UDDI-type validation here.
        if (id == null || id.length() == 0)
            throw new NullOrEmptyArgumentException("id");
        pId = id;
    }

    /**
     * Gets the guid
     * 
     */
    public String getID() {
        return pId;
    }

    /**
     * Gets guid as string
     * 
     * @return
     */
    public String oString() {
        return pId;
    }
}
