/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import java.util.ArrayList;
import java.util.List;

import dk.itst.uddi.client.types.core.BindingTemplate;
import dk.itst.uddi.client.types.core.BindingTemplates;

/**
 * This class holds all binding templates of a service
 * 
 */
public class ArsBindingSet {

    // private List<ArsBinding> _bindings = new List<ArsBinding>();
    private List _bindings = new ArrayList();

    /**
     * Gets the list of ArsBindings
     * 
     */
    public List getBindings() {
        return _bindings;
    }

    /**
     * Default constructor
     * 
     */
    public ArsBindingSet() {
    }

    /**
     * Initializes with an existing ArsBindingSet
     * 
     * @param bindings
     *            an existing ArsBindingSet
     */
    public ArsBindingSet(List bindings) {
        _bindings = bindings;
    }

    /**
     * Constructor
     * 
     * @param templates
     *            An array of binding templates from which the object will be
     *            constructed
     * @throws ArsBindingSetUnexpectedException
     */
    public ArsBindingSet(BindingTemplate[] templates) throws ArsBindingSetUnexpectedException {

        try {
            for (int i = 0; i < templates.length; i++) {
                _bindings.add(new ArsBinding(templates[i])); // new
                                                                // BindingTemplate(templates[i])));
            }
        } catch (Exception exp) {
            throw new ArsBindingSetUnexpectedException(exp);
        }
    }

    /**
     * Adds a binding to the bindingset
     * 
     * @param binding
     *            the ArsBinding to add
     */
    public void add(ArsBinding binding) {

        // 1. add it to the list
        _bindings.add(binding);
    }

    /**
     * Removes a binding from the bindingset
     * 
     * @param binding
     *            the binding to remove
     */
    public void remove(ArsBinding binding) {

        // 1. remove a ArsBinding from the list
        _bindings.remove(binding);
    }

    /**
     * Gets all bindingtemplates
     * 
     * @return a collection with all bindingtemplates
     * @throws ArsBindingSetUnexpectedException
     */
    public BindingTemplates getBindingTemplates() throws ArsBindingSetUnexpectedException {

        // 1. set temp bindingTemplate array
        BindingTemplates tempBindings = null;
        try {
            tempBindings = BindingTemplates.Factory.newInstance();

            tempBindings.setBindingTemplateArray((BindingTemplate[]) _bindings.toArray());
        } catch (Exception exp) {
            throw new ArsBindingSetUnexpectedException(exp);
        }
        return tempBindings;
    }

    /**
     * Validates the embedded data, and eventually returns a structured report
     * if validations fails
     * 
     * @param EntityName
     * @param Failures
     * @return
     */
    public boolean isValid(String entityName) { // , ref
                                                // ValidationFailureCollection
                                                // Failures) {
    // ValidationFailureCollection ChildFailures = null;
    //
    // foreach (ArsBinding Binding in _bindings)
    // Binding.IsValid("Binding", ref ChildFailures);
    //
    // if (ChildFailures != null)
    // ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName,
    // this.GetType(),
    // ChildFailures, ref Failures);
    //
    // return Failures == null;
        for (int i = 0; i < _bindings.size(); i++) {
            ArsBinding tmp = (ArsBinding) _bindings.get(i);
            if (!tmp.isValid("Binding")) {
                return false;
            }
        }
        return true;
    }

}
