/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.OIOTaxonomy;
import dk.gov.oiosi.uddi.category.OrganizationKeyType;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaimCode;
import dk.gov.oiosi.uddi.identifier.OrganizationKey;
import dk.itst.uddi.client.types.core.BusinessEntity;
import dk.itst.uddi.client.types.core.Description;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.Name;

/** 
 * Represents an ARS UDDI business entity registration. This class only holds the 
 * business entity registration, not the services included in this.
 * 
 */
public class ArsBusinessEntity extends RegistrationEntity implements IRegistrationEntity {

    private BusinessEntity _businessEntity = BusinessEntity.Factory.newInstance();
    
    private void init() {
    }

    /** 
     * Default constructor. Creates an empty business entity with a random id.
     * 
     */
    public ArsBusinessEntity() {
        // Create new instance with new ID
        _businessEntity = BusinessEntity.Factory.newInstance();
        //_businessEntity.CategoryBag = new CategoryBag();
        setDefaultProperties();
        init();
    }

    /** 
     * Constructor. Constructs an ArsBusinessEntity from a BusinessEntity object.
     * 
     * @param  businessEntity The BusinessEntity object
     */ 
    public ArsBusinessEntity(BusinessEntity businessEntity) {
        init();
        _businessEntity = businessEntity;
    }

    /** 
     * Sets default properties
     * 
     */
    private void setDefaultProperties() {
        setRegistrationConformanceClaim 
        ( new RegistrationConformanceClaim(new RegistrationConformanceClaimCode(RegistrationConformanceClaimCode.oiosi1_1)) );
    }


    /** 
     * Constructor, taking the values to set on the businessentity. 
     * This constructor sets a default registration conformance claim of
     * OIOSI 1.1. 
     * 
     * @param  oioTaxonomy OIO taxonomy
     * @param  organizationKey Organization identifier
     * @param  organizationKeyType Type of the organization identifier
     * @param  businessEntityContact Contact for the entity
     * @param  businessDiscoveryUrl discovery url
     * @param  businessName Name of the businessentity
     * @param  description descriptions of the entity
     */ 
    public ArsBusinessEntity(OIOTaxonomy oioTaxonomy,
            OrganizationKey organizationKey,
            OrganizationKeyType organizationKeyType,
            /*UddiContactSimple*/ Object businessEntityContact,
            /*UddiDiscoveryUrl*/ Object businessDiscoveryUrl,
            Name businessName,
            Description description
    ) { 
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - construct - TODO Implement" );
        
//        //1. set businessentity attributes, contacts and discoveryUrl
//        RegistrationConformanceClaim = new RegistrationConformanceClaim(RegistrationConformanceClaimCode.oiosi1_1);
//        
//        setBusinessEntityDetails(RegistrationConformanceClaim,
//                oioTaxonomy,
//                organizationKey,
//                organizationKeyType,
//                businessEntityContact,
//                businessDiscoveryUrl,
//                businessName,
//                description
//        );
    } 

    /** 
     * Constructor, taking the values to set on the businessentity. Some of these values are raw
     * Uddi 3.0 types
     * 
     * @param  registrationConformanceClaim Claims regarding conformance to the UDDI registration model
     * @param  oioTaxonomy OIO taxonomy
     * @param  organizationKey Organization identifier
     * @param  organizationKeyType Type of the organization identifier
     * @param  businessEntityContact Contact for the entity
     * @param  businessDiscoveryUrl discovery url
     * @param  businessName Name of the businessentity
     * @param  description descriptions of the entity
     */ 
    public ArsBusinessEntity(
            RegistrationConformanceClaim registrationConformanceClaim,
            OIOTaxonomy oioTaxonomy,
            OrganizationKey organizationKey,
            OrganizationKeyType organizationKeyType,
            /*UddiContactSimple*/Object businessEntityContact,
            /*UddiDiscoveryUrl*/Object businessDiscoveryUrl,
            Name businessName,
            Description description
    ) {
        //1. set businessentity attributes, contacts and discoveryUrl
        setBusinessEntityDetails(
                registrationConformanceClaim,
                oioTaxonomy,
                organizationKey,
                organizationKeyType,
                businessEntityContact,
                businessDiscoveryUrl,
                businessName,
                description
        );
    }

    /** 
     * Gets the registration conformance claim regarding Uddi registration model
     * 
     */
    public RegistrationConformanceClaim getRegistrationConformanceClaim() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getRegistrationConformanceClaim - TODO Implement" );
        else return null;
        
//        if (_businessEntity != null && _businessEntity.CategoryBag != null) {
//            RegistrationConformanceClaim regClaim = new RegistrationConformanceClaim();
//            
//            KeyedReference keyRef = _businessEntity.CategoryBag.GetCategoryByIdentifier(regClaim.CategoryID);
//            if (keyRef != null) {
//                regClaim.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return regClaim;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    
    /** 
     * Gets the registration conformance claim regarding Uddi registration model
     * 
     */
    public void setRegistrationConformanceClaim (RegistrationConformanceClaim value){
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setRegistrationConformanceClaim - TODO Implement" );

//        RegistrationConformanceClaim test = new RegistrationConformanceClaim();
//        if (value != null) {
//            if (_businessEntity == null) return;
//            // Check that category ID is valid:
//            
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                        
//                throw new Exception("Cannot set registration conformance claim: " +
//                        "Tried to set with wrong category ID ('" + value.CategoryID + "')");
//            }
//
//            if (_businessEntity.CategoryBag == null) {
//                _businessEntity.CategoryBag = new CategoryBag();
//            }
//
//            _businessEntity.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            _businessEntity.CategoryBag.RemoveCategoryById(test.CategoryID);
//        }
    }


    /** 
     * Gets or sets the OIO Taxonomy
     */
    public OIOTaxonomy getOIOTaxonomy() { 
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getOIOTaxonomy - TODO Implement" );
        else return null;
        
//        if (_businessEntity != null && _businessEntity.CategoryBag != null) {
//            OIOTaxonomy tax = new OIOTaxonomy();
//            KeyedReference keyRef = 
//                _businessEntity.CategoryBag.GetCategoryByIdentifier(tax.CategoryID);
//            if (keyRef == null) {
//                return null;
//            } else {
//                //tax.SetCategoryValue(tax.CategoryName/*keyRef.KeyName*/, keyRef.KeyName/*keyRef.KeyValue*/);
//                tax.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return tax;
//            }
//        } else {
//            return null;
//        }
    }
    
    /** 
     * Gets or sets the OIO Taxonomy
     */
    public void setOIOTaxonomy( OIOTaxonomy value){ 
        if (value != null) {
            setCategory(value.getAsKeyedReference());
        } else {
            setCategory(null);
        }
    } 


    /** 
     * Gets or sets the organization key
     */
    public OrganizationKey getOrganizationKey() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getOrganizationKey - TODO Implement" );
        else return null;
        
//        if (_businessEntity != null && _businessEntity.getIdentifierBag() != null) {
//            OrganizationKey orgKey = new OrganizationKey();
//            KeyedReference keyRef =
//                _businessEntity.getIdentifierBag().GetCategoryByIdentifier(orgKey.IdentifierID);
//            if (keyRef == null) {
//                return null;
//            } else {
//                orgKey.Value = keyRef.KeyValue;
//                return orgKey;
//            }
//        } else {
//            return null;
//        }
    }
    
    /** 
     * Gets or sets the organization key
     */
    public void setOrganizationKey (OrganizationKey value){
        setIdentifier(value.getAsKeyedReference());
    } 



    /** 
     * Gets or sets the organization keytype
     */
    public OrganizationKeyType getOrganizationKeyType() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getOrganizationKeyType - TODO Implement" );
        else return null;
        
//        if (_businessEntity != null && _businessEntity.CategoryBag != null) {
//            OrganizationKeyType keyType = new OrganizationKeyType();
//            KeyedReference keyRef =
//                _businessEntity.getCategoryBag.GetCategoryByIdentifier(keyType.CategoryID);
//            if (keyRef == null) {
//                return null;
//            } else {
//                keyType.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return keyType;
//            }
//        } else {
//            return null;
//        }
    }
    
    /** 
     * Gets or sets the organization keytype
     */
    public void setOrganizationKeyType (OrganizationKeyType value) {
        if (value != null) {
            setCategory(value.getAsKeyedReference());
        } else {
            setCategory(null);
        }
    } 
    


    /** 
     * Adds an identifier to the identifier bag
     * 
     * @param  uddiKeyRef The keyed reference to add to the identifier bag
     */ 
    private void setIdentifier(KeyedReference uddiKeyRef) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setIdenifier - TODO Implement" );

//        KeyedReference keyRef = new KeyedReference(uddiKeyRef);
//        if (keyRef != null) {
//            if (_businessEntity != null) {
//                if (_businessEntity.IdentifierBag == null) {
//                    _businessEntity.IdentifierBag = new CategoryBag(keyRef);
//                } else {
//                    _businessEntity.IdentifierBag.SetCategory(keyRef);
//                }
//            } else {
//                return;
//            }
//        } else {
//            if (_businessEntity != null && _businessEntity.IdentifierBag != null) {
//                _businessEntity.IdentifierBag.RemoveCategory(keyRef);
//            } else {
//                return;
//            }
//        }
    }


    /** 
     * Adds a category to the category bag
     * 
     * @param  uddiKeyRef The keyed reference
     */ 
    private void setCategory(KeyedReference uddiKeyRef) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getCategory - TODO Implement" );

//        KeyedReference keyRef = new KeyedReference(uddiKeyRef);
//        if (keyRef != null) {
//            if (_businessEntity != null) {
//                if (_businessEntity.CategoryBag == null) {
//                    _businessEntity.CategoryBag = new CategoryBag(keyRef);
//                } else {
//                    _businessEntity.CategoryBag.SetCategory(keyRef);
//                }
//            } else {
//                return;
//            }
//        } 
    }


    /** 
     * The name of the business entity
     */
    public Name getName() {
        if (_businessEntity.getNameArray() != null && _businessEntity.getNameArray().length > 0) {
            return _businessEntity.getNameArray(0);
        } else {
            return Name.Factory.newInstance();
        }
    }
    
    /** 
     * The name of the business entity
     */
    public void setName( Name value) {
        if (value != null) {
            _businessEntity.setNameArray( new Name[] { value });
        } else {
            _businessEntity.setNameArray(null);
        }
    }


    /** 
     * Description of the business entity
     */
    public Description getDescription() {

        if (_businessEntity.getDescriptionArray() != null && _businessEntity.getDescriptionArray().length > 0) {
            return _businessEntity.getDescriptionArray(0);
        } else {
            return Description.Factory.newInstance();
        }
        
    }

    /** 
     * Description of the business entity
     */
    public void setDescription (Description value) {
        if (value != null) {
            _businessEntity.setDescriptionArray(new Description[] { value } );
        } else {
            _businessEntity.setDescriptionArray(null);
        }
    }


    /** 
     * Gets or sets the business entity contact information
     */
    public /*UddiContactSimple*/Object getContact() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getContact - TODO Implement" );
        else return null;
        
//        if (_businessEntity.Provider.contacts != null
//                && _businessEntity.Provider.contacts.Length > 0) {
//            return new UddiContactSimple(_businessEntity.Provider.contacts[0]);
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the business entity contact information
     */
    public void setContact(/*UddiContactSimple*/Object value) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setContact - TODO Implement" );

//        if (value != null) {
//            _businessEntity.Provider.contacts = new contact[] { value.Value };
//        } else {
//            _businessEntity.Provider.contacts = null;
//        }
    }


    /** 
     * Gets or sets the homepage of the business entity
     */
    public /*UddiDiscoveryUrl*/Object getHomepage() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setHomepae - TODO Implement" );
        else return null;
        
//        if (_businessEntity.Provider.discoveryURLs != null
//                && _businessEntity.Provider.discoveryURLs.Length > 0) {
//            return new UddiDiscoveryUrl(_businessEntity.Provider.discoveryURLs[0]);
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the homepage of the business entity
     */
    public void getHomepage(/*UddiDiscoveryUrl*/Object value) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getHomepae - TODO Implement" );

//        if (value != null) {
//            _businessEntity.Provider.discoveryURLs = new discoveryURL[] { value.Value };
//        } else {
//            _businessEntity.Provider.discoveryURLs = null;
//        }
    }


    //public ArsEndpointSet Endpoints = new ArsEndpointSet();

    /** 
     * The Endpoints of the entity
     */
    public ArsEndpointSet getEndpoints() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getEndpoints - TODO Implement" );
        else return null;
        
//        if (_businessEntity != null && _businessEntity.Services != null && _businessEntity.Services.Length > 0) {
//            return new ArsEndpointSet(_businessEntity.Services);
//        } else {
//            return new ArsEndpointSet();
//        }
    }

    /** 
     * The Endpoints of the entity
     */
    public void setEndpoints(ArsEndpointSet value) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setEndpoints - TODO Implement" );

//        if (value != null && value.Endpoints != null && value.Endpoints.Count > 0) {
//            BusinessService[] services = new BusinessService[value.Endpoints.Count];
//            for (int i = 0; i < value.Endpoints.Count; i++) {
//                services[i] = value.Endpoints[i].Service;
//            }
//            _businessEntity.Services = services;
//        } else {
//            _businessEntity.Services = null;
//        }
    }


    /** 
     * Gets or sets the uuid of the entity
     */
    public UddiId getID() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - getId - TODO Implement" );
        else return null;
        
//        if (_businessEntity!= null && _businessEntity.Provider.businessKey != null && 
//                _businessEntity.Provider.businessKey.Length > 0)
//            return new UddiGuidId(_businessEntity.Provider.businessKey, true);
//        else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the uuid of the entity
     */
    public void setID(UddiId value) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setID - TODO Implement" );

//        if (value == null) {
//            _businessEntity.Provider.businessKey = null;
//        } else {
//            _businessEntity.Provider.businessKey = value.ID;
//        }
    }


    /** 
     * Saves the entity to the Uddi registry. Only saves this entity, not e.g. associated endpoints and processes.
     */ 
    public void save() throws MainException {
        // 1. Verify properties
        validate();
        
        // 2. Save
        try {
            saveBusinessEntity();
        } catch (MainException e){
            throw e;
        } catch (Exception e){
            throw new MainException(e);
        }
    }        

    /** 
     * Validates a entity
     */
    public void validate() {
    }

    /** 
     * Updates a entity in the UDDI registry
     * @throws MainException 
     */
    public void update() throws MainException {
        // 1. Verify properties
        validate();

        // 2. Update
        try {
            saveBusinessEntity();
        } catch (MainException e){
            throw e;
        } catch (Exception e){
            throw new MainException(e);
        }
    }

    /** 
     * Deletes a entity and all underlying binding registrations and portTypeRegistrations
     */
    public void delete() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - delete - TODO Implement" );

//        //1. check that an id exists
//        if (_businessEntity.Provider.businessKey.Length > 0) {
//
//            String[] temp = new Sstring[1];
//            temp[0] = _businessEntity.Provider.businessKey;
//            
//            //2. call Publication.Delete
//            try {
//                if (_businessEntity.Provider.businessServices != null) {
//                    foreach (BusinessService service in _businessEntity.Services) {
//                        ArsEndpoint endpoint = new ArsEndpoint(service);
//                        endpoint.DeleteRelatedBusinessProcessInstances();
//                    }   
//                }
//                    
//                Publication uddiPublicationProxy = new Publication(Connection);
//                bool deleted = uddiPublicationProxy.DeleteBusinessEntity(temp);
//                if (deleted) {
//                    _businessEntity = null;
//                }
//            } catch (DeleteBusinessException) {
//                throw;
//            } catch (PublicationUnexpectedException) {
//                throw;
//            } catch (Exception exp) {
//                throw new ArsBusinessEntityUnexpectedException(exp);
//            }
//        } else {
//            throw new Exception("No businessentity key was found");
//        }
    }

    /** 
     * Deletes the specified service, and removes the reference to it from this business entity
     * 
     * @param  endpoint The endpoint to be deleted
     */ 
    public void deleteEndpoint(ArsEndpoint endpoint) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - deleteEndpoint - TODO Implement" );

//        String[] businessServiceKey = new String[1];
//        businessServiceKey[0] = endpoint.ID.ID;
//        boolean _endpointFoundAndDeleted = false;            
//
//        //1. check that the endpoint exists on the businessentity            
//        try {
//            ArsEndpointSet finalEndpointSet = new ArsEndpointSet();
//            ArsEndpointSet currentEndpointSet = Endpoints;
//
//            for (int i = 0; i < currentEndpointSet.Endpoints.Count; i++) {
//                
//                if (currentEndpointSet.Endpoints[i].ID.ID == endpoint.ID.ID) {
//                    //DeleteBusinessServiceFromEntity(endpoint.ID.ID);
//                    currentEndpointSet.Endpoints[i].Delete();
//                    _endpointFoundAndDeleted = true;
//                } else {
//                    finalEndpointSet.Add(currentEndpointSet.Endpoints[i]);
//                }
//            }
//
//            // Set the updated list of endpoints:
//            Endpoints = finalEndpointSet;
//
//            if (!_endpointFoundAndDeleted) {
//                throw new Exception("An error occured during deletion of endpoint: " +
//                        endpoint.ID.ID + ". It was either not found or couldn't be deleted");
//            }
//        } catch (DeleteServiceException) {
//            throw;
//        } catch (PublicationUnexpectedException) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsBusinessEntityUnexpectedException(exp);
//        }
//        //return this;
    }

    /*
        private void DeleteBusinessServiceFromEntity(string serviceKey)
        {
            if (_businessEntity.Provider.businessServices != null && 
                _businessEntity.Provider.businessServices.Length > 0)
            {
                businessService[] services = new businessService[_businessEntity.Provider.businessServices.Length-1];

                for (int i = 0; i < _businessEntity.Provider.businessServices.Length; i++)
                {
                    if (_businessEntity.Provider.businessServices[i].serviceKey != serviceKey)
                    {
                        services[i] = _businessEntity.Provider.businessServices[i];
                    }
                }
                _businessEntity.Provider.businessServices = services;
            }
        }*/

    /** 
     * get a ArsBusinessEntity
     * 
     * @param  businessEntityId the uuid of the entity to retrieve
     * @param  getRecursively If getRecursively is true, all subentities are got as well, i.e.
     * all ArsEndpoint objects
     * @return a businessentity
     */ 
    public static ArsBusinessEntity get(UddiId businessEntityId, boolean getRecursively) throws ArsException {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - get - TODO Implement" );
        else return null;
        
//        //1. create a temp businessentity
//        ArsBusinessEntity tempBusinessEntity = new ArsBusinessEntity();
//
//        //2. create a uddi inquire instance
//        Inquiry inq = new Inquiry();
//
//        //3. call getbusinessdetail
//        BusinessEntity[] entities;
//        try {
//            GetBusinessDetail detail = new GetBusinessDetail(businessEntityId.ID);
//            entities = inq.GetDetail(detail.GetProviderDetail);
//
//            if (entities != null && entities.Length == 1) {
//                tempBusinessEntity._businessEntity = entities[0];
//            } else {
//                return null;
//            }
//        } catch (GetBusinessEntityException) {
//            throw;
//        } catch (InquiryUnexpectedException) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsBusinessEntityUnexpectedException(exp);
//        }
//
//        //4. make a if / else for recursively
//        if (getRecursively) {
//            
//            //5. get ArsEndpoints
//            tempBusinessEntity.Endpoints = new ArsEndpointSet(tempBusinessEntity._businessEntity.Services);
//        }
//
//        return tempBusinessEntity;
    }

    private void saveBusinessEntity() throws ArsException {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - saveBusinessEntity - TODO Implement" );

//        BusinessEntity[] providers = new BusinessEntity[1];
//        providers[0] = _businessEntity;
//
//        try {
//            //1. call Publication
//            Publication uddiPublicationProxy = new Publication(Connection);
//            businessDetail detail = uddiPublicationProxy.Save(providers);
//            _businessEntity = new BusinessEntity(detail.businessEntity[0]);
//        } catch (SaveBusinessEntityException) {
//            throw;
//        } catch (PublicationUnexpectedException) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsBusinessEntityUnexpectedException(exp);
//        }
    }

    /** 
     * Sets the specified endpoint. If the endoint does not exist in the business entity, it is added.
     * If an endpoint with the same ID exists, it is replaced with the specified endpoint.
     * NOTE: Replacement only works for endpoints with an UDDI ID. Newly created endpoints without this ID are simply added.
     * Use ReplaceEndpoint() if you update in-memory endpoint objects without UDDI ID's.
     * 
     * @param  endpoint The endpoint to set
     */ 
    public void setEndpoint(ArsEndpoint endpoint) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setEndpoint - TODO Implement" );

//        if (endpoint == null) return;
//        // 1. Does an endpoint with the same ID exist?
//        if (endpoint.ID != null) {
//            foreach (ArsEndpoint existingEndpoint in Endpoints.Endpoints) {
//                // If a match was found, replace the data
//                if (existingEndpoint.ID != null) {
//                    if (existingEndpoint.ID.ID == endpoint.ID.ID) {
//                        existingEndpoint.Service.Value = endpoint.Service.Value;
//                        return;
//                    }
//                } 
//            }
//        }
//
//        // 2. If no match was found, add the endpoint to the endpoint collection
//        ArsEndpointSet endpointSet = Endpoints;
//        endpointSet.Add(endpoint);
//        Endpoints = endpointSet;
    }


    /** 
     * Replaces an existing endpoint with a new endpoint. If the specified existing endpoint does not exist, the new endpoint is added instead.
     * Comparison is reference-based, not based on the ID of the endpoint.
     * 
     * @param  existingEndpoint The existing endpoint
     * @param  newEndpoint The new endpoint
     */ 
    public void replaceEndpoint(ArsEndpoint existingEndpoint, ArsEndpoint newEndpoint) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - replaceEndpoint - TODO Implement" );

//        if (existingEndpoint == null || newEndpoint == null) return;
//        
//        foreach (ArsEndpoint endpoint in Endpoints.Endpoints) {
//            // If a match was found, replace the data
//            if (existingEndpoint.Service.Value == endpoint.Service.Value) {
//                existingEndpoint.Service.Value = newEndpoint.Service.Value;
//                return;
//            }
//        }
//
//        // If no match was found, add the endpoint instead
//        ArsEndpointSet endpointSet = Endpoints;
//        endpointSet.Add(newEndpoint);
//        Endpoints = endpointSet;

    }

    /** 
     * This method sets all the businessentities values
     * 
     * @param  registrationConformanceClaim Claims regarding conformance to the UDDI registration model
     * @param  oioTaxonomy OIO taxonomy
     * @param  organizationKey Organization identifier
     * @param  organizationKeyType Organization keytype
     * @param  businessEntityContact Entity contact
     * @param  businessDiscoveryUrl Discovery Url
     * @param  description Description text
     * @param  businessName Name of the business entity
     */ 
    public void setBusinessEntityDetails(
            RegistrationConformanceClaim registrationConformanceClaim,
            OIOTaxonomy oioTaxonomy, 
            OrganizationKey organizationKey, 
            OrganizationKeyType organizationKeyType, 
            /*UddiContactSimple*/Object businessEntityContact, 
            /*UddiDiscoveryUrl*/Object businessDiscoveryUrl,
            Name businessName,
            Description description
        ) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setBusinessEntityDetails - TODO Implement" );

//        try {
//            //1. set custom attributes
//            setCustomAttributes(registrationConformanceClaim, oioTaxonomy, organizationKey,
//                    organizationKeyType);
//
//            //2. set contact
//            Contact = businessEntityContact;
//
//            //3. set discoveryUrl
//            Homepage = businessDiscoveryUrl;
//
//            //4. set names
//            Name = businessName;
//
//            //5. set description
//            Description = description;
//        } catch (Exception exp) {
//            throw new ArsBusinessEntityUnexpectedException(exp);
//        }
    }

    /** 
     * Sets all custom attributes for the businessentity; categories and identifiers
     * 
     * @param  registrationConformanceClaim registrationconformanceclaim profile
     * @param  oioTaxonomy OIO taxonomy
     * @param  organizationKey Specific organizationkey
     * @param  organizationKeyType organizationkeytype
     */ 
    private void setCustomAttributes(RegistrationConformanceClaim registrationConformanceClaim, 
            OIOTaxonomy oioTaxonomy, OrganizationKey organizationKey, 
            OrganizationKeyType organizationKeyType) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessEntity - setCustomAttributes - TODO Implement" );

//        try {
//            //1. set categorybag
//            CategoryBag categoryBag = new CategoryBag();
//            this.RegistrationConformanceClaim = registrationConformanceClaim;
//            categoryBag.AddCategory(oioTaxonomy.GetAsKeyedReference());
//            categoryBag.AddCategory(organizationKeyType.GetAsKeyedReference());
//            categoryBag.AddCategory(registrationConformanceClaim.GetAsKeyedReference());
//
//            //2. set identifierbag
//            KeyedReferenceCollection identifierBag = new KeyedReferenceCollection();
//            identifierBag.Add(organizationKey.GetAsKeyedReference());
//            
//            //3. add categorybag to businessentity
//            _businessEntity.Provider.categoryBag = categoryBag.Value;
//            
//            //4. add identifierbag to businessentity
//            _businessEntity.Provider.identifierBag = identifierBag.GetKeyedReferenceCollection();
//        } catch (Exception exp) {
//            throw new ArsBusinessEntityUnexpectedException(exp);
//        }
    }


    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */
    public boolean isValid(String entityName) { //, ref ValidationFailureCollection Failures) {
//        ValidationFailureCollection ChildFailures = null;
//        boolean valid = true;
//
//        RegistrationConformanceClaim.IsValid("RegistrationConformanceClaim", ref Failures);
//        OrganizationKey.IsValid("OrganizationKey", ref Failures);
//        OIOTaxonomy.IsValid("OIO taksonomi", ref Failures);
//        BusinessEntity.IsValid(_businessEntity, "_businessEntity", ref ChildFailures);
//
//        Endpoints.IsValid("Endpoints", ref ChildFailures);
//
//        if (ChildFailures != null){
//            ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, typeof(BusinessEntity),
//                    ChildFailures, ref Failures);
//            Valid = false;
//        }
//
//        return Valid;
        boolean valid = true;

        valid = getRegistrationConformanceClaim().isValid("RegistrationConformanceClaim");
        if(!valid) return false;
        
        valid = getOrganizationKey().isValid("OrganizationKey");
        if(!valid) return false;

        valid = getOIOTaxonomy().isValid("OIO taksonomi");
        if(!valid) return false;

        valid = _businessEntity.validate();
        if(!valid) return false;

        valid = getEndpoints().isValid("Endpoints");
        if(!valid) return false;

        return true;
        
    }

}
