/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.BusinessProcessIdentifierType;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaimCode;
import dk.gov.oiosi.uddi.identifier.BusinessProcessDocument;
import dk.itst.uddi.client.types.core.Description;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.Name;
import dk.itst.uddi.client.types.core.TModel;
    
/** 
 * Represents the Oasis v.2.0.2 recommendations for registering a WSDL in UDDI.
 * 
 */
public class ArsBusinessProcessDefinition extends RegistrationEntity implements IRegistrationEntity {
        
    private TModel _tModel;

    /** 
     * Initialization
     * 
     */
    private void init() {
        _tModel = TModel.Factory.newInstance();
        setDefaultProperties();
    }

    private void setupConnection() {
    }

    /** 
     * Default constructor. Constructs emtpy internal 
     * tModel entities.
     * 
     */
    public ArsBusinessProcessDefinition() {
        init();
        setupConnection();
    }

    /** 
     * Constructor. Constructs the ArsBusinessProcessDefinition from the underlying tModel
     * 
     * @param  tModel The underlying tModel
     */
    public ArsBusinessProcessDefinition(TModel tModel) {
        _tModel = tModel;
        setupConnection();
    }

    /** 
     * Constructor that initializes a BusinessProcessDefinition with a specific porttype
     * 
     * @param  portTypeRegistrationTModelKey porttype registration tmodel
     */ 
    public ArsBusinessProcessDefinition(UddiGuidId portTypeRegistrationTModelKey) {
        init();
        
        //1. set the tmodelkey attribute on the tmodel
        _tModel.setTModelKey( portTypeRegistrationTModelKey.getID());
        setupConnection();
    }

    /** 
     * Sets the default properties of this instance.
     * 
     */
    private void setDefaultProperties() {
        _tModel.getCategoryBag().setKeyedReferenceArray( new KeyedReference[] { new RegistrationConformanceClaim(
                new RegistrationConformanceClaimCode(RegistrationConformanceClaimCode.oiosi1_1)).getAsKeyedReference() });
    }   


    /** 
     * Gets the uuid of the port type registration
     * 
     */
    public UddiId getID() {
        if (_tModel == null || _tModel.getTModelKey() == null || _tModel.getTModelKey() == null ||
                _tModel.getTModelKey().length() < 1) {
            return null;
        } else {
            return IdentifierUtility.getUddiIDFromString(_tModel.getTModelKey());
        } 
    }
    public void setID(UddiId value) {
        if (value != null) {
            _tModel.setTModelKey(value.getID());
        } else {
            _tModel.setTModelKey("");
        }
    }


    /** 
     * Gets or sets the business process definition reference
     * 
     */
    public BusinessProcessDocument getBusinessProcessDocument() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - getBusinessProcessDocument - TODO Implement" );
        else return null;
//        if (_tModel != null && _tModel.IdentifierBag != null) {
//            BusinessProcessDocument procRef = new BusinessProcessDocument();
//            KeyedReference keyref = _tModel.IdentifierBagAsCategoryBagObject.GetCategoryByIdentifier(procRef.IdentifierID);
//            if (keyref != null) {
//                procRef.Value = keyref.KeyValue;
//                return procRef;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    public void setBusinessProcessDocument (BusinessProcessDocument value){
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - setBusinessProcessDocument - TODO Implement" );
//        if (value != null) {
//            if (_tModel.IdentifierBag == null) {
//                CategoryBag identifiers = new CategoryBag();
//                identifiers.SetCategory(value.GetAsKeyedReference());
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            } else {
//                CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//                identifiers.SetCategory(value.GetAsKeyedReference());
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            }
//        } else {
//            BusinessProcessDocument procRefSample = new BusinessProcessDocument();
//            CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//            identifiers.RemoveCategoryById(procRefSample.IdentifierID);
//            _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//        }
    }



    /** 
     * Gets or sets the business process identifier type
     * 
     */ 
    public BusinessProcessIdentifierType getBusinessProcessIdentifierType() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - getBusinessProcessIdentifierType - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            BusinessProcessIdentifierType idType = new BusinessProcessIdentifierType();
//            KeyedReference keyref = _tModel.CategoryBag.GetCategoryByIdentifier(idType.CategoryID);
//            if (keyref != null) {
//                idType.SetCategoryValue(keyref.KeyName, keyref.KeyValue);
//                return idType;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    public void setBusinessProcessIdentifierType (BusinessProcessIdentifierType value){
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - setBusinessProcessIdentifierType - TODO Implement" );
//        if (value != null) {
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            BusinessProcessIdentifierType idTypeSample = new BusinessProcessIdentifierType();
//            _tModel.CategoryBag.RemoveCategoryById(idTypeSample.CategoryID);
//        }
    }
    

    /** 
     * Description of the endpoint
     * 
     */ 
    public Description getDescription() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - getDescription - TODO Implement" );
        else return null;
//        if (_tModel.DescriptionCollection != null && _tModel.DescriptionCollection.Count > 0) {
//            return _tModel.DescriptionCollection[0];
//        } else {
//            return null;
//        }
    }
    public void setDescription (Description value){
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - setDescription - TODO Implement" );
//        if (value != null) {
//            DescriptionCollection descColl = new DescriptionCollection(value);
//            _tModel.DescriptionCollection = descColl;
//        } else {
//            _tModel.DescriptionCollection = null;
//        }
    }


    /** 
     * The name of the endpoint
     * 
     */ 
    public Name getName() {
        if (_tModel != null) {
            return _tModel.getName();
        } else {
            return null;
        }
    }
    public void setName (Name value){
        if (_tModel != null) {
            _tModel.setName(value);
        }
    }


    /** 
     * Sets the category in the category bag, if 'category' is not null.
     * If the category already exists, it is overwritten. If not, it is added.
     * 
     * @param  category The category to add
     */ 
    /*
            private void setCategory(ArsCategory category) {
            if (category == null) return;
            // 1. Does the category already exist?
            ArsCategory existingCategory = getCategory(category);
            if (existingCategory == null) {
                CategoryBag bag = new CategoryBag(_tModel.Value.categoryBag);
                bag.AddCategory(category.GetAsKeyedReference());
                _tModel.Value.categoryBag = bag.Value;
            } else {
                existingCategory.SetCategoryValue(category.CategoryName, category.Value);
            }
        }
     */
    
    /** 
     * Gets an ARS category by the category ID of the 'categoryObject' 
     * parameter. Note that we need an actual ArsCategory descendant 
     * object to get the category identifier. If no match is found, null is returned.
     * 
     * @param  category The category object from which to get the category ID.
     * Another object, with key and value set, is returned as a result.
     * @return 
     */ 
    /*
        private ArsCategory getCategory(ArsCategory category) {
            if (category == null) return null;

            CategoryBag bag = new CategoryBag(_tModel.Value.categoryBag);
            KeyedReference keyRef = bag.GetCategoryByIdentifier(category.CategoryID);
            if (keyRef == null) return null;
            category.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
            return category;
        }
        */
        
    /** 
     * Saves all underlying UDDI registrations associated with this portType
     * 
     */ 
    public void save() {
        // 1. Validate
        validate();
        
        saveTModel();
    }        

    /** 
     * Validates all underlying UDDI registrations associated with this portType
     * 
     */
    public void validate() {
    }

    /** 
     * Updates all underlying UDDI registrations associated with this portType
     * @throws ArsBusinessProcessDefinitionUnexpectedException 
     * 
     */
    public void update() throws MainException {
            
        try {
            // 1. Validate
            validate();

            // 3. Update
            saveTModel();
        } catch (Exception exp) {
            throw new ArsBusinessProcessDefinitionUnexpectedException(exp);
        }
    }

    private void saveTModel() {
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - saveTModel - TODO Implement" );

        TModel[] tmodels = new TModel[1];
        tmodels[0] = _tModel;

//        try {
//
//            //1. call Publication                
//            Publication uddiPublicationProxy = new Publication(Connection);
//            tModelDetail detail = uddiPublicationProxy.Save(tmodels);
//        }   catch (SaveTModelException) {
//            throw;
//        } catch (PublicationUnexpectedException) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsBusinessProcessDefinitionUnexpectedException(exp);
//        }
    }

    /** 
     * Deletes all underlying UDDI registrations associated with this portType
     * NOTE: associated bindings are not deleted as part of this operation.
     * 
     */
    public void delete() throws MainException {
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - delete - TODO Implement" );

//        //1. check that an id exists
//        if (_tModel.Value.tModelKey.Length > 0) {
//
//            String[] temp = new String[1];
//            temp[0] = _tModel.Value.tModelKey;
//
//            //2. call Publication.Delete
//            try {
//                Publication uddiPublicationProxy = new Publication(Connection);
//                bool deleted = uddiPublicationProxy.DeleteTModel(temp);
//
//                //3. removes the tmodel
//                if (deleted) {
//                    _tModel = null;
//                } else {
//                    throw new DeleteEntityException(_tModel.Value.tModelKey);
//                }
//            } catch (DeleteTModelException e) {
//                throw;
//            } catch (PublicationUnexpectedException e) {
//                throw;
//            } catch (Exception exp) {
//                throw new ArsBusinessProcessDefinitionUnexpectedException(exp);
//            }
//        } else {
//            throw new DeleteEntityException("Entity has no identifier. Probably not instantiated");
//        }
    }

    /** 
     * Gets the UDDI registration associated with this portType
     * 
     * @param  businessProcessDefinitionTModelKey uuid of the process definition tmodel
     * @return the porttyperegistration
     */
    public static ArsBusinessProcessDefinition get(UddiId businessProcessDefinitionTModelKey) {
        if( true ) throw new NoSuchMethodError( "ArsBusinessProcessDefinition - get - TODO Implement" );
        else return null;
//        //1. create a temp tmodel
//        ArsBusinessProcessDefinition definition = null;
//
//        //2. create a uddi inquire instance
//        Inquiry inq = new Inquiry();
//
//        //3. call gettmodeldetail
//        TModel[] tmodels;
//        try {
//            GetTModelDetail detail = new GetTModelDetail(businessProcessDefinitionTModelKey.ID);
//            tmodels = inq.GetDetail(detail.Value);
//
//            if (tmodels.Length == 1) {
//                definition = new ArsBusinessProcessDefinition(tmodels[0]);
//                /*definition._tModel = tmodels[0];
//
//                    CategoryBag bag = tmodels[0].CategoryBag;
//                    KeyedReferenceCollection identifierBag = tmodels[0].IdentifierBag;
//
//                    if (bag != null && identifierBag != null) {
//                        KeyedReference processIdentifierType = bag.GetCategoryByName(new BusinessProcessIdentifierType().CategoryName);
//                        definition.BusinessProcessIdentifierType = processIdentifierType.KeyValue;
//
//                        foreach (KeyedReference keyref in identifierBag.GetInnerCollectionAsList()) {
//                            if (keyref.KeyName == new BusinessProcessDocument().IdentifierName) {
//                                definition.BusinessProcessDocument = keyref.KeyValue;
//                            }
//                        }
//                    }*/
//            }
//        } catch (GetBusinessEntityException e) {
//            throw;
//        } catch (InquiryUnexpectedException e) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsBusinessProcessDefinitionUnexpectedException(exp);
//        }
//        return definition;
    }
    
    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */
    public boolean isValid(String EntityName) { //, ref dk.gov.oiosi.uddi.Validation.ValidationFailureCollection Failures) {
//            ValidationFailureCollection ChildFailures = null;
//
//            TModel.IsValid(_tModel, "_tModel", ref ChildFailures);
//
//            if (ChildFailures != null)
//                ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
//                    ChildFailures, ref Failures);
//
//            return Failures == null;
            return _tModel.validate();
        }
    }
