/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import java.util.ArrayList;
import java.util.List;

import dk.itst.uddi.client.types.core.BusinessService;

/**
 * Represents a set of endpoints
 * 
 */
public class ArsEndpointSet {

    // private List<ArsEndpoint> _endpoints = new List<ArsEndpoint>();
    private List _endpoints = new ArrayList();

    /**
     * Default constructor
     * 
     */
    public ArsEndpointSet() {
    }

    /**
     * Constructor that initializes an ArsEndpointSet with existing
     * businessservices
     * 
     * @param services
     *            an array of existing businessServices
     * @throws ArsEndpointSetUnexpectedException
     *             Thrown if an unexpected error occures
     */
    public ArsEndpointSet(BusinessService[] services) throws ArsEndpointSetUnexpectedException {
        try {
            if (services != null) {
                // 1. runs through all services and adds them to the innerlist
                for (int i = 0; i < services.length; i++) {
                    ArsEndpoint tempEndpoint = new ArsEndpoint(services[i]);
                    _endpoints.add(tempEndpoint);
                }
            }
        } catch (Exception ex) {
            throw new ArsEndpointSetUnexpectedException(ex);
        }
    }

    /**
     * Gets and sets Endpoints <ArsEndpoint>
     */
    public List getEndpoints() {
        return _endpoints;
    }

    public void setEndpoints(List value) {
        _endpoints = value;
    }

    /**
     * Adds a endpoint to the innerlist
     * 
     * @param endpoint
     *            the endpoint to add
     * @throws ArsEndpointSetUnexpectedException
     *             Thrown if an unexpected error occures
     */
    public void add(ArsEndpoint endpoint) throws ArsEndpointSetUnexpectedException {
        try {
            // 1. add the endpoint to the innerlist
            if (endpoint != null) {
                getEndpoints().add(endpoint);
            }
        } catch (Exception ex) {
            throw new ArsEndpointSetUnexpectedException(ex);
        }
    }

    /**
     * Removes a endpoint from the innerlist
     * 
     * @param endpoint
     *            the endpoint to remove
     * @throws ArsEndpointSetUnexpectedException
     *             Thrown if an unexpected error occures
     */
    public void remove(ArsEndpoint endpoint) throws ArsEndpointSetUnexpectedException {

        try {
            // 1. remove a endpoint from the innerlist
            if (endpoint != null) {
                getEndpoints().remove(endpoint);
            }
        } catch (Exception ex) {
            throw new ArsEndpointSetUnexpectedException(ex);
        }
    }

    /**
     * Validates the embedded data, and eventually returns a structured report
     * if validations fails
     * 
     * @param EntityName
     * @param Failures
     * @return
     */
    public boolean isValid(String entityName) { // , ref
                                                // ValidationFailureCollection
                                                // Failures) {
    // ValidationFailureCollection ChildFailures = null;
    //
    // foreach (ArsEndpoint Endpoint in _endpoints)
    // Endpoint.IsValid("Endpoint", ref ChildFailures);
    //
    // if (ChildFailures != null)
    // ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName,
    // this.GetType(),
    // ChildFailures, ref Failures);
    //
    // return Failures == null;
        for (int i = 0; i < _endpoints.size(); i++) {
            ArsEndpoint tmp = (ArsEndpoint) _endpoints.get(i);
            if (!tmp.isValid("Endpoint")) {
                return false;
            }
        }
        return true;
    }

}
