/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.BusinessProcessDefinitionReference;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaimCode;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;
import dk.itst.uddi.client.types.core.Description;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.Name;
import dk.itst.uddi.client.types.core.TModel;
import dk.itst.uddi.client.types.core.TModelInstanceInfo;

/**
 * Represents an ARS type process registration
 * 
 */
public class ArsProcessInstance extends RegistrationEntity implements IRegistrationEntity {

    private TModel _tModel = TModel.Factory.newInstance();

    /**
     * Gets the tmodelinstanceinfo
     * 
     */
    public TModelInstanceInfo getInstanceInfo() {
//        TModelInstanceInfo instanceInfo = new TModelInstanceInfo();
//        instanceInfo.Value.tModelKey = _tModel.Value.tModelKey;
        TModelInstanceInfo instanceInfo = TModelInstanceInfo.Factory.newInstance();
        instanceInfo.setTModelKey( _tModel.getTModelKey());
        return instanceInfo;
    }

    /**
     * Gets the uuid of the tmodel
     * 
     */
    public UddiId getID() {
        if (_tModel.getTModelKey() != null && _tModel.getTModelKey().length() > 0)
            return IdentifierUtility.getUddiIDFromString(_tModel.getTModelKey());
        else {
            return null;
        }
    }

    public void setID(UddiId value) {
        if (value != null) {
            _tModel.setTModelKey(value.getID());
        } else {
            _tModel.setTModelKey(null);
        }
    }

    /**
     * The name of the model / instance
     * 
     */
    public Name getName() {
        if (_tModel != null) {
            return _tModel.getName();
        } else {
            return null;
        }
    }

    public void setName(Name value) {
        if (_tModel != null) {
            _tModel.setName(value);
        }
    }

    /**
     * Description of the model / instance
     * 
     */
    public Description getDescription() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - getDescription - TODO Implement" );
        else return null;
        
//        if (_tModel.DescriptionCollection != null && _tModel.DescriptionCollection.Count > 0) {
//            return _tModel.DescriptionCollection[0];
//        } else {
//            return null;
//        }
    }

    public void setDescription(Description value) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setDescription - TODO Implement" );
//        if (value != null) {
//            DescriptionCollection descColl = new DescriptionCollection(value);
//            _tModel.DescriptionCollection = descColl;
//        } else {
//            _tModel.DescriptionCollection = null;
//        }
    }

    // private string _processRoleIdentifier;

    /**
     * Gets the role identifier
     * 
     */
    public BusinessProcessRoleIdentifier getProcessRoleIdentifier() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - getProcessRoleIdentifier - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.IdentifierBag != null && _tModel.IdentifierBag.Value.Items.Length > 0) {
//            BusinessProcessRoleIdentifier roleId = new BusinessProcessRoleIdentifier();
//            KeyedReference keyRef = _tModel.IdentifierBagAsCategoryBagObject.GetCategoryByIdentifier(roleId.IdentifierID);
//            if (keyRef != null) {
//                return new BusinessProcessRoleIdentifier(keyRef.KeyValue);
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    public void setProcessRoleIdentifier(BusinessProcessRoleIdentifier value) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setProcessRoleIdentifier - TODO Implement" );

//        if (value != null) {
//            if (_tModel.IdentifierBag == null) {
//                CategoryBag identifiers = new CategoryBag();
//                identifiers.SetCategory(value.GetAsKeyedReference());
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            } else {
//                CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//                identifiers.SetCategory(value.GetAsKeyedReference());
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            }
//        } else {
//            CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//            if (identifiers != null) {
//                identifiers.RemoveCategoryById(new BusinessProcessRoleIdentifier().IdentifierID);
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            }
//        }
    }

    // private string _processRoleIdentifierType;

    /**
     * Gets the role identifier type
     * 
     */
    public BusinessProcessRoleIdentifierType getProcessRoleIdentifierType() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - getProcessRoleIdentifierType - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null && _tModel.CategoryBag.Value.Items.Length > 0) {
//            BusinessProcessRoleIdentifierType roleId = new BusinessProcessRoleIdentifierType();
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(roleId.CategoryID);
//            if (keyRef != null) {
//                return new BusinessProcessRoleIdentifierType(keyRef);
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    public void setProcessRoleIdentifierType(BusinessProcessRoleIdentifierType value) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setProcessRoleIdentifierType - TODO Implement" );
//        if (value != null) {
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            _tModel.CategoryBag.RemoveCategoryById(new BusinessProcessRoleIdentifierType().CategoryID);
//        }
    }

    // private string _processDefinitionReference;

    /**
     * Gets the process definition reference
     * 
     */
    public BusinessProcessDefinitionReference getProcessDefinitionReference() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - getProcessDefinitionReference - TODO Implement" );
        else return null;
//        if (_tModel != null && _tModel.CategoryBag != null && _tModel.CategoryBag.Value.Items.Length > 0) {
//            BusinessProcessDefinitionReference roleId = new BusinessProcessDefinitionReference();
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(roleId.CategoryID);
//            if (keyRef != null) {
//                return new BusinessProcessDefinitionReference(keyRef);
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    public void setProcessDefinitionReference(BusinessProcessDefinitionReference value) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setProcessDefinitionReference - TODO Implement" );
//        if (value != null) {
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            _tModel.CategoryBag.RemoveCategoryById(new BusinessProcessDefinitionReference().CategoryID);
//        }
    }

    /**
     * Default constructor
     * 
     */
    public ArsProcessInstance() {
        _tModel = TModel.Factory.newInstance();
        // _tModel.aCategoryBag = new CategoryBag();
        setDefaultProperties();
    }

    /**
     * Constructor used when initializing an excisting processinstance
     * 
     * @param tmodel
     *            an existing tmodel
     */
    public ArsProcessInstance(TModel tmodel) {
        // try {
        _tModel = tmodel;
        // } catch(Exception e) {
        // throw;
        // }
    }

    /**
     * Sets the default properties
     * 
     */
    private void setDefaultProperties() {
        _tModel.getCategoryBag().setKeyedReferenceArray(
                new KeyedReference[] {new RegistrationConformanceClaim(new RegistrationConformanceClaimCode(
                        RegistrationConformanceClaimCode.oiosi1_1)).getAsKeyedReference()});
    }

    /**
     * Constructor used when creating a new processinstance
     * 
     * @param businessProcessRoleIdentifer
     *            Identifies a specific role that the service provider takes
     *            within a specific business process.
     * @param businessProcessRoleIdentifierType
     *            Identifies a type of role identifier. The type indicates
     *            the format of the role identifier
     * @param businessProcessDefinitionReference
     *            A reference to the business process definition tModel
     * @param tModelName
     *            the name of the tmodel
     * @param descriptions
     *            the descriptions of the tmodel
     */
    public ArsProcessInstance(BusinessProcessRoleIdentifier businessProcessRoleIdentifer,
            BusinessProcessRoleIdentifierType businessProcessRoleIdentifierType,
            BusinessProcessDefinitionReference businessProcessDefinitionReference, Name tModelName,
            /*DescriptionCollection*/Object descriptions) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - constructor - TODO Implement" );
        try {
            // 1. set tmodel attributes, names and descriptions
            setProcessInstanceDetails(businessProcessRoleIdentifer, businessProcessRoleIdentifierType,
                    businessProcessDefinitionReference, tModelName, descriptions);
        } catch (Exception e) {
            Log log = LogFactory.getLog(ArsProcessInstance.class);
            log.error("Could not create ArsProcessInstance " + e, e);
            // throw;
            // throw;
        }
    }

    /**
     * Gets the UDDI registration associated with this process instance
     * 
     * @param processInstanceId
     *            The ID of the process instance
     * @return Returns the process instance
     */
    public static ArsProcessInstance get(UddiId processInstanceId) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - get - TODO Implement" );
        else return null;

//        // 0. create a temp processinstance
//        ArsProcessInstance tempProcessInstance = null;
//
//        // 1. create a uddi inquire instance
//        Inquiry inq = new Inquiry();
//
//        // 2. call gettmodeldetail
//        TModel[] tmodels;
//        try {
//            GetTModelDetail detail = new GetTModelDetail(processInstanceId.ID);
//            tmodels = inq.GetDetail(detail.Value);
//
//            if (tmodels.Length == 1) {
//                tempProcessInstance = new ArsProcessInstance(tmodels[0]);
//            }
//        } catch (GetTModelDetailException) {
//            throw;
//        } catch (InquiryUnexpectedException) {
//            throw;
//        } catch (Exception) {
//            throw;
//        }
//        return tempProcessInstance;
    }

    /**
     * This method is used to gather all attributes and call there relative
     * set methods
     * @throws ArsException 
     * 
     * @throws ArsProcessInstanceUnexpectedException
     *             Thrown if an unexpected error occures
     */
    private void setProcessInstanceDetails(BusinessProcessRoleIdentifier businessProcessRoleIdentifer,
            BusinessProcessRoleIdentifierType businessProcessRoleIdentifierType,
            BusinessProcessDefinitionReference businessProcessDefinitionReference, Name tModelName,
            /*DescriptionCollection*/Object descriptions) throws ArsException {

        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setProcessInstanceDetails - TODO Implement" );

        try {
            // 1. set custom attributes
            setCustomAttributes(businessProcessRoleIdentifer, businessProcessRoleIdentifierType, businessProcessDefinitionReference);

            // 2. set name
            setName(tModelName);

            // 3. description
            setDescriptions(descriptions);
        } catch (Exception ex) {
            throw new ArsProcessInstanceUnexpectedException(ex);
        }
    }


    /**
     * Sets all custom attributes for the tmodel; categories and identifiers
     * 
     * @throws ArsProcessInstanceUnexpectedException
     *             Thrown if an unexpected error occures
     */
    private void setCustomAttributes(BusinessProcessRoleIdentifier businessProcessRoleIdentifer,
            BusinessProcessRoleIdentifierType businessProcessRoleIdentifierType,
            BusinessProcessDefinitionReference businessProcessDefinitionReference) {

        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setCustomAttributes - TODO Implement" );

//        try {
//
//            // 1. set categorybag
//            _tModel.CategoryBag = new CategoryBag();
//
//            SetDefaultProperties();
//
//            _tModel.CategoryBag.SetCategory(businessProcessRoleIdentifierType.GetAsKeyedReference());
//            _tModel.CategoryBag.SetCategory(businessProcessDefinitionReference.GetAsKeyedReference());
//
//            // 2. set identifierbag
//            KeyedReferenceCollection identifiers = new KeyedReferenceCollection();
//            identifiers.Add(businessProcessRoleIdentifer.GetAsKeyedReference());
//            _tModel.IdentifierBag = identifiers;
//        } catch (Exception ex) {
//            throw new ArsProcessInstanceUnexpectedException(ex);
//        }
    }

    /**
     * Sets the name of the tmodel
     * 
     * @throws ArsProcessInstanceUnexpectedException
     *             Thrown if an unexpected error occures
     */
    private void WHYsetName(Name tModelName) {

        // 1. add all names to the businessservice
        try {
            if (tModelName != null) {
                _tModel.setName(tModelName);
            }
        } catch (Exception ex) {
            // throw new ArsProcessInstanceUnexpectedException(ex);
        }
    }

    /**
     * Sets the description of the tmodel
     * 
     * @throws ArsProcessInstanceUnexpectedException
     *             Thrown if an unexpected error occures
     */
    private void setDescriptions(/*DescriptionCollection*/Object descriptions) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - setDescription - TODO Implement" );

//        // 1. add all descriptions to the businessservice
//        try {
//            if (descriptions != null) {
//                _tModel.DescriptionCollection = descriptions;
//            }
//        } catch (Exception ex) {
//            throw new ArsProcessInstanceUnexpectedException(ex);
//        }
    }

    /**
     * Saves the process registration
     * 
     */
    public void save() {
        try {
            // 1. Verify properties
            validate();

            // 3. Save
            saveTModel();
        } catch(Exception e) {
            Log log = LogFactory.getLog(ArsProcessInstance.class);
            log.error("Could not save ArsProcessInstance " + e, e);
            // throw;
        }
    }

    /**
     * Validates the process registration
     * 
     */
    public void validate() {
        // try {
        // } catch {
        // throw;
        // }
    }

    /**
     * Updates the process registration
     * 
     */
    public void update() {
        try {
            // 1. Verify properties
            validate();

            // 3. Update in uddi
            saveTModel();
        } catch (Exception e) {
            Log log = LogFactory.getLog(ArsProcessInstance.class);
            log.error("Could not update ArsProcessInstance " + e, e);
            // throw;
        }
    }

    /**
     * Deletes the associated UDDI tModel representing this business process
     * instance
     * 
     * @throws DeleteEntityException
     *             This exception is thrown if an error during deletion is
     *             thrown
     * @throws ArsProcessInstanceUnexpectedException
     *             Thrown if a unexpected error occures
     * @throws DeleteEntityException
     *             Thrown if an error occured during deletion of an entity
     */
    public void delete() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - delete - TODO Implement" );

//        // 1. check that an id exists
//        if (_tModel.Value.tModelKey.Length > 0) {
//
//            string[] temp = new string[1];
//            temp[0] = _tModel.Value.tModelKey;
//
//            // 2. call Publication.Delete
//            try {
//                Publication uddiPublicationProxy = new Publication(Connection);
//                bool deleted = uddiPublicationProxy.DeleteTModel(temp);
//
//                // 3. removes the tmodel
//                if (deleted) {
//                    _tModel = null;
//                } else {
//                    throw new DeleteEntityException(_tModel.Value.tModelKey);
//                }
//            } catch (System.ServiceModel.FaultException<uddiorg.api_v3.dispositionReport>) {
//                throw;
//            } catch (DeleteTModelException) {
//                throw;
//            } catch (PublicationUnexpectedException) {
//                throw;
//            } catch (Exception ex) {
//                throw new ArsProcessInstanceUnexpectedException(ex);
//            }
//        } else {
//            throw new DeleteEntityException("Entity has no identifier. Probably not instantiated");
//        }
    }

    /**
     * Saves or updates the tmodel in the UDDI registry
     * 
     * @throws ArsProcessInstanceUnexpectedException
     *             Thrown if a unexpected error occures
     */
    private void saveTModel() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstance - saveTModel - TODO Implement" );

//        TModel[] tmodels = new TModel[1];
//        tmodels[0] = _tModel;
//
//        try {
//
//            // 1. call Publication
//            Publication uddiPublicationProxy = new Publication(Connection);
//            tModelDetail detail = uddiPublicationProxy.Save(tmodels);
//            _tModel = new TModel(detail.tModel[0]);
//        } catch (SaveTModelException) {
//            throw;
//        } catch (PublicationUnexpectedException) {
//            throw;
//        } catch (Exception ex) {
//            throw new ArsProcessInstanceUnexpectedException(ex);
//        }
    }

    /**
     * Validates the embedded data, and eventually returns a structured
     * report if validations fails
     * 
     * @param EntityName
     * @param Failures
     * @return
     */
    public boolean isValid(String entityName) { //, ref ValidationFailureCollection Failures) {
    //            ValidationFailureCollection ChildFailures = null;
    //            bool Valid = true;
    //
    //            TModel.IsValid(_tModel, "_tModel", ref ChildFailures);
    //
    //            if (ChildFailures != null){
    //                ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
    //                    ChildFailures, ref Failures);
    //                Valid = false;
    //            }
    //
    //            return Valid;
        return _tModel.validate();
    }

}
