/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.uddi.IRegistrationEntity;
    
/** 
 * Contains a set of process
 * 
 */
public class ArsProcessInstanceSet implements IRegistrationEntity {

    // private List<ArsProcessInstance> _processes = new List<ArsProcessInstance>();
    private List _processes = new ArrayList();


    /** 
     * Get current process'
     * 
     */ 
    public List getProcesses() {
        return _processes; 
    }

    /** 
     * Removes a processinstance from the instanceset
     * 
     * @param  instance the process instance to remove
     */ 
    public void remove(ArsProcessInstance instance) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstanceSet - remove - TODO Implement" );
//        try {
//            for (int i = 0; i < _processes.Count; i++) {
//                if (_processes[i].ID.ID == instance.ID.ID) {
//                    _processes.RemoveAt(i);
//                    break;
//                }
//            }
//        } catch (Exception e) {
//            throw;
//        }
    }

    /** 
     * Default constructor
     * 
     */
    public ArsProcessInstanceSet() {
    }

    /** 
     * Constructor.
     * 
     * @param  instanceSet 
     */ 
    public ArsProcessInstanceSet(ArsProcessInstanceSet instanceSet) {
//        foreach (ArsProcessInstance instance in instanceSet.Processes)
//        {
//            _processes.Add(instance);
//        }
        List list = instanceSet.getProcesses();
        for(int i=0; i<list.size();i++) {
            ArsProcessInstance tmp = (ArsProcessInstance) list.get(i);
            _processes.add(tmp);
        }
    }
        

    /** 
     * Saves the process set
     * 
     */
    public void save() {

        try {
            validate();
            
            for(int i=0; i<_processes.size(); i++) {
                ArsProcessInstance tmp = (ArsProcessInstance)_processes.get(i);
                tmp.save();
            }
        } catch(Exception e){
            Log log = LogFactory.getLog(ArsProcessInstanceSet.class);
            log.error("Could not save ArsProcessInstanceSet " + e, e);
            //throw;
        }
    }

    /** 
     * Validates the process set
     * 
     */
    public void validate() {
        try {
            for(int i=0; i<_processes.size(); i++) {
                ArsProcessInstance tmp = (ArsProcessInstance)_processes.get(i);
                tmp.validate();
            }
        } catch(Exception e) {
            Log log = LogFactory.getLog(ArsProcessInstanceSet.class);
            log.error("Could not validate ArsProcessInstanceSet " + e, e);
            //throw;
        }
    }

    /** 
     * Updates the process set
     * 
     */
    public void update() {

        try {
            validate();
            for(int i=0; i<_processes.size(); i++) {
                ArsProcessInstance tmp = (ArsProcessInstance)_processes.get(i);
                tmp.update();
            }
        } catch(Exception e) {
            Log log = LogFactory.getLog(ArsProcessInstanceSet.class);
            log.error("Could not update ArsProcessInstanceSet " + e, e);
            //throw;
        }
    }

    /** 
     * Deletes the process set
     * 
     */
    public void delete() {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstanceSet - delete - TODO Implement" );
//        try {
//            // 1. Delete processes:
//            List<ArsProcessInstance> toBeDeleted = new List<ArsProcessInstance>(_processes);
//            foreach (ArsProcessInstance process in toBeDeleted) {
//                // a. delete from UDDI
//                process.Delete();
//
//                // b. remove from list
//                _processes.Remove(process);
//            }
//        } catch {
//            throw;
//        }
    }
        
    /** 
     * Adds a process to the set, but does not update the UDDI.
     * 
     * @param  process The process registration to add
     */ 
    public void add(ArsProcessInstance process) {
        if( true ) throw new NoSuchMethodError( "ArsProcessInstanceSet - add - TODO Implement" );

//        boolean exists = false;
//
//        try {
//            if (process != null) {
//
//                for (int i = 0; i < _processes.Count; i++) {
//                    if (process.ID!=null && _processes[i].ID.ID == process.ID.ID) {
//                        _processes[i] = process;
//                        exists = true;
//                    }
//                }
//
//                if (!exists) {
//                    _processes.Add(process);
//                }
//
//                /* ALSO IN DOT NET
//                    //1. run through all process' and see if it already exists
//                    foreach (ArsProcessInstance instance in _processes) {
//                        if (instance.ID == process.ID) {                            
//                            exists = true;
//                        }
//                    }
//
//                    //2. if the process does not exists, add it to the list
//                    if (!exists) {
//                        _processes.Add(process);
//                    }
//                    else {
//                        //3. if the process already exists, override it
//                        
//                    }
//                    */
//            }
//        } catch {
//            throw;
//        }
    }

    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */
    public boolean isValid(String entityName) { //, ref dk.gov.oiosi.uddi.Validation.ValidationFailureCollection Failures) {
        //            ValidationFailureCollection ChildFailures = null;
        //
        //            foreach (ArsProcessInstance Process in _processes)
        //                Process.IsValid("_processes[i]", ref ChildFailures);
        //
        //            if (ChildFailures != null)
        //                ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
        //                    ChildFailures, ref Failures);
        //
        //            return Failures == null;
            for(int i=0; i<_processes.size(); i++) {
                ArsProcessInstance tmp = (ArsProcessInstance)_processes.get(i);
                if (!tmp.isValid("_processes[i]")) {
                    return false;
                }
            }
            return true;
        }
        
    }
