/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiGuidId;

/**
 * Represents all binding registrations of a OasisServiceRegistrationReference
 * 
 */
public class BindingRegistrationReferenceSet implements IRegistrationEntity {

    // private List<OasisBindingRegistrationReference> pBindingRegistrations
    // = new List<OasisBindingRegistrationReference>();
    private List pBindingRegistrations = new ArrayList();

    /**
     * Default constructor
     * 
     */
    public BindingRegistrationReferenceSet() {
    }

    /**
     * Gets the list of porttype registrations List<OasisBindingRegistrationReference>
     */
    public List getBindingRegistrations() {
        return pBindingRegistrations;
    }

    /**
     * Adds a binding registration to a List
     * 
     * @param reference
     *            the registrationreference to add
     */
    public void add(OasisBindingRegistrationReference reference) {

        try {
            pBindingRegistrations.add(reference);
        } catch (Exception e) {
            Log log = LogFactory.getLog(BindingRegistrationReferenceSet.class);
            log.error("Could not add to BindingRegistrationReferenceSet " + e, e);
            // throw;
        }
    }

    /**
     * Removes a binding registration from a List
     * 
     * @param reference
     *            the registrationreference to delete
     */
    public void delete(OasisBindingRegistrationReference reference) {

        try {
            pBindingRegistrations.remove(reference);
        } catch (Exception e) {
            Log log = LogFactory.getLog(BindingRegistrationReferenceSet.class);
            log.error("Could not remove to BindingRegistrationReferenceSet " + e, e);
            // throw;
        }
    }

    /**
     * Validates that an tmodelkey exists in the referenceset
     * 
     * @param bindingReferenceTModelKey
     *            the tmodelkey to check
     * @return returns true if the tmodelkey exists
     */
    public boolean referenceExists(UddiGuidId bindingReferenceTModelKey) {

        boolean _tempExists = false;

        try {
            // foreach (OasisBindingRegistrationReference bindingRef in
            // pBindingRegistrations) {
            // if (bindingRef.ID == bindingReferenceTModelKey) {
            // _tempExists = true;
            // break;
            // }
            // }
            for (int i = 0; i < pBindingRegistrations.size(); i++) {
                OasisBindingRegistrationReference tmp = (OasisBindingRegistrationReference) pBindingRegistrations.get(i);
                if (tmp.getID().equals(bindingReferenceTModelKey)) {
                    _tempExists = true;
                    break;
                }
            }
        } catch (Exception e) {
            Log log = LogFactory.getLog(BindingRegistrationReferenceSet.class);
            log.error("Could not remove to BindingRegistrationReferenceSet " + e, e);
            // throw;
        }
        return _tempExists;
    }

    /**
     * Saves the portType registrations in this set
     * 
     */
    public void save() {
        // 1. Verify properties for each bindingRegistration
        validate();
    }

    /**
     * Validates the portType registrations in this set
     * 
     */
    public void validate() {
    }

    /**
     * Updates the portType registrations in this set
     * 
     */
    public void update() {
        // 1. Verify properties
        validate();
    }

    /**
     * Delete the portType registrations in this set
     * 
     */
    public void delete() {
    }

    /**
     * Validates the embedded data, and eventually returns a structured report
     * if validations fails
     * 
     * @param EntityName
     * @param Failures
     * @return
     */
    public boolean isValid(String entityName) { // , ref
                                                // ValidationFailureCollection
                                                // Failures) {
    // ValidationFailureCollection ChildFailures = null;
    //
    // foreach (OasisBindingRegistrationReference BindingRef in
    // pBindingRegistrations)
    // BindingRef.IsValid("BindingRegistrations", ref ChildFailures);
    //
    // if (ChildFailures != null)
    // ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName,
    // this.GetType(),
    // ChildFailures, ref Failures);
    //
    // return Failures == null;
        for (int i = 0; i < pBindingRegistrations.size(); i++) {
            OasisBindingRegistrationReference tmp = (OasisBindingRegistrationReference) pBindingRegistrations.get(i);
            if (!tmp.isValid("BindingRegistrations")) {
                return false;
            }
        }
        return true;
    }
}
