/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.UddiOrgWsdlCategorizationTransport;
import dk.gov.oiosi.uddi.category.UddiOrgWsdlPortTypeReference;
import dk.gov.oiosi.uddi.category.UddiOrgXmlNamespace;
import dk.itst.uddi.client.types.core.Description;
import dk.itst.uddi.client.types.core.Name;
import dk.itst.uddi.client.types.core.OverviewDoc;
import dk.itst.uddi.client.types.core.TModel;
    
/** 
 * OASIS binding registration
 * 
 */
public class OasisBindingRegistration extends RegistrationEntity implements IRegistrationEntity {

    private TModel _tModel;

    private void setDefaultProperties() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - setDefaultProperties - TODO Implement" );
        
//        // Sets the registration profile conformance claim
//        _tModel.getCategoryBag().addNewKeyedReference().setKeyedReferenceArray(new KeyedReference [] { new RegistrationConformanceClaim(
//                new RegistrationConformanceClaimCode(RegistrationConformanceClaimCode.oiosi1_1)).getAsKeyedReference() } );
//
//        // Sets the wsdl entity type as default. 
//        UddiOrgWsdlTypes wsdlEntityType = new UddiOrgWsdlTypes(new UddiOrgWsdlTypeCode(UddiOrgWsdlTypeCode.binding));
//        _tModel.CategoryBag.SetCategory(wsdlEntityType.GetAsKeyedReference());
//
//        // Sets the SOAP 1.2 protocol as default.
//        // For now it is set to soap 1.1, because no UDDI taxonomy
//        // exists for indicating this 
//        UddiOrgWsdlCategorizationProtocol bindingProtocol =
//            new UddiOrgWsdlCategorizationProtocol(
//                    UddiOrgWsdlCategorizationProtocolCode.soap1_1Protocol);
//        _tModel.CategoryBag.SetCategory(bindingProtocol.GetAsKeyedReference());
//
//        // Sets the http binding as default.
//        UddiOrgWsdlCategorizationTransport bindingTransport =
//            new UddiOrgWsdlCategorizationTransport(
//                    UddiOrgWsdlCategorizationTransportCode.http);
//        _tModel.CategoryBag.SetCategory(bindingTransport.GetAsKeyedReference());
//
//        // Sets the legacy wsdl type indicator.
//        UddiOrgTypes legacyWsdlIndicator = new UddiOrgTypes(UddiOrgTypesCode.wsdlSpec);
//        _tModel.CategoryBag.SetCategory(legacyWsdlIndicator.GetAsKeyedReference());
    }

    /** 
     * Default constructor. Creates a new tModel entity and generates a key for it.
     */
    public OasisBindingRegistration() {
        _tModel =  TModel.Factory.newInstance();
        //_tModel.CategoryBag = new CategoryBag();
        setDefaultProperties();
    }



    /** 
     * Url to the binding registration definition WSDL document
     */
    public OverviewDoc getOverviewDocument() {
        if (_tModel != null && _tModel.getOverviewDocArray() != null) {
            return _tModel.getOverviewDocArray(0);
        } else {
            return null;
        }
    }
    
    /** 
     * Url to the binding registration definition WSDL document
     */
    public void setOverviewDocument(OverviewDoc value) {
        _tModel.setOverviewDocArray(new OverviewDoc[] {value});
    }

    
    /** 
     * Gets the uuid of the central service definition porttype
     */
    public UddiId getID() {
        if (_tModel != null && _tModel.getTModelKey() != null) {
            return IdentifierUtility.getUddiIDFromString(_tModel.getTModelKey());
        } else {
            return null;
        }
    }

    /** 
     * Gets the uuid of the central service definition porttype
     */
    public void setID(UddiId value) {
        if (value != null) {
            _tModel.setTModelKey(value.getID());
        } else {
            _tModel.setTModelKey("");
        }
    }


    /** 
     * Saves the all associated UDDI entities
     */
    public void save() {
        saveTModel();
    }

    private void saveTModel() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - saveTModel - TODO Implement" );

//        TModel[] tmodels = new TModel[1];
//        tmodels[0] = _tModel;
//
//        try {
//
//            //1. add categories to categorybag
//            _tModel.CategoryBag.SetCategory(BindingNamespace.GetAsKeyedReference());
//            _tModel.CategoryBag.SetCategory(PortTypeDefinitionReference.GetAsKeyedReference());
//
//            //1. call Publication
//            Publication uddiPublicationProxy = new Publication(Connection);
//            tModelDetail detail = uddiPublicationProxy.Save(tmodels);
//            _tModel = new TModel(detail.tModel[0]);
//        } catch (SaveTModelException) {
//            throw;
//        } catch (PublicationUnexpectedException) {
//            throw;
//        } catch (Exception) {
//            throw;
//        }
    }

    /** 
     * Validates all associated UDDI entities
     */
    public void validate() {
    }

    /** 
     * Updates all associated UDDI entities
     */
    public void update() {
        saveTModel();
    }

    /** 
     * Deletes all associated UDDI entities
     */
    public void delete() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - delete - TODO Implement" );

//        //1. check that an id exists
//        if (_tModel.Value.tModelKey.Length > 0) {
//            
//            String[] temp = new String[1];
//            temp[0] = _tModel.Value.tModelKey;
//
//            //2. call Publication.Delete
//            try {
//                Publication uddiPublicationProxy = new Publication(Connection);
//                boolean deleted = uddiPublicationProxy.DeleteTModel(temp);
//
//                //3. removes the tmodel
//                if (deleted) {
//                    _tModel = null;
//                } else {
//                    throw new DeleteEntityException(_tModel.Value.tModelKey);
//                }
//            } catch (DeleteTModelException) {
//                throw;
//            } catch (PublicationUnexpectedException) {
//                throw;
//            } catch (Exception) {
//                throw; ;
//            }
//        } else {
//            throw new DeleteEntityException("Entity has no identifier. Probably not instantiated");
//        }
    }

    /** 
     * Gets an OASIS binding registration object
     * 
     * @param  bindingRegistrationTModelId uuid of the binding registration tmodel
     */ 
    public static OasisBindingRegistration get(UddiId bindingRegistrationTModelId) {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - get - TODO Implement" );
        else return null;
        
//        //1. create a temp tmodel
//        OasisBindingRegistration tempRegistration = new OasisBindingRegistration();
//
//        //2. create a uddi inquire instance
//        Inquiry inq = new Inquiry();
//
//        //3. call gettmodeldetail
//        TModel[] tmodels;
//        try {
//            GetTModelDetail detail = new GetTModelDetail(bindingRegistrationTModelId.ID);
//            tmodels = inq.GetDetail(detail.Value);
//            
//            if (tmodels.Length == 1) {
//                tempRegistration._tModel = tmodels[0];
//                
//                CategoryBag bag = tmodels[0].CategoryBag;
//
//                KeyedReference bindingNamespace = bag.GetCategoryByName(new UddiOrgXmlNamespace().CategoryName);
//                tempRegistration.BindingNamespace = new UddiOrgXmlNamespace(new Uri(bindingNamespace.KeyValue));
//            }
//        } catch (GetBusinessEntityException) {
//            throw;
//        } catch (InquiryUnexpectedException) {
//            throw;
//        } catch (Exception) {
//            throw;
//        }
//        return tempRegistration;
    }

    /** 
     * Gets or sets the name of the underlying tModel
     */
    public Name getName() {
        if (_tModel != null) {
            return _tModel.getName();
        } else {
            return null;
        }
    }

    /** 
     * Gets or sets the name of the underlying tModel
     */
    public void setName( Name value) {
        if (_tModel != null) {
            _tModel.setName(value);
        }
    }


    /** 
     * Description of the endpoint
     */
    public Description getDescription() {
        if (_tModel != null && _tModel.getDescriptionArray() != null && _tModel.sizeOfDescriptionArray()> 0) {
            return _tModel.getDescriptionArray(0);
        } else {
            return null;
        }
    }
    
    /** 
     * Description of the endpoint
     */
    public void setDescription( Description value) {
        if (value != null) {
            //DescriptionCollection descColl = new DescriptionCollection(value);
            _tModel.setDescriptionArray(new Description[] {value});
        } else {
            _tModel.setDescriptionArray(null);
        }
    }


    /** 
     * Gets or sets the namespace of the WSDL binding element
     */
    public UddiOrgXmlNamespace getBindingNamespace() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - getBindingNamespace - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            UddiOrgXmlNamespace uddiOrgXmlNs = new UddiOrgXmlNamespace();
//            
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(uddiOrgXmlNs.CategoryID);
//            if (keyRef != null) {
//                uddiOrgXmlNs.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return uddiOrgXmlNs;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the namespace of the WSDL binding element
     */
    public void setBindingNamespace(UddiOrgXmlNamespace value) {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - setBindingNamespace - TODO Implement" );

//        UddiOrgXmlNamespace test = new UddiOrgXmlNamespace();
//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                
//                throw new Exception("Cannot set namespace: tried to set with wrong category ID ('" +
//                        value.CategoryID + "')");
//            }
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel != null && _tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(test.CategoryID);
//            }
//        }
    }


    /** 
     * Gets or sets the transport of the WSDL binding element
     */
    public UddiOrgWsdlCategorizationTransport getTransport() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - getTransport - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            UddiOrgWsdlCategorizationTransport transport = new UddiOrgWsdlCategorizationTransport();
//            
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(transport.CategoryID);
//            if (keyRef != null) {
//                transport.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return transport;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the transport of the WSDL binding element
     */
    public void setTransport(UddiOrgWsdlCategorizationTransport value) {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - setTransport - TODO Implement" );

//        UddiOrgWsdlCategorizationTransport test = new UddiOrgWsdlCategorizationTransport();
//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                
//                throw new Exception("Cannot set transport: tried to set with wrong category ID ('" +
//                        value.CategoryID + "')");
//            }
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel != null && _tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(test.CategoryID);
//            }
//        }
    }


    /** 
     * Gets or sets the uddi reference to the portType definition tModel
     */
    public UddiOrgWsdlPortTypeReference getPortTypeDefinitionReference() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - getPortTypeDefinitionReference - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            UddiOrgWsdlPortTypeReference portTypeRef = new UddiOrgWsdlPortTypeReference();
//            
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(portTypeRef.CategoryID);
//            if (keyRef != null) {
//                portTypeRef.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return portTypeRef;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the uddi reference to the portType definition tModel
     */
    public void setPortTypeDefinitionReference(UddiOrgWsdlPortTypeReference value) {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - setPortTypeDefinitionReference - TODO Implement" );

//        UddiOrgWsdlPortTypeReference test = new UddiOrgWsdlPortTypeReference();
//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                
//                throw new Exception("Cannot set namespace: tried to set with wrong category ID ('" +
//                        value.CategoryID + "')");
//            }
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel != null && _tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(test.CategoryID);
//            }
//        }
    }

    
    /** 
     * Gets the registration conformance claim regarding Uddi registration model
     */
    public RegistrationConformanceClaim getRegistrationConformanceClaim() {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - getRegistrationConformanceClaim - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            RegistrationConformanceClaim regClaim = new RegistrationConformanceClaim();
//            
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(regClaim.CategoryID);
//            if (keyRef != null) {
//                regClaim.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return regClaim;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets the registration conformance claim regarding Uddi registration model
     */
    public void setRegistrationConformanceClaim( RegistrationConformanceClaim value ) {
        if( true ) throw new NoSuchMethodError( "OasisBindingRegistration - setRegistrationConformanceClaim - TODO Implement" );

//        RegistrationConformanceClaim test = new RegistrationConformanceClaim();
//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                        
//                throw new Exception("Cannot set registration conformance claim: " +
//                        "Tried to set with wrong category ID ('" + value.CategoryID + "')");
//            }
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel != null && _tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(test.CategoryID);
//            }
//        }
    }

    
    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */
    public boolean isValid(String entityName) { // , ref dk.gov.oiosi.uddi.Validation.ValidationFailureCollection Failures) {
        //        ValidationFailureCollection ChildFailures = null;
        //
        //        TModel.IsValid(_tModel, "_tModel", ref ChildFailures);
        //
        //        if (ChildFailures != null)
        //            ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
        //                    ChildFailures, ref Failures);
        //
        //        return Failures == null;
        return _tModel.validate();
    }

}
