/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.EndpointAddressTypeCode;
import dk.itst.uddi.client.types.core.TModelInstanceInfo;

/**
 * Holds a reference to a binding registration tmodel
 * 
 */
public class OasisBindingRegistrationReference {

    private TModelInstanceInfo _bindingReference = TModelInstanceInfo.Factory.newInstance();

    private EndpointAddressTypeCode _transport = new EndpointAddressTypeCode(EndpointAddressTypeCode.http);

    /**
     * Gets or sets the type of endpoint address
     * 
     */
    public EndpointAddressTypeCode getTransport() {
        return _transport;
    }

    public void setTransport(EndpointAddressTypeCode value) {
        _transport = value;
    }

    /**
     * Default constructor
     * 
     */
    public OasisBindingRegistrationReference() {
    }

    /**
     * Constructs a binding registration reference from a TModelInstanceInfo
     * object
     * 
     * @param instanceInfo
     *            The TModelInstanceInfo object
     */
    public OasisBindingRegistrationReference(TModelInstanceInfo instanceInfo) {
        this._bindingReference = instanceInfo;
    }

    /**
     * Constructor with porttype reference
     * 
     * @param bindingTModelKey
     *            UddiGuidId of porttype registration tmodel
     */
    public OasisBindingRegistrationReference(UddiGuidId bindingTModelKey) {

        try {
            setBindingReference(bindingTModelKey);
        } catch (Exception e) {
            Log log = LogFactory.getLog(OasisBindingRegistrationReference.class);
            log.error("Error constructing OasisBindingRegistrationReference " + bindingTModelKey + " : " + e, e);
            // throw;
        }

    }

    /**
     * Gets the uuid binding reference
     * 
     */
    public UddiId getID() {
        return new UddiGuidId(_bindingReference.getTModelKey(), true);
    }

    public void setID(UddiId value) {
        _bindingReference.setTModelKey(value.getID());
    }

    /**
     * Gets a tmodelinstanceinfo
     * 
     */
    public TModelInstanceInfo getBindingReference() {
        return _bindingReference;
    }

    /**
     * Adds the reference to the binding centralservice definition
     * 
     * @param bindingTModelKey
     *            UddiGuidId of binding registration tmodel
     */
    public void setBindingReference(UddiGuidId bindingTModelKey) {

        try {
            // 1. set tmodelkey attribute to the tmodelkey
            _bindingReference.setTModelKey(bindingTModelKey.getID());
        } catch (Exception e) {
            Log log = LogFactory.getLog(OasisBindingRegistrationReference.class);
            log.error("Error setting BindingReference : " + bindingTModelKey + " : " + e, e);
            // throw;
        }
    }

    /**
     * Validates the embedded data, and eventually returns a structured report
     * if validations fails
     * 
     * @param EntityName
     * @param Failures
     * @return
     */
    public boolean isValid(String entityName) { // , ref
                                                // ValidationFailureCollection
                                                // Failures) {
    // ValidationFailureCollection ChildFailures = null;
    //
    // TModelInstanceInfo.IsValid(_bindingReference, "_bindingReference", ref
    // ChildFailures);
    //            
    // if (ChildFailures != null)
    // ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName,
    // this.GetType(),
    // ChildFailures, ref Failures);
    //
    // return Failures == null;
        return _bindingReference.validate();
    }

}
