/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.ArsCategory;
import dk.gov.oiosi.uddi.category.OIOTaxonomy;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.UddiOrgWsdlTypes;
import dk.gov.oiosi.uddi.category.UddiOrgXmlNamespace;
import dk.gov.oiosi.uddi.identifier.ServiceContactEmail;
import dk.gov.oiosi.uddi.identifier.ServiceDocumentationUrl;
import dk.itst.uddi.client.types.core.Description;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.Name;
import dk.itst.uddi.client.types.core.OverviewDoc;
import dk.itst.uddi.client.types.core.TModel;

/** 
 * Represents the Oasis v.2.0.2 recommendations for registering a WSDL in UDDI.
 * 
 */
public class OasisPortTypeRegistration extends RegistrationEntity implements IRegistrationEntity {

    private TModel _tModel = TModel.Factory.newInstance();

    /** 
     * Initialization. Creates an empty tModel and an ID
     */
    private void init() {
        _tModel = TModel.Factory.newInstance();
        _tModel.addNewCategoryBag();
    }

    private void setDefaultProperties() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setDefaultProperties - TODO Implement" );

//        // Sets the registration profile conformance claim
//        _tModel.CategoryBag.SetCategory(new RegistrationConformanceClaim(
//                RegistrationConformanceClaimCode.oiosi1_1).GetAsKeyedReference());
//
//        WsdlType = new UddiOrgWsdlTypes(UddiOrgWsdlTypeCode.portType);
    }

    /** 
     * Default constructor. Constructs emtpy internal 
     * tModel entities.
     */
    public OasisPortTypeRegistration() {
        init();
        setDefaultProperties();
    }

    /** 
     * Constructor that initializes a OasisPortTypeRegistration with a specific porttype
     * 
     * @param  portTypeRegistrationTModelKey porttype registration tmodel
     */ 
    public OasisPortTypeRegistration(UddiGuidId portTypeRegistrationTModelKey) {
        init();
        setDefaultProperties();

        //1. set the tmodelkey attribute on the tmodel
        _tModel.setTModelKey(portTypeRegistrationTModelKey.getID());
    }

    /** 
     * Gets the uuid of the port type registration
     */
    public UddiId getID() {
        if (_tModel.getTModelKey()!=null && _tModel.getTModelKey().length() > 0) {
            return IdentifierUtility.getUddiIDFromString(_tModel.getTModelKey());
        } else {
            return null;
        }
    }
    public void setID(UddiId value) {
        if (value != null) {
            if (_tModel != null) {
                _tModel.setTModelKey(value.getID());
            }
        } else {
            _tModel.setTModelKey("");
        }
    }
        

    /** 
     * Description of the endpoint
     */
    public Description getDescription() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getDescription - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.DescriptionCollection != null && _tModel.DescriptionCollection.Count > 0) {
//            return _tModel.DescriptionCollection[0];
//        } else {
//            return null;
//        }
    }
    
    public void setDescription( Description value ){
        if (value != null) {
            // DescriptionCollection descColl = new DescriptionCollection(value);
            _tModel.setDescriptionArray(new Description[]{value});
        } else {
            _tModel.setDescriptionArray(null);
        }
    }

    /** 
     * The name of the endpoint
     */
    public Name getName() {
        if (_tModel != null) {
            return _tModel.getName();
        } else {
            return null;
        }
    }
    public void setName( Name value){
        if (_tModel != null) {
            _tModel.setName(value);
        }
    }

    String myName = null;
    /** 
     * The name of the endpoint
     */
    public String getMyName() {
        return null;
    }
    public void setMyName(String value) {
        myName = value;
    }

    /** 
     * Gets or sets the service contact email
     */
    public ServiceContactEmail getContactEmail() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getContactEmail - TODO Implement" );
        else return null;

//        if (_tModel != null && _tModel.IdentifierBag != null) {
//            ServiceContactEmail email = new ServiceContactEmail();
//            KeyedReference keyref = _tModel.IdentifierBagAsCategoryBagObject.GetCategoryByIdentifier(email.IdentifierID);
//            if (keyref != null) {
//                email.Value = keyref.KeyValue;
//                return email;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
        
    public void setContactEmail(ServiceContactEmail value) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setContactEmail - TODO Implement" );

//        if (value != null) {
//            if (_tModel.IdentifierBag == null) {
//                CategoryBag identifiers = new CategoryBag();
//                identifiers.SetCategory(value.GetAsKeyedReference());
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            } else {
//                CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//                if (identifiers != null) {
//                    identifiers.SetCategory(value.GetAsKeyedReference());
//                    _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//                }
//            }
//        } else {
//            ServiceContactEmail emailSample = new ServiceContactEmail();
//            CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//            if (identifiers != null) {
//                identifiers.RemoveCategoryById(emailSample.IdentifierID);
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            }
//        }
    }

    
    /** 
     * Gets or sets the OIO taxonomy. 
     */ 
    public OIOTaxonomy getTaxonomy() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getTaxonomy - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            OIOTaxonomy tax = new OIOTaxonomy();
//            KeyedReference keyref = _tModel.CategoryBag.GetCategoryByIdentifier(tax.CategoryID);
//            if (keyref != null) {
//                //tax.SetCategoryValue(new OIOTaxonomy(). CategoryName /*keyref.KeyName*/, keyref.KeyValue);
//                tax.SetCategoryValue(keyref.KeyName, keyref.KeyValue);
//                return tax;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    public void setTaxonomy(OIOTaxonomy value){
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setTaxonomy - TODO Implement" );

//        if (value != null) {
//            if (_tModel.CategoryBag == null) {
//                CategoryBag bag = new CategoryBag();
//                bag.AddCategory(value.GetAsKeyedReference());
//                _tModel.CategoryBag = bag;
//                return;
//            } else {
//                _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//            }
//        } else {
//            if (_tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(new OIOTaxonomy().CategoryID);
//            }
//        }
    }
    

    /** 
     * Gets or sets the service documentation url
     */ 
    public ServiceDocumentationUrl getDocumentationUrl() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getDocumentationUrl - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.IdentifierBag != null) {
//            ServiceDocumentationUrl docUrl = new ServiceDocumentationUrl();
//            KeyedReference keyref = _tModel.IdentifierBagAsCategoryBagObject.GetCategoryByIdentifier(docUrl.IdentifierID);
//            if (keyref != null) {
//                docUrl.Value = keyref.KeyValue;
//                return docUrl;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    
    public void setDocumentationUrl(ServiceDocumentationUrl value) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setDocumentationUrl - TODO Implement" );

//        if (value != null) {
//            if (_tModel.IdentifierBag == null) {
//                CategoryBag identifiers = new CategoryBag();
//                identifiers.SetCategory(value.GetAsKeyedReference());
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            } else {
//                CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//                if (identifiers != null) {
//                    identifiers.SetCategory(value.GetAsKeyedReference());
//                    _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//                }
//            }
//        } else {
//            ServiceDocumentationUrl docUrlSample = new ServiceDocumentationUrl();
//            CategoryBag identifiers = _tModel.IdentifierBagAsCategoryBagObject;
//            if (identifiers != null) {
//                identifiers.RemoveCategoryById(docUrlSample.IdentifierID);
//                _tModel.IdentifierBag = identifiers.GetInnerCollectionAsKeyedReferenceCollection();
//            }
//        }
    }
    
    
    /** 
     * Gets or sets the version of the portType registration (major, minor, revision)
     * Note that 'revision' is mapped to the 'build' field of the 
     * .NET builtin 'System.Version' class.
     */ 
    public Version getVersion() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getVersion - TODO Implement" );
        else return null;
        
//        // 1. Get the individual version parts:
//        Version v = null;
//        try {
//            VersionMajor major = new VersionMajor();
//            major = (VersionMajor)getCategory(major);
//            if (major == null) return null;
//            
//            VersionMinor minor = new VersionMinor();
//            minor = (VersionMinor)getCategory(minor);
//            
//            VersionRevision rev = new VersionRevision();
//            rev = (VersionRevision)getCategory(rev);
//            
//            // 2. Create the version number
//            int majorInt = 0;
//            int minorInt = 0;
//            int revisionInt = 0;
//            
//            majorInt = Int32.Parse(major.Value);
//            if (minor != null) minorInt = Int32.Parse(minor.Value);
//            if (rev != null) revisionInt = Int32.Parse(rev.Value);
//            
//            v = new Version(majorInt, minorInt, revisionInt);
//        } catch {
//            throw;
//        }
//        return v;
    }
    
    public void setVersion( Version value ){
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setVersion - TODO Implement" );

//        if (value != null) {
//            _tModel.CategoryBag.RemoveCategoryById(new VersionMajor().CategoryID);
//            _tModel.CategoryBag.RemoveCategoryById(new VersionMinor().CategoryID);
//            _tModel.CategoryBag.RemoveCategoryById(new VersionRevision().CategoryID);
//
//            SetCategory(new VersionMajor(value.Major.ToString()).GetAsKeyedReference());
//            SetCategory(new VersionMinor(value.Minor.ToString()).GetAsKeyedReference());
//            SetCategory(new VersionRevision(value.Build.ToString()).GetAsKeyedReference());
//        } else {
//            _tModel.CategoryBag.RemoveCategoryById(new VersionMajor().CategoryID);
//            _tModel.CategoryBag.RemoveCategoryById(new VersionMinor().CategoryID);
//            _tModel.CategoryBag.RemoveCategoryById(new VersionRevision().CategoryID);
//        }
    }

    /** 
     * Sets the category in the category bag, if 'category' is not null.
     * If the category already exists, it is overwritten. If not, it is added.
     * 
     * @param  category The category to add
     */ 
    private void setCategory(ArsCategory category) {
        if (category == null) return;
        setCategory(category.getAsKeyedReference());
    }

    /** 
     * Adds a category to the category bag
     * 
     * @param  keyRef A keyed reference
     */
    private void setCategory(KeyedReference keyRef) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setCategory - TODO Implement" );

//        if (keyRef != null) {
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag(keyRef);
//            } else {
//                _tModel.CategoryBag.SetCategory(keyRef);
//            }
//        }
    }

    /** 
     * Removes a category from the bag.
     * 
     * @param  category The category to remove
     */ 
    private void removeCategory(ArsCategory category) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - removeCategory - TODO Implement" );

//        //1. Call remove on the categorybag
//        _tModel.CategoryBag.RemoveCategory(category.GetAsKeyedReference());
    }

    /** 
     * Gets an ARS category by the category ID of the 'categoryObject' 
     * parameter. Note that we need an actual ArsCategory descendant 
     * object to get the category identifier. If no match is found, null is returned.
     * 
     * @param  category The category object from which to get the category ID.
     * Another object, with key and value set, is returned as a result.
     * @return 
     */
    private ArsCategory getCategory(ArsCategory category) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getCategory - TODO Implement" );
        else return null;
        
//        if (category == null) return null;
//        CategoryBag bag = new CategoryBag(_tModel.Value.categoryBag);
//        KeyedReference keyRef = bag.GetCategoryByIdentifier(category.CategoryID);
//        if (keyRef == null) return null;
//        category.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//        return category;
    }



    /** 
     * Url to the porType definition WSDL document
     */
    public OverviewDoc getOverviewDocument() {
        if (_tModel != null && _tModel.sizeOfOverviewDocArray() > 0) {
            return _tModel.getOverviewDocArray(0);
        } else {
            return null;
        }
    }

    public void setOverviewDocument(OverviewDoc value) {
        _tModel.setOverviewDocArray(new OverviewDoc[] { value } );
    }

    /** 
     * Gets or sets the namespace of the WSDL binding element
     */
    public UddiOrgXmlNamespace getBindingNamespace() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getBindingNamespace - TODO Implement" );
        else return null;

//        if (_tModel != null && _tModel.CategoryBag != null) {
//            UddiOrgXmlNamespace uddiOrgXmlNs = new UddiOrgXmlNamespace();
//            
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(uddiOrgXmlNs.CategoryID);
//            if (keyRef != null) {
//                uddiOrgXmlNs.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return uddiOrgXmlNs;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    public void setBindingNamespace(UddiOrgXmlNamespace value) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setBindingNamespace - TODO Implement" );

//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            UddiOrgXmlNamespace test = new UddiOrgXmlNamespace();
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                
//                throw new Exception("Cannot set namespace: tried to set with wrong category ID ('" +
//                        value.CategoryID + "')");
//            }   
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//            
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(new UddiOrgXmlNamespace().CategoryID);
//            }
//        }
    }
    

    /** 
     * Gets or sets the type of WSDL section that the underlying tModel represents
     */ 
    private void setWsdlType(UddiOrgWsdlTypes value) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - setWsdlType - TODO Implement" );

//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            UddiOrgWsdlTypes test = new UddiOrgWsdlTypes();
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                
//                throw new Exception("Cannot set WSDL type: tried to set with wrong category ID ('" +
//                        value.CategoryID + "')");
//            }   
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//            
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(new UddiOrgWsdlTypes().CategoryID);
//            }
//        }
    }

    /** 
     * Gets the registration conformance claim regarding Uddi registration model
     */
    public RegistrationConformanceClaim getRegistrationConformanceClaim() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getRegistrationConformanceClaim - TODO Implement" );
        else return null;
        
//        if (_tModel != null && _tModel.CategoryBag != null) {
//            RegistrationConformanceClaim regClaim = new RegistrationConformanceClaim();
//                
//            KeyedReference keyRef = _tModel.CategoryBag.GetCategoryByIdentifier(regClaim.CategoryID);
//            if (keyRef != null) {
//                regClaim.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return regClaim;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }

    public void RegistrationConformanceClaim(RegistrationConformanceClaim value) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - getRegistrationConformanceClaim - TODO Implement" );

//        if (value != null) {
//            if (_tModel == null) return;
//            // Check that category ID is valid:
//            RegistrationConformanceClaim test = new RegistrationConformanceClaim();
//            if (value.CategoryID.ToLower() != test.CategoryID.ToLower()) {
//                
//                throw new Exception("Cannot set registration conformance claim: " +
//                        "Tried to set with wrong category ID ('" + value.CategoryID + "')");
//            }   
//
//            if (_tModel.CategoryBag == null) {
//                _tModel.CategoryBag = new CategoryBag();
//            }
//
//            _tModel.CategoryBag.SetCategory(value.GetAsKeyedReference());
//        } else {
//            if (_tModel.CategoryBag != null) {
//                _tModel.CategoryBag.RemoveCategoryById(new RegistrationConformanceClaim().CategoryID);
//            }
//        }
    }
        


    /** 
     * Saves all underlying UDDI registrations associated with this portType
     */
    public void save() throws MainException {

        try {
            // 1. Validate
            validate();
            
            saveTModel();
        } catch(MainException e) {
            throw e;
        } catch(Exception e) {
            throw new MainException(e);
        }
    }

    private void saveTModel() throws MainException {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - saveTModel - TODO Implement" );

//        TModel[] tmodels = new TModel[1];
//        tmodels[0] = _tModel;
//
//        try {
//
//            //1. call Publication
//            Publication uddiPublicationProxy = new Publication(Connection);
//            tModelDetail detail = uddiPublicationProxy.Save(tmodels);
//            _tModel = new TModel(detail.tModel[0]);
//        } catch (SaveTModelException e) {
//            log.error("Could not saveTModel " + e, e);
//            // throw;
//        } catch (PublicationUnexpectedException e) {
//            log.error("Could not saveTModel " + e, e);
//            //throw;
//        } catch (Exception e) {
//            log.error("Could not saveTModel " + e, e);
//            //throw;
//        }
    }

    /** 
     * Validates all underlying UDDI registrations associated with this portType
     */
    public void validate() {
    }

    /** 
     * Updates all underlying UDDI registrations associated with this portType
     * @throws MainException 
     */
    public void update() throws MainException {

        try {
            // 1. Validate
            validate();

            // 3. Update
            saveTModel();
        } catch(MainException e) {
            throw e;
        } catch(Exception e) {
            throw new MainException(e);
        }
    }

    /** 
     * Deletes all underlying UDDI registrations associated with this portType
     * NOTE: associated bindings are not deleted as part of this operation.
     */
    public void delete() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - delete - TODO Implement" );

//        //1. check that an id exists
//        if (_tModel.Value.tModelKey.Length > 0) {
//            
//            if (!bindingReferencesExists()) {
//                string[] temp = new string[1];
//                temp[0] = _tModel.Value.tModelKey;
//
//                //2. call Publication.Delete
//                try {
//                    Publication uddiPublicationProxy = new Publication(Connection);
//                    bool deleted = uddiPublicationProxy.DeleteTModel(temp);
//
//                    //3. removes the tmodel
//                    if (deleted) {
//                        _tModel = null;
//                    } else {
//                        throw new DeleteEntityException(_tModel.Value.tModelKey);
//                    }
//                } catch (DeleteTModelException e) {
//                    log.error("Could not delete " + e, e);
//                    //throw;
//                } catch (PublicationUnexpectedException e) {
//                    log.error("Could not delete " + e, e);
//                    // throw;
//                } catch (Exception e) {
//                    log.error("Could not delete " + e, e);
//                    //throw; ;
//                }
//            }
//        } else {
//            throw new DeleteEntityException("Entity has no identifier. Probably not instantiated");
//        }
    }

    /** 
     * Determines if other references to this porttype exists.
     * 
     * @return true if references exists
     */ 
    private boolean bindingReferencesExists() {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - bindingReferencesExists - TODO Implement" );
        else return false;
        
//        Inquiry inq = new Inquiry();
//        CategoryBag bag = new CategoryBag();
//        
//        RegistrationConformanceClaim regConfClaim = new RegistrationConformanceClaim(RegistrationConformanceClaimCode.oiosi1_1);
//        UddiOrgWsdlTypes wsdlTypes = new UddiOrgWsdlTypes(UddiOrgWsdlTypeCode.binding);
//        UddiOrgWsdlPortTypeReference portTypeReference = new UddiOrgWsdlPortTypeReference(ID);
//
//        bag.AddCategory(regConfClaim.GetAsKeyedReference());
//        bag.AddCategory(wsdlTypes.GetAsKeyedReference());
//        bag.AddCategory(portTypeReference.GetAsKeyedReference());
//
//        FindTModel find = new FindTModel(bag.GetInnerCollectionAsKeyedReferenceCollection());
//        tModelList tmodels = inq.Find(find);
//
//        if (tmodels.tModelInfos != null && tmodels.tModelInfos.Length > 0) {
//            return true;
//        } else {
//            return false;
//        }
    }

    /** 
     * Gets the UDDI registration associated with this portType
     * 
     * @param  portTypeRegistrationTModelId uuid of the porttype registration tmodel
     * @return the porttyperegistration
     */ 
    public static OasisPortTypeRegistration get(UddiId portTypeRegistrationTModelId) {
        if( true ) throw new NoSuchMethodError( "OasisPortTypeRegistration - get - TODO Implement" );
        else return null;
        
//        //1. create a temp tmodel
//        OasisPortTypeRegistration tempRegistration = null;
//
//        //2. create a uddi inquire instance
//        Inquiry inq = new Inquiry();
//
//        //3. call gettmodeldetail
//        TModel[] tmodels;
//        try {
//            GetTModelDetail detail = new GetTModelDetail(portTypeRegistrationTModelId.ID);
//            tmodels = inq.GetDetail(detail.Value);
//
//            if (tmodels.Length == 1) {
//                tempRegistration = new OasisPortTypeRegistration();
//                tempRegistration._tModel = tmodels[0];
//
//                CategoryBag bag = tmodels[0].CategoryBag;
//                
//                KeyedReference bindingNamespace = bag.GetCategoryByName(new UddiOrgXmlNamespace().CategoryName);
//                if (bindingNamespace != null) {
//                    tempRegistration.BindingNamespace = new UddiOrgXmlNamespace(new Uri(bindingNamespace.KeyValue));
//                }
//            }
//        } catch (GetBusinessEntityException e) {
//            log.error("Could not get " + e, e);
//            //throw;
//        } catch (GetTModelDetailException e) {
//            log.error("Could not get " + e, e);
//            return null;
//        } catch (InquiryUnexpectedException e) {
//            log.error("Could not get " + e, e);
//            //throw;
//        } catch (Exception e) {
//            log.error("Could not get " + e, e);
//            throw new InquiryUnexpectedException(e);
//        }
//        return tempRegistration;
    }

    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */
    public boolean isValid(String entityName) { // , ref dk.gov.oiosi.uddi.Validation.ValidationFailureCollection Failures) {
        //            ValidationFailureCollection ChildFailures = null;
        //
        //            TModel.IsValid(_tModel, "_tModel", ref ChildFailures);
        //
        //            if (ChildFailures != null)
        //                ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
        //                    ChildFailures, ref Failures);
        //
        //            return Failures == null;
        return _tModel.validate();
    }

}
