/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.itst.uddi.client.types.core.TModelInstanceInfo;

/**
 * Holds a reference to a PortType registration tmodel
 * 
 */
public class OasisPortTypeRegistrationReference {

    private TModelInstanceInfo _portTypeReference = TModelInstanceInfo.Factory.newInstance();

    /**
     * Default constructor
     * 
     */
    public OasisPortTypeRegistrationReference() {
        _portTypeReference = TModelInstanceInfo.Factory.newInstance();
    }

    /**
     * Constructs the object from a TModelInstanceInfo object
     * 
     * @param instanceInfo
     *            The TModelInstanceInfo object
     */
    public OasisPortTypeRegistrationReference(TModelInstanceInfo instanceInfo) {
        this._portTypeReference = instanceInfo;
    }

    /**
     * Constructor with porttype reference
     * 
     * @param portTypeTModelKey
     *            tmodelkey of porttype registration tmodel
     */
    public OasisPortTypeRegistrationReference(UddiGuidId portTypeTModelKey) {

        try {
            setPortTypeReference(portTypeTModelKey);
        } catch (Exception e) {
            Log log = LogFactory.getLog(OasisPortTypeRegistrationReference.class);
            log.error("Error constructing OasisPortTypeRegistrationReference " + portTypeTModelKey + " : " + e, e);
            // throw;
        }
    }

    /**
     * Gets the uuid of the tmodel representing the porttype reference
     * 
     */
    public UddiId getID() {
        return new UddiGuidId(_portTypeReference.getTModelKey(), true);
    }

    public void setID(UddiId value) {
        _portTypeReference.setTModelKey(value.getID());
    }

    /**
     * Gets a tmodelinstanceinfo
     * 
     */
    public TModelInstanceInfo getPortTypeReference() {
        return _portTypeReference;
    }

    /**
     * Adds the reference to the porttype centralservice definition
     * 
     * @param portTypeTModelKey
     *            UddiGuidId of porttype registration tmodel
     */
    public void setPortTypeReference(UddiGuidId portTypeTModelKey) {

        try {
            // 1. set tmodelkey attribute to the tmodelkey
            _portTypeReference.setTModelKey(portTypeTModelKey.getID());
        } catch (Exception e) {
            Log log = LogFactory.getLog(OasisPortTypeRegistrationReference.class);
            log.error("Could not set PortTypeReference " + portTypeTModelKey.getID() + " : " + e, e);
            // throw;
        }
    }

    /**
     * Validates the embedded data, and eventually returns a structured report
     * if validations fails
     * 
     * @param EntityName
     * @param Failures
     * @return
     */
    public boolean isValid(String entityName) { // , ref
                                                // dk.gov.oiosi.uddi.Validation.ValidationFailureCollection
                                                // Failures) {
    // ValidationFailureCollection ChildFailures = null;
    //
    // TModelInstanceInfo.IsValid(_portTypeReference, "_portTypeReference", ref
    // ChildFailures);
    //
    // if (ChildFailures != null)
    // ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName,
    // this.GetType(),
    // ChildFailures, ref Failures);
    //
    // return Failures == null;
        return _portTypeReference.validate();
    }

}
