/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.category.EndpointAddressType;
import dk.gov.oiosi.uddi.category.EndpointAddressTypeCode;

/** 
 * Service registration
 * 
 */
public class OasisServiceSingleBindingRegistration { 

    OasisPortTypeRegistrationReference _portTypeRegistrationRef;
    OasisBindingRegistrationReference _bindingRegistrationRef;

    EndpointAddressType _transport = new EndpointAddressType(new EndpointAddressTypeCode(EndpointAddressTypeCode.email));

    /** 
     * The selected binding / transport type
     */
    public EndpointAddressType getTransport() {
        return _transport; 
    }
    
    public void setTransport(EndpointAddressType value) {
        _transport = value;
    }

    /** 
     * Constructor taking registration set
     * 
     */
    public OasisServiceSingleBindingRegistration() {

        try {
            ArsDefaultInstances.checkDefaultInstances();

            //1. initialize the sets
            //SetPortTypeRef(portTypeRef);
            //SetBindingRef(bindingRef);

            _portTypeRegistrationRef =
                new OasisPortTypeRegistrationReference(
                        (UddiGuidId)IdentifierUtility.getUddiIDFromString(ArsDefaultInstances.getInvoicePortTypeUddiId()));

            _bindingRegistrationRef =
                new OasisBindingRegistrationReference(
                        (UddiGuidId)IdentifierUtility.getUddiIDFromString(ArsDefaultInstances.getInvoiceBindingHttpUddiId()));

            /*_smtpBindingRegistrationRef =
                    new OasisBindingRegistrationReference(
                        (UddiGuidId)IdentifierUtility.GetUddiIDFromString(ArsDefaultInstances.InvoiceBindingSmtpUddiId));
             */
        } catch(Exception e) {
            Log log = LogFactory.getLog(OasisServiceSingleBindingRegistration.class);
            log.error("Error constructing OasisServiceSingleBindingRegistration " + e, e) ;
        }
    }

    private void setPortTypeRef(OasisPortTypeRegistrationReference portTypeRef) {
        if (portTypeRef != null && _portTypeRegistrationRef != null) {
            _portTypeRegistrationRef.setID( IdentifierUtility.getUddiIDFromString(portTypeRef.getID().getID()));
        } else {
            _portTypeRegistrationRef.setID(null);
        }
    }

    private void setBindingRef(OasisBindingRegistrationReference bindingRef) {
        if (bindingRef != null && _bindingRegistrationRef != null) {
            _bindingRegistrationRef.setID(IdentifierUtility.getUddiIDFromString(bindingRef.getID().getID()));
            this.setTransport( new EndpointAddressType(new EndpointAddressTypeCode(bindingRef.getTransport().getCode())));
        } else {
            _bindingRegistrationRef.setID(null);
        }
    }

    private OasisPortTypeRegistrationReference getPortTypeRef() {
        if (_portTypeRegistrationRef != null) {
            OasisPortTypeRegistrationReference temp = new OasisPortTypeRegistrationReference();
            temp.setID(IdentifierUtility.getUddiIDFromString(_portTypeRegistrationRef.getID().getID()));
            return temp;
        } else {
                return null;
        }
    }

    private OasisBindingRegistrationReference getBindingRef() {
        if (_bindingRegistrationRef != null) {
            OasisBindingRegistrationReference temp = new OasisBindingRegistrationReference();
            temp.setID(_bindingRegistrationRef.getID() );
            return temp;
        } else {
            return null;
        }
    }

    /** 
     * Set of porttype registrations
     */
    public OasisPortTypeRegistrationReference getPortTypeRegistrationRef() {
        return getPortTypeRef(); 
    }
    
    /** 
     * Set of porttype registrations
     */
    public void setPortTypeRegistrationRef(OasisPortTypeRegistrationReference value) {
        setPortTypeRef(value); 
    }

    /** 
     * Set of binding registrations
     */
    public OasisBindingRegistrationReference getBindingRegistrationRef() {
        return getBindingRef(); 
    }
    
    /** 
     * Set of binding registrations
     */
    public void setBindingRegistrationRef(OasisBindingRegistrationReference value) {
        setBindingRef(value); 
    }
    

    /** 
     * Validates a portType registration and updates the service registration reference set
     * as well
     */
    public void validate() {
        if (_portTypeRegistrationRef == null || _bindingRegistrationRef == null) {
                
        }
    }

    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */ 
    public boolean isValid(String entityName) { // , ref dk.gov.oiosi.uddi.Validation.ValidationFailureCollection Failures) {
//            ValidationFailureCollection ChildFailures = null;
//
//            BindingRegistrationRef.IsValid("Email Binding", ref ChildFailures);
//            //HttpBindingRegistrationRef.IsValid("Http Binding", ref ChildFailures);
//            PortTypeRegistrationRef.IsValid("PortType Reference", ref ChildFailures);
//
//            if (ChildFailures != null)
//                ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
//                    ChildFailures, ref Failures);
//
//            if (BindingRegistrationRef == null)
//                DataValidationFailure.AddFailure(MissingServiceDefinitionBindingFailure.Message(), "Bindings",
//                    typeof(OasisBindingRegistrationReference), ref Failures);
//
//            return Failures == null;
        
        boolean valid = _bindingRegistrationRef.isValid("Email Binding");
        if(!valid) {
            return false;
        }
        valid = _portTypeRegistrationRef.isValid("PortType Reference");
        if(!valid) {
            return false;
        }

        return true;
    }

}
