/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiGuidId;

/**
 * PortType registration set
 * 
 */
public class PortTypeRegistrationReferenceSet implements IRegistrationEntity {

    /**
     * The list of portTypeRegistrations <OasisPortTypeRegistrationReference>
     */
    private List _portTypeRegistrations = new ArrayList();

    /**
     * Returns a list of portType registration entities
     * 
     */
    public List getPortTypeRegistrations() {
        return _portTypeRegistrations;
    }

    public void setPortTypeRegistrations(List value) {
        _portTypeRegistrations = value;
    }

    /**
     * Adds a PortType registration to a List
     * 
     * @param reference
     *            the registrationreference to add
     */
    public void add(OasisPortTypeRegistrationReference reference) {

        try {
            _portTypeRegistrations.add(reference);
        } catch (Exception e) {
            Log log = LogFactory.getLog(PortTypeRegistrationReferenceSet.class);
            log.error("Could not add OasisPortTypeRegistrationReference " + reference + " : " + e, e);
            // throw;
        }
    }

    /**
     * Removes a PortType registration from a List
     * 
     * @param reference
     *            the registrationreference to delete
     */
    public void delete(OasisPortTypeRegistrationReference reference) {

        try {
            _portTypeRegistrations.remove(reference);
        } catch (Exception e) {
            Log log = LogFactory.getLog(PortTypeRegistrationReferenceSet.class);
            log.error("Could not delete OasisPortTypeRegistrationReference " + reference + " : " + e, e);
            // throw;
        }
    }

    /**
     * Validates that an tmodelkey exists in the referenceset
     * 
     * @param PortTypeReferenceTModelKey
     *            the tmodelkey to check
     * @return returns true if the tmodelkey exists
     */
    public boolean referenceExists(UddiGuidId PortTypeReferenceTModelKey) {

        boolean _tempExists = false;

        try {
            // foreach (OasisPortTypeRegistrationReference PortTypeRef in
            // _portTypeRegistrations) {
            // if (PortTypeRef.ID == PortTypeReferenceTModelKey) {
            // _tempExists = true;
            // break;
            // }
            // }
            for (int i = 0; i < _portTypeRegistrations.size(); i++) {
                OasisPortTypeRegistrationReference tmp = (OasisPortTypeRegistrationReference) _portTypeRegistrations.get(i);
                if (tmp.getID().getID().equals(PortTypeReferenceTModelKey.getID())) {
                    return true;
                }
            }
        } catch (Exception e) {
            Log log = LogFactory.getLog(PortTypeRegistrationReferenceSet.class);
            log.error("Search failed :" + PortTypeReferenceTModelKey + " : " + e, e);
            // throw e;
        }
        return _tempExists;
    }

    /**
     * Saves the portType registrations in this set
     * 
     */
    public void save() {
        // 1. Verify properties for each PortTypeRegistration
        validate();

        // 3. Save
    }

    /**
     * Validates the portType registrations in this set
     * 
     */
    public void validate() {
    }

    /**
     * Updates the portType registrations in this set
     * 
     */
    public void update() {
        // 1. Verify properties
        validate();

        // 3. Update in uddi
    }

    /**
     * Delete the portType registrations in this set
     * 
     */
    public void delete() {
    }

    /**
     * True if valid
     * 
     * @param EntityName
     *            The name of the entity
     * @param Failures
     *            Collection of failures
     * @return True if valid
     */
    public boolean isValid(String entityName) { // , ref
                                                // dk.gov.oiosi.uddi.Validation.ValidationFailureCollection
                                                // Failures) {
    // ValidationFailureCollection ChildFailures = null;
    //
    // foreach (OasisPortTypeRegistrationReference portTypeRef in
    // _portTypeRegistrations)
    // portTypeRef.IsValid("portType", ref ChildFailures);
    //
    // if (ChildFailures != null)
    // ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName,
    // this.GetType(),
    // ChildFailures, ref Failures);
    //
    // return Failures == null;
        for (int i = 0; i < _portTypeRegistrations.size(); i++) {
            OasisPortTypeRegistrationReference tmp = (OasisPortTypeRegistrationReference) _portTypeRegistrations.get(i);
            if (!tmp.isValid("portType")) {
                return false;
            }
        }
        return true;

    }
}
