/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.itst.uddi.client.types.core.KeyedReference;

/**
 * Identifies a type of role identifier. The type indicates the format of a role identifier.
 */
public class BusinessProcessRoleIdentifierType extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static {
        // 1. Set baseclass default values:
        categoryId = new UddiGuidId("uddi:8dd0fa3e-be33-47f9-847b-8d974952a8dc", true);
        categoryName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/businessProcessRoleIdentifierType/";
        defaultKeyName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/businessProcessRoleIdentifierType/";
        defaultKeyValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/UBL2.0ProcessRole/";

        // 2. Set list of categories & possible values for each category
        String[] values = {"http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/UBL2.0ProcessRole/"};

        setCategoryAndValues("http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/businessProcessRoleIdentifierType/", values);
    }

    /**
     * Default constructor. Sets to default.
     */
    public BusinessProcessRoleIdentifierType() {
    }

    /**
     * Constructs a BusinessProcessIdentifierType from a keyed reference.
     * 
     * @param keyRef
     */
    public BusinessProcessRoleIdentifierType(KeyedReference keyRef) throws Exception {
        setCategoryValue(keyRef.getKeyName(), keyRef.getKeyValue());
    }

    /**
     * Use this constructor to set a value.
     */
    public BusinessProcessRoleIdentifierType(BusinessProcessRoleIdentifierTypeCode businessProcessRoleIdentifierType) {

        switch (businessProcessRoleIdentifierType.getCode()) {
        case BusinessProcessRoleIdentifierTypeCode.ubl2_0_ProcessRole:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/UBL2.0ProcessRole/";
            break;
        default:
            pValue = "";
            break;
        }
    }

    // Start ArsCategory abstract members.

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid).
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name.
     */
    private static String categoryName;

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name.
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value.
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members.

    /**
     * String value of code.
     * 
     * @return
     */
    public String toString() {
        return pValue;
    }

}
