/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;

/**
 * Certificate subject string. In UDDI 2.0, possible attributes for holding this
 * value are no longer than 255 chars. We reference the certificate with the
 * certificate subject serialnumber: KeyedReference in the IdentifierBag
 * associated with the TModel for the web service. NOTE: we do this in UDDI 3.0
 * as well.
 */
public class EndpointCertificate extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static { // EndpointCertificate() {
        // 1. Set baseclass default values:
        categoryId = new UddiGuidId("uddi:e6feac92-0aee-4824-ae02-882760609e8a", true);
        categoryName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/endpointCertificate/";
        defaultKeyName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/endpointCertificate/";
        defaultKeyValue = "";

        String[] values = {};

        setCategoryAndValues("http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/endpointCertificate/", values);
    }

    /**
     * Default constructor. Sets to default.
     */
    public EndpointCertificate() {
    }

    /**
     * Use this constructor to set a value.
     * 
     * @param endpointCertificate
     *            endpoint certificate serial
     */
    public EndpointCertificate(String endpointCertificate) {
        pValue = endpointCertificate;
    }

    /**
     * Validate Value within the categorylist idenfied by Category.
     * 
     * @param valueName
     * @param required
     * @param Failures
     * @return
     */
    public boolean isValid(String valueName, boolean required) {
        boolean valid = true;

        if (!ValUtil.isNullOrEmpty(getValue())) {

            if (!valid) {
                // DataValidationFailure.AddFailure(InvalidCertificateValueFailure.Message(Value),
                // ValueName, typeof(ArsCategory), ref Failures);
                valid = false;
            }
        } else if (required) {
            // If field is required then it must not be null

            // DataValidationFailure.AddFailure(
            // RequiredFieldFailure.Message(ValueName), ValueName,
            // typeof(ArsCategory), ref Failures);
            valid = false;
        }

        return valid;
    }

    /**
     * Validate Value within the categorylist idenfied by Category
     * 
     * @param ValueName
     * @param Failures
     * @return
     */
    public boolean isValid(String valueName) {
        return isValid(valueName, false);
    }

    // Start ArsCategory abstract members

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid)
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name
     */
    private static String categoryName;

    /**
     * Gets the category name
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members

}
