/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

/**
 * Represents the endpoint key type code for the following four codes;
 * EAN, CVR, OVT and other.
 */
public class EndpointKeyTypeCode {
    private int value = -1;
    /**
     * Constant value that defines EAN number
     */
    public static final int ean = 0;
    /**
     * Constant value that defines OVT number
     */
    public static final int ovt = 1;
    /**
     * Constant value that defines CVR number
     */
    public static final int cvr = 2;
    /**
     * Constant value that defines other
     */
    public static final int other = 3;
    
    /**
     * Constructor that takes the key type code.
     * 
     * @param v
     */
    public EndpointKeyTypeCode(int v) {
        value = v;
    }

    /**
     * Constructor that takes the key type as a string.
     * 
     * @param type
     */
    public EndpointKeyTypeCode(String type) {
        if ("ean".equalsIgnoreCase(type)) {
            value = ean;
        } else if ("cvr".equalsIgnoreCase(type)) {
            value = cvr;
        } else if ("ovt".equalsIgnoreCase(type)) {
            value = ovt;
        } else {
            value = other;
        }
    }

    /**
     * Get the key type code.
     * 
     * @return
     */
    public int getCode() {
        return value;
    }

    /**
     * Gets the endpoint key type code for EAN number.
     * 
     * @return
     */
    public static EndpointKeyTypeCode getEan() {
        return new EndpointKeyTypeCode(ean);
    }

    /**
     * Gets the endpoint key type code for OVT number.
     * 
     * @return
     */
    public static EndpointKeyTypeCode getOvt() {
        return new EndpointKeyTypeCode(ovt);
    }

    /**
     * Gets the endpoint key type code for CVR number.
     * 
     * @return
     */
    public static EndpointKeyTypeCode getCvr() {
        return new EndpointKeyTypeCode(cvr);
    }

    /**
     * Gets the endpoint key type code for other.
     * 
     * @return
     */
    public static EndpointKeyTypeCode getOther() {
        return new EndpointKeyTypeCode(other);
    }
}
