/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;

/**
 * UDDI category representing an OIO taxonomy.
 */
public class OIOTaxonomy extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static {
        // 1. Set baseclass default values:
        categoryId = new UddiGuidId("uddi:f699264c-384d-47a2-bb46-c6a476242e55", true);
        categoryName = "OIO subject scheme";
        defaultKeyName = "OIO subject scheme / TRADE AND INDUSTRY / E-COMMERCE /";
        defaultKeyValue = "uddi:ea4bc88f-9479-4f9b-a354-4acabdb99336";

        String[] values = {"OIO subject scheme / CULTURE / LITERATURE AND LANGUAGES /",
                "OIO subject scheme / TRADE AND INDUSTRY / E-COMMERCE /", "uddi:41d2b968-d99c-4496-8b9e-1d67f43a4fbc",
                "uddi:ea4bc88f-9479-4f9b-a354-4acabdb99336", "uddi:f699264c-384d-47a2-bb46-c6a476242e55" };

        setCategoryAndValues("OIO subject scheme", values);
        setCategoryAndValues("OIO subject scheme / TRADE AND INDUSTRY / E-COMMERCE /",
                new String[] {"uddi:ea4bc88f-9479-4f9b-a354-4acabdb99336" });
        setCategoryAndValues("OIO subject scheme / CULTURE / LITERATURE AND LANGUAGES /",
                new String[] {"uddi:41d2b968-d99c-4496-8b9e-1d67f43a4fbc" });
    }

    /**
     * Override of baseclass method. Sets the category name and value, but does
     * not check the validity of the value.
     * 
     * @param category
     *            The category name
     * @param value
     *            The category value
     */
    public void setCategoryValue(String category, String value) {
        if (category == null)
            throw new NullArgumentException("category");
        if (value == null)
            throw new NullArgumentException("value");
        pKey = category;
        pValue = value;
    }

    /**
     * Default constructor. Sets to default.
     */
    public OIOTaxonomy() {
    }

    /**
     * Use this constructor to set a value.
     * 
     * @param category
     *            readable taxonomyname
     * @param categoryGuid
     *            guid for the taxonomy
     */
    public OIOTaxonomy(String category, String categoryGuid) {
        pKey = category;
        pValue = categoryGuid;
    }

    // Start ArsCategory abstract members.

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid).
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name.
     */
    private static String categoryName;

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name.
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value.
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members.

}
