/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;

/**
 * Type of the organization identifier, e.g. CVR.
 */
public class OrganizationKeyType extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static {
        // 1. Set baseclass default values:
        categoryId = new UddiGuidId("uddi:0debdbf2-1a1b-46e8-b7fe-7263dc939242", true);
        categoryName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/organizationKeyType/";
        defaultKeyName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/organizationKeyType/";
        defaultKeyValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/cvrNumber/";

        // 2. Set list of categories & possible values for each category
        String[] values = { "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/cvrNumber/",
                "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/ovtNumber/" };

        setCategoryAndValues("http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/organizationKeyType/", values);
    }

    /**
     * Default constructor. Sets to default.
     */
    public OrganizationKeyType() {
    }

    /**
     * Use this constructor to set a value.
     * 
     * @param organizationKeyTypeCode
     *            Organization keytype; cvr or ovt
     */
    public OrganizationKeyType(OrganizationKeyTypeCode organizationKeyTypeCode) {

        switch (organizationKeyTypeCode.getCode()) {
        case OrganizationKeyTypeCode.cvrNumber:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/cvrNumber/";
            break;
        case OrganizationKeyTypeCode.ovtNumber:
            pValue = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/ovtNumber/";
            break;
        default:
            pValue = "";
            break;
        }
    }

    // Start ArsCategory abstract members.

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid).
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name.
     */
    private static String categoryName;

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name.
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value.
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members.

}
