/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiNonGuidId;

/**
 * OASIS recom. 2.0.2 for registering WSDL in UDDI. The type of tModel, i.e. this tModel represents a WSDL portType definition.
 */
public class UddiOrgWsdlTypes extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static {
        // 1. Set baseclass default values:
        // Systinet does not use the uddi: format for old taxonomies
        // _categoryId = new
        // UddiGuidId("uddi:6e090afa-33e5-36eb-81b7-1ca18373f457", true);
        categoryId = new UddiNonGuidId("uddi:uddi.org:wsdl:types");
        categoryName = "uddi-org:wsdl:types";
        defaultKeyName = "uddi-org:wsdl:types";
        defaultKeyValue = "";

        // 2. Set list of categories & possible values for each category
        String[] values = {"portType", "binding"};

        setCategoryAndValues("uddi-org:wsdl:types", values);
    }

    /**
     * Default constructor. Sets to default.
     */
    public UddiOrgWsdlTypes() {
    }

    /**
     * Use this constructor to set a value.
     * 
     * @param wsdlType
     *            the type of tmodel
     */
    public UddiOrgWsdlTypes(UddiOrgWsdlTypeCode wsdlType) {
        pValue = wsdlType.toString();
    }

    // Start ArsCategory abstract members.

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid)
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name.
     */
    private static String categoryName;

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name.
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value.
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members.

}
