/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiNonGuidId;

/**
 * OASIS recom. 2.0.2 for registering WSDL in UDDI. The namespace of the WSDL
 * portType. Any URL to the WDSL containing the portType definition, e.g.
 * "http://example.com/travelagent/wsdl".
 */
public class UddiOrgXmlNamespace extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static {
        // 1. Set baseclass default values:
        // Systinet does not use the uddi: format for old taxonomies
        // _categoryId = new
        // UddiGuidId("uddi:d01987d1-ab2e-3013-9be2-2a66eb99d824", true);
        categoryId = new UddiNonGuidId("uddi:uddi.org:xml:namespace");
        categoryName = "uddi-org:xml:namespace";
        defaultKeyName = "uddi-org:xml:namespace";
        defaultKeyValue = "";

        String[] values = {};

        setCategoryAndValues("uddi-org:xml:namespace", values);
    }

    /**
     * Default constructor. Sets to default.
     */
    public UddiOrgXmlNamespace() {
    }

    /**
     * Use this constructor to set a value.
     * 
     * @param xmlNamespace
     *            Any URL to the WDSL containing the portType definition, e.g.
     *            "http://example.com/travelagent/wsdl"
     */
    public UddiOrgXmlNamespace(Uri xmlNamespace) {
        pValue = xmlNamespace.toString();
    }

    // Start ArsCategory abstract members.

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid)
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name.
     */
    private static String categoryName;

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value.
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members.
}
