/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.identifier;

import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;

/**
 * Identifies a specific role that the service provider takes within a specific business process. Indicates a role as a text string.
 * No roles defined yet for the UBL processes.
 */
public class BusinessProcessRoleIdentifier extends ArsIdentifier {

    private static List possibleValues = new ArrayList();

    private static UddiId identifierId;

    private static String identifierName;

    private static String defaultIdentifierValue = "";

    private static boolean defaultValueExists = false;

    private static boolean fixedValueListExists = false;

    /**
     * Static constructor. Initializes name, id and default valute of the identifier
     */
    static {
        identifierId = new UddiGuidId("uddi:4b2e5d7e-8e5d-4c03-92ca-3597b7f52444", true);
        identifierName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/businessProcessRoleIdentifier/";
        defaultValueExists = false;
        fixedValueListExists = false;
    }

    /**
     * Default constructor. Sets to default.
     * 
     */
    public BusinessProcessRoleIdentifier() {
    }

    /**
     * Use this constructor to set a value
     * 
     * @param businessProcessRoleIdentifier
     *            Indicates a role as a text string
     */
    public BusinessProcessRoleIdentifier(String businessProcessRoleIdentifier) {
        // No roles defined yet for the UBL processes.
        pValue = businessProcessRoleIdentifier;
    }

    /**
     * Gets the UDDI name of the identifier
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getIdentifierName()
     */
    public String getIdentifierName() {
        return identifierName;
    }

    /**
     * Gets the UDDI tModel id of the identifier
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getIdentifierID()
     */
    public String getIdentifierID() {
        return identifierId.getID();
    }

    /**
     * True if a default value exists
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getDefaultValueExists()
     */
    public boolean getDefaultValueExists() {
        return defaultValueExists;
    }

    /**
     * True if a predefined list of possible values exist for this identifier
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getFixedValueListExists()
     */
    public boolean getFixedValueListExists() {
        return fixedValueListExists;
    }

    /**
     * Returns the default value of this identifier if any exist, else returns null. You may check "DefaultValueExists" before
     * calling.
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getDefaultValue()
     */
    public String getDefaultValue() {
        return defaultIdentifierValue;
    }

    /**
     * Returns a list of possible values
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getPossibleValues()
     */
    public List getPossibleValues() {
        return possibleValues;
    }

    /**
     * Compares the two objects and returns true if they have equal values
     * 
     * @param other
     *            The object to compare to
     * @return Returns true if the two objects have identical values
     */
    public boolean equals(BusinessProcessRoleIdentifier other) {
        if (other == null) {
            return false;
        }

        if (getIdentifierID().equals(other.getIdentifierID()))
            return false;
        return true;
    }

}
