/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.identifier;

import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;

/**
 * Link to a document containing technical description of all interfaces of the service definition (other than WSDL). Example:
 * http://testdomain.com/documentation.pdf
 */
public class ServiceDocumentationUrl extends ArsIdentifier {

    private static List possibleValues = new ArrayList();

    private static UddiId identifierId;

    private static String identifierName;

    private static String defaultIdentifierValue = "";

    private static boolean defaultValueExists = false;

    private static boolean fixedValueListExists = false;

    /**
     * Static constructor. Initializes name, id and default valute of the identifier
     */
    static {
        identifierId = new UddiGuidId("uddi:a2d88813-d3db-4890-b1dc-170215106896", true);
        identifierName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Identifiers/documentationURL/";
        defaultValueExists = false;
        fixedValueListExists = false;
    }

    /**
     * Default constructor. Sets to default.
     */
    public ServiceDocumentationUrl() {
    }

    /**
     * Use this constructor to set a value
     * 
     * @param serviceDocumentationUrl
     *            Url for the servicedocumentation
     */
    public ServiceDocumentationUrl(String serviceDocumentationUrl) {
        pValue = serviceDocumentationUrl;
    }

    /**
     * Gets the UDDI name of the identifier
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getIdentifierName()
     */
    public String getIdentifierName() {
        return identifierName;
    }

    /**
     * Gets the UDDI tModel id of the identifier
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getIdentifierID()
     */
    public String getIdentifierID() {
        return identifierId.getID();
    }

    /**
     * True if a default value exists
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getDefaultValueExists()
     */
    public boolean getDefaultValueExists() {
        return defaultValueExists;
    }

    /**
     * True if a predefined list of possible values exist for this identifier
     * 
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getFixedValueListExists()
     */
    public boolean getFixedValueListExists() {
        return fixedValueListExists;
    }

    /**
     * Returns the default value of this identifier if any exist, else returns null. You may check "DefaultValueExists" before
     * calling.
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getDefaultValue()
     */
    public String getDefaultValue() {
        return defaultIdentifierValue;
    }

    /**
     * Returns a list of possible values
     * 
     * (non-Javadoc)
     * 
     * @see dk.gov.oiosi.uddi.identifier.ArsIdentifier#getPossibleValues()
     */
    public List getPossibleValues() {
        return possibleValues;
    }

}
