/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.xml.xslt;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import javax.xml.stream.XMLStreamException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;

/**
 * This utility is used for xslt transforming documents.
 */
public class XsltUtility {

    /**
     * logger.
     */
    private static Log log = LogFactory.getLog(XsltUtility.class);

    /**
     * Method to transform any xmldocument with a given xslt.
     *
     * @param xmlDoc
     *            The xml document to transform.
     * @param stylesheet
     *            The xslt to transform xml document.
     * @return The transformed xml document
     * @throws XmlException
     *             Xml transformations errors
     * @throws Exception
     *             yet unknown errors
     */
    public XmlDocument transformXML(XmlDocument xmlDoc, XmlDocument stylesheet) throws XmlException {

        XmlDocument transformedXML = null;
        try {
            Source xsltSource = new DOMSource(stylesheet.getW3Document());

            TransformerFactory transFact = TransformerFactory.newInstance();
            Transformer trans;

            Element w3doc = xmlDoc.getW3Document();

            StreamResult sr = new StreamResult(new ByteArrayOutputStream());
            trans = transFact.newTransformer(xsltSource);
            log.debug("Doing transformation...");
            trans.transform(new DOMSource(w3doc.getOwnerDocument()), sr);

            String result = sr.getOutputStream().toString();
            log.debug("Creating AXOMbuilder from: " + result);
            if ("<?xml version=\"1.0\" encoding=\"utf-8\"?>".equals(result)) {
                log.warn("?? Transformation is empty ?? ");
            } else {
                StAXOMBuilder builder = new StAXOMBuilder(new ByteArrayInputStream(result.getBytes()));
                transformedXML = new XmlDocument(builder.getDocumentElement());
            }
        } catch (TransformerConfigurationException e) {
            throw new XmlException("XSL transformation error: " + e.getMessage());
        } catch (TransformerException e) {
            throw new XmlException("XSL transformation error: " + e.getMessage());
        } catch (XMLStreamException e) {
            throw new XmlException("XSL transformation error: " + e.getMessage());
        } catch (Exception e) {
            log.error("Failed : " + e, e);
            throw new RaspLibraryException("RASP transformation error ! " + e, e);
        }
        log.debug("Transformation finished successfully !");

        return transformedXML;

    }

}
