package dk.gov.oiosi.test.unit.configuration;

import java.text.ParseException;

import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.communication.configuration.DocumentEndpointInformation;
import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.KeyTypeMapping;
import dk.gov.oiosi.communication.configuration.KeyTypeMappingExpression;
import dk.gov.oiosi.communication.configuration.KeyTypeMappingFailedException;
import dk.gov.oiosi.communication.configuration.ServiceEndpointFriendlyName;
import dk.gov.oiosi.communication.configuration.ServiceEndpointKey;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.uddi.UddiConfig;
import dk.gov.oiosi.uddi.UddiLookupClientFactoryConfig;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import junit.framework.TestCase;

public class RaspConfigurationTest extends TestCase {
	
	public void testDocumentTypeConfiguration() throws ConfigurationException, KeyTypeMappingFailedException {
		RaspConfigurationHandler handler = RaspConfigurationHandler.getInstance();
		DocumentTypeCollectionConfig rdtcc = handler.getRaspDocumentTypeCollectionConfig();
		DocumentTypeConfig[] docTypes = rdtcc.getDocumentTypes();
		assertTrue(docTypes.length>0);
		DocumentTypeConfig docType = docTypes[0];
		docType.getCustomHeaderConfiguration().getXPaths();
		
		assertEndpointType(docType.getEndpointType());		
		assertNotNullOrEmpty(docType.getFriendlyName());
		assertNotNullOrEmpty(docType.getId().getGuid());
		
		assertPositiveInteger(docType.getIdentifierDiscriminators().getXPathDiscriminatorConfigs().length);
		
		assertPositiveInteger(docType.getNamespaces().length);
		
		assertNotNullOrEmpty(docType.getRootName());
		assertNotNullOrEmpty(docType.getRootNamespace());
		assertNotNullOrEmpty(docType.getSchemaPath());
		assertNotNullOrEmpty(docType.getServiceContractTModel());
		assertNotNullOrEmpty(docType.getStylesheetPath());
		assertNotNull(docType.getXsltTransformStylesheetPath());
	}
	
	public void testUddiConfiguration() throws ParseException {
		UddiLookupResponse ulr = RaspConfigurationHandler.getInstance().getUddiClientTestConfig();
		ulr.getActivationDate();
		ulr.getCertificateSubjectSerialNumber();
		ulr.getEndpointAddress();
		ulr.getEndpointIdentifierActual();
		ulr.getExpirationDate();
		ulr.getNewerVersionReference();
		ulr.getServiceContactEmail();
		ulr.getTermsOfUseUrl();
		ulr.getVersion();
		
		UddiConfig uc = RaspConfigurationHandler.getInstance().getUddiConfig();
		assertPositiveInteger(uc.getFallbackTimeoutMinutes());
		assertPositiveInteger(uc.getGatewayRange().getRangeStartInt());
		assertPositiveInteger(uc.getGatewayRange().getRangeEndInt());
		assertPositiveInteger(uc.getLookupReturnOptions().getCode());
		assertNotNullOrEmpty(uc.getLookupReturnOptions().getValue());
		
		assertNotNullOrEmpty(uc.getRegistrationConformanceClaim());
		assertNotNullOrEmpty(uc.getUddiInquireEndpointURL());
		assertNotNullOrEmpty(uc.getUddiInquireEndpointURLFallback());
		assertNotNullOrEmpty(uc.getUddiPublishEndpointURL());
		assertNotNullOrEmpty(uc.getUddiSecurityEndpointURL());
		
		UddiLookupClientFactoryConfig ulcfc = RaspConfigurationHandler.getInstance().getUddiLookupClientFactoryConfig();
		assertNotNullOrEmpty(ulcfc.getImplementationNamespaceClass());
	}
	
	private void assertEndpointType(DocumentEndpointInformation endpointInfo) throws KeyTypeMappingFailedException {
		assertNotNull(endpointInfo);
		assertNotNullOrEmpty(endpointInfo.getReplyAction());
		assertNotNullOrEmpty(endpointInfo.getRequestAction());
		
		ServiceEndpointFriendlyName friendlyName = endpointInfo.getEndpointFriendlyName();
		assertNotNullOrEmpty(friendlyName.getXPath());
		ServiceEndpointFriendlyName senderFriendlyName = endpointInfo.getSenderFriendlyName();
		assertNotNullOrEmpty(senderFriendlyName.getXPath());
	
		String keyType = "EndpointKeyType";
		ServiceEndpointKey key = endpointInfo.getKey();
		KeyTypeMappingExpression keyMapping = key.getMappingExpression(keyType);
		assertKeyTypeMappingExpression(keyMapping);
		
		ServiceEndpointKey senderKey = endpointInfo.getSenderKey();
		KeyTypeMappingExpression senderKeyMapping = senderKey.getMappingExpression(keyType);
		assertKeyTypeMappingExpression(senderKeyMapping);
	}
	
	private void assertKeyTypeMappingExpression(KeyTypeMappingExpression keyMapping) throws KeyTypeMappingFailedException {
		assertNotNull(keyMapping);
		String cvrMappingName = "CVR";
		KeyTypeMapping mapping = keyMapping.getMapping(cvrMappingName);
		assertNotNull(mapping);
	}
	
	private void assertNotNullOrEmpty(String s){
		assertNotNull("String was null",s);
		assertFalse("String was empty", ValUtil.isNullOrEmpty(s));
	}
	
	private void assertPositiveInteger(int i){
		assertTrue("Negative integer " + i, i>=0);
	}
	
	private void assertPositiveInteger(long i){
		assertTrue("Negative integer " + i, i>=0);
	}
}
