/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.xml.documenttype;

import java.io.FileNotFoundException;

import junit.framework.TestCase;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.NoDocumentTypeFoundFromXmlDocumentException;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;

/**
 * test document type config search.
 */
public class DocumentTypeConfigSearcherTest extends TestCase {

    /**
     * searcher.
     */
    private static DocumentTypeConfigSearcher searcher;

    /**
     * test init searcher.
     */
    public void testDocumentTypeConfigSearcher() {
        try {
            searcher = new DocumentTypeConfigSearcher();
            assertTrue(true);
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /**
     * search for invoice.
     */
    public void testSearchForInvoice() {
        try {
            String path = TestConstants.PATH_INVOICE_XML;
            DocumentTypeConfig documentType = searchForDocument(path);
            assertEquals(documentType.getRootName(), "Invoice");
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /**
     * search for credit note.
     */
    public void testSearchForCreditNote() {
        try {
            String path = TestConstants.PATH_CREDITNOTE_XML;
            DocumentTypeConfig documentType = searchForDocument(path);
            assertEquals(documentType.getRootName(), "CreditNote");
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }
    
    /**
     * search for order.
     */
    public void testSearchForOrder() {
        try {
            String path = TestConstants.PATH_ORDER_XML;
            DocumentTypeConfig documentType = searchForDocument(path);
            assertEquals(documentType.getRootName(), "Order");
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }
    
    
    /**
     * find unknown type.
     */
    public void testSearchForUnkownType() {
        try {
            String path = TestConstants.PATH_UNKNOWNTYPE_XML;
            DocumentTypeConfig documentType = searchForDocument(path);
            assertNotNull(documentType);
            assertNotNull(documentType.getFriendlyName());
            assertTrue(!documentType.getFriendlyName().equals(""));
        } catch (NoDocumentTypeFoundFromXmlDocumentException e) {
            // Expected no result from search...
            assertTrue(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }

    }

    /**
     * parameter test.
     */
    public void testNullArguement() {
        try {
            searcher.findUniqueDocumentType(null);
            assertTrue(false);
        } catch (NullArgumentException e) {
            // 
            assertTrue(true);
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /**
     * read document from filesystem to be used for searching.
     * 
     * @param path
     *            path
     * @return documnet type config
     * @throws MainException
     * @throws FileNotFoundException 
     */
    private DocumentTypeConfig searchForDocument(String path) throws MainException, FileNotFoundException {
        XmlDocument document = new XmlDocument();
        document.load(path);
        DocumentTypeConfig documentType = searcher.findUniqueDocumentType(document);
        return documentType;
    }
}
