/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.xml.schema;

import java.io.FileNotFoundException;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidationFailedException;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schema.UnexpectedNamespaceException;

/**
 * perform schema validation tests.
 */
public class SchemaValidationTest extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(SchemaValidationTest.class);

    /**
     * document searcher.
     */
    private static DocumentTypeConfigSearcher docSearcher;

    /**
     * init searcher.
     */
    public void testInitDocumentTypes() {
        try {
            docSearcher = new DocumentTypeConfigSearcher();
            assertTrue(true);
        } catch (MainException e) {
            log.error("Init DocumentTypeConfigSearcher should not fail : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test app response.
     */
    public void testApplicationResponseValidation() {
        log.info("Positive test : testApplicationResponseValidation");
        try {
            String xmlPath = TestConstants.PATH_APPLICATIONRESPONSE_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());

        }
    }

    /**
     * test credit note.
     */
    public void testCreditNoteValidation() {
        log.info("Positive test : testCreditNoteValidation");
        try {
            String xmlPath = TestConstants.PATH_CREDITNOTE_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test invoice.
     */
    public void testInvoiceValidation() {
        log.info("Positive test : testInvoiceValidation");
        try {
            String xmlPath = TestConstants.PATH_INVOICE_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test order.
     */
    public void testOrderValidation() {
        log.info("Positive test : testOrderValidation");
        try {
            String xmlPath = TestConstants.PATH_ORDER_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test order response.
     */
    public void testOrderResponseSimpleValidation() {
        log.info("Positive test : testOrderResponseSimpleValidation");
        try {
            String xmlPath = TestConstants.PATH_ORDERRESPONSESIMPLE_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test reminder.
     */
    public void testReminderValidation() {
        log.info("Positive test : testReminderValidation");
        try {
            String xmlPath = TestConstants.PATH_REMINDER_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);
//        } catch (SchemaValidationFailedException e) {
            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test invoice 07.
     */
    public void testInvoice07Validation() {
        log.info("Positive test : testInvoice07Validation");
        try {
            String xmlPath = TestConstants.PATH_INVOICE07_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test creditnote 07.
     */
    public void testCreditNote07Validation() {
        log.info("Positive test : testCreditNote07Validation");
        try {
            String xmlPath = TestConstants.PATH_CREDITNOTE07_XML;
            String schemaPath = null; // get from document searcher
            validate(xmlPath, schemaPath);

            assertTrue(true);
        } catch (Exception e) {
            log.error("Document should validate : " + e);
            fail(e.getMessage());
        }
    }

    /**
     * test validation failure.
     */
    public void testInvoiceWrongNamespaceValidation() {
        log.info("Failure test : testInvoiceWrongNamespaceValidation()");
        try {
            String xmlPath = TestConstants.PATH_INVOICEWRONGNAMESPACE_XML;
            String schemaPath = TestConstants.PATH_INVOICE_XSD;
            validate(xmlPath, schemaPath); // _validator201);
            log.error("Document should NOT validate : " + TestConstants.PATH_INVOICEWRONGNAMESPACE_XML);
            fail("This should fail");
        } catch (SchemaValidationFailedException e) {
            log.info("Document validation failed as excpected : " + e);
            assertTrue(true);
        } catch (UnexpectedNamespaceException e) {
            log.info("Document validation failed as excpected : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("Test should faild with SchemaValidationFailedException - got : " + e, e);
            fail("This should fail" + e);
        }
    }

    /**
     * test validation failure.
     */
    public void testInvoiceWrongElementValidation() {
        log.info("Failure test : testInvoiceWrongElementValidation()");
        try {
            String xmlPath = TestConstants.PATH_INVOICEWRONGELEMENT_XML;
            String schemaPath = TestConstants.PATH_INVOICE_XSD;
            validate(xmlPath, schemaPath);
            log.error("Document should NOT validate : " + TestConstants.PATH_INVOICEWRONGELEMENT_XML);
            assertTrue(false);
        } catch (SchemaValidationFailedException e) {
            log.info("Document validation failed as excpected : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("Test should faild with SchemaValidationFailedException - got : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * perform validation.
     * 
     * @param xmlDocumentPath
     *            document to validate
     * @param xmlSchemaPath
     *            path to schema
     * @throws SchemaValidationFailedException
     * @throws MainException
     * @throws FileNotFoundException 
     */
    private void validate(String xmlDocumentPath, String xmlSchemaPath) throws SchemaValidationFailedException, MainException, FileNotFoundException {

        log.info("Schema Validation started - document path : " + xmlDocumentPath);

        SchemaValidator validator = new SchemaValidator();

        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);

        if (xmlSchemaPath == null) {
            DocumentTypeConfig documentType = docSearcher.findUniqueDocumentType(document);
            xmlSchemaPath = documentType.getSchemaPath();
        }
        log.info(" - Schema Path : " + xmlSchemaPath);
        
        XmlSchema schema = new XmlSchema(xmlSchemaPath);

        validator.schemaValidateXmlDocument(document, schema);
    }
}
