/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.xml.schematron;

import java.io.FileNotFoundException;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.SchematronErrorException;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * perform schematron validation.
 */
public class SchematronValidationTest extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(SchematronValidationTest.class);

    /**
     * document searcher.
     */
    private static DocumentTypeConfigSearcher docSearcher;

    /**
     * init doc searcher.
     */
    public void testInitDocumentTypes() {
        try {
            docSearcher = new DocumentTypeConfigSearcher();
            assertTrue(true);
        } catch (MainException e) {
            log.error("Init DocumentTypeConfigSearcher should not fail : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test invoice.
     */
    public void testOioublInvoiceValidation() {
        log.info("Positive test : testOioublInvoiceValidation");
        try {
            validate(TestConstants.PATH_INVOICE_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublInvoiceValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test application response.
     */
    public void testOioublApplicationResponseValidation() {
        log.info("Positive test : testOioublApplicationResponseValidation");
        try {
            validate(TestConstants.PATH_APPLICATIONRESPONSE_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublApplicationResponseValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test credit note.
     */
    public void testOioublCreditNoteValidation() {
        log.info("Positive test : testOioublCreditNoteValidation");
        try {
            validate(TestConstants.PATH_CREDITNOTE_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublCreditNoteValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test order.
     */
    public void testOioublOrderValidation() {
        log.info("Positive test : testOioublOrderValidation");
        try {
            validate(TestConstants.PATH_ORDER_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublOrderValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test order response.
     */
    public void testOioublOrderResponseSimpleValidation() {
        log.info("Positive test : testOioublOrderResponseSimpleValidation");
        try {
            validate(TestConstants.PATH_ORDERRESPONSESIMPLE_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublOrderResponseSimpleValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test reminder.
     */
    public void testOioublReminderValidation() {
        log.info("Positive test : testOioublReminderValidation");
        try {
            validate(TestConstants.PATH_REMINDER_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublReminderValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test invoice 07.
     */
    public void testOioxmlInvoiceValidation07() {
        log.info("Positive test : testOioxmlInvoiceValidation - 0.7");
        try {
            validate(TestConstants.PATH_INVOICE07_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioxmlInvoiceValidation should not fail" + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test creditnote 07.
     */
    public void testOioxmlCreditNoteValidation07() {
        log.info("Positive test : testOioxmlCreditNoteValidation - 0.7");
        try {
            validate(TestConstants.PATH_CREDITNOTE07_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioxmlInvoiceValidation should not fail" + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * check schematron validation failure.
     */
    public void testOioublNoIdInvoiceValidation() {
        log.info("Failure test : testOioublNoIdInvoiceValidation");
        try {
            validate(TestConstants.PATH_INVOICENOID_XML);
            fail("testOioublNoIdInvoiceValidation should fail");
        } catch (SchematronErrorException e) {
            log.info("Failed as expected : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublNoIdInvoiceValidation() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * check schematron validation failure.
     */
    public void testOioxmlInvalidEanNumberInvoiceValidation() {
        log.info("Failure test : testOioxmlInvalidEanNumberInvoiceValidation");
        try {
            validate(TestConstants.PATH_INVOICEN7INVALIDEANNUMBER_XML);
            fail("testOioxmlInvalidEanNumberInvoiceValidation should give error");
        } catch (SchematronErrorException e) {
            log.info("Failed as expected : " + e);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioxmlInvalidEanNumberInvoiceValidation should give error " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * perform validation.
     * 
     * @param xmlDocumentPath
     *            path to document
     * @throws FileNotFoundException 
     * @throws MainException 
     */
    private void validate(String xmlDocumentPath) throws FileNotFoundException, MainException {
        log.info("Schematron validation started " + xmlDocumentPath);
        
        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);

        DocumentTypeConfig documentType = docSearcher.findUniqueDocumentType(document);
        SchematronValidationConfig schematronValidationConfig = documentType.getSchematronValidationConfig();

        log.info("Document type = " + documentType.getFriendlyName());
        //log.info("Document = \n" + document.getOMDocument());
        SchematronValidator validator = new SchematronValidator(schematronValidationConfig);

        validator.schematronValidateXmlDocument(document);
    }

}
