/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.xml.xpath;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import junit.framework.TestCase;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;

/**
 * test xpath.
 */
public class TestXPaths extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(TestXPaths.class);

    /**
     * document searcher.
     */
    private static DocumentTypeConfigSearcher searcher;

    /**
     * Initialize searcher.
     */
    public void testSetUp() {

        try {
            searcher = new DocumentTypeConfigSearcher();
            assertTrue(true);
        } catch (MainException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * find expected result in document.
     */
    public void testApplicationResponse() {
        doTest(TestConstants.PATH_APPLICATIONRESPONSE_XML, "28186371");
    }

    /**
     * find expected result in document.
     */
    public void testCreditNote() {
        doTest(TestConstants.PATH_CREDITNOTE_XML, "5798009811578");
    }

    /**
     * find expected result in document.
     */
    public void testInvoice() {
        doTest(TestConstants.PATH_INVOICE_XML, "5798009811578");
    }

    /**
     * find expected result in document.
     */
    public void testOrder() {
        doTest(TestConstants.PATH_ORDER_XML, "18018845");
    }

    /**
     * find expected result in document.
     */
    public void testOrderResponseSimple() {
        doTest(TestConstants.PATH_ORDERRESPONSESIMPLE_XML, "5798009811578");
    }

    /**
     * find expected result in document.
     */
    public void testReminder() {
        doTest(TestConstants.PATH_REMINDER_XML, "5798009811585");
    }

    /**
     * Utility method to perform test.
     * 
     * @param path
     *            path to document
     * @param validateKey
     *            expected result
     */
    private void doTest(String path, String validateKey) {
        try {
            log.info("Look for key : " + validateKey + " in " + path);
            XmlDocument document = new XmlDocument();
            document.load(path);

            IIdentifier identifier = Utilities.getReceiverEndpointKey(document, searcher.tryFindUniqueDocumentType(document));
            
            assertEquals(validateKey, identifier.getAsString());
            log.info("Found match  : " + identifier.getAsString());
        } catch (Exception e) {
            log.error("Failed to get " + validateKey + " : " + e, e);
            fail("Failed to get " + validateKey + " :" + e);
        } catch (Throwable e) {
            log.error("Failed to get " + validateKey + " : " + e, e);
            fail("Failed to get " + validateKey + " :" + e);
        }

    }
}
