/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.xml.xslt;

import java.io.FileNotFoundException;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import dk.gov.oiosi.xml.schematron.SchematronValidator;
import dk.gov.oiosi.xml.xslt.XsltUtility;

/**
 * perform schematron validation.
 */
public class XsltTransformationTest extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(XsltTransformationTest.class);

    /**
     * document searcher.
     */
    private static DocumentTypeConfigSearcher docSearcher;

    /**
     * Initialize doc searcher.
     */
    public void testInitDocumentTypes() {
        try {
            docSearcher = new DocumentTypeConfigSearcher();
            assertTrue(true);
        } catch (MainException e) {
            log.error("Init DocumentTypeConfigSearcher should not fail : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * Test invoice.
     */
    public void testOioublInvoice() {
        log.info("Positive test : testOioublInvoice");
        try {
        	docSearcher = new DocumentTypeConfigSearcher();
            transform(TestConstants.PATH_INVOICE_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublInvoice() failed : " + e, e);
            fail(e.getMessage());
        }
    }


    /**
     * test credit note.
     */
    public void testOioublCreditNote() {
        log.info("Positive test : testOioublCreditNote");
        try {
        	docSearcher = new DocumentTypeConfigSearcher();
            transform(TestConstants.PATH_CREDITNOTE_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioublCreditNote() failed : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test invoice 07.
     */
    public void testOioxmlInvoice07() {
        log.info("Positive test : testOioxmlInvoice07");
        try {
        	docSearcher = new DocumentTypeConfigSearcher();
            transform(TestConstants.PATH_INVOICE07_XML);
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioxmlInvoiceValidation should not fail" + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test creditnote 07.
     */
    public void testOioxmlCreditNote07() {
        log.info("Positive test : testOioxmlCreditNote07");
        try {
        	docSearcher = new DocumentTypeConfigSearcher();
            transform(TestConstants.PATH_CREDITNOTE07_XML);
            //transform("test/Xml/PCM.xml");
            assertTrue(true);
        } catch (Exception e) {
            log.error("testOioxmlCreditNote07 should not fail" + e, e);
            fail(e.getMessage());
        }
    }


    /**
     * perform validation.
     * 
     * @param xmlDocumentPath
     *            path to document
     * @throws FileNotFoundException 
     * @throws MainException 
     */
    private void transform(String xmlDocumentPath) throws FileNotFoundException, MainException {
        log.info("Transform and Test document : " + xmlDocumentPath);
        
        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);

        DocumentTypeConfig documentType = docSearcher.findUniqueDocumentType(document);
        log.info(" - DocumentType : " + documentType.getFriendlyName());

        String stylesheetPath = documentType.getXsltTransformStylesheetPath();
        log.info(" - stylesheet : " + stylesheetPath);
        
        XmlDocument stylesheet = new XmlDocument();
        stylesheet.load(stylesheetPath);

        //
        XsltUtility xsltUtility = new XsltUtility();
        XmlDocument transformed = xsltUtility.transformXML(document, stylesheet);
        
        
        DocumentTypeConfig transformedDocumentType = docSearcher.findUniqueDocumentType(transformed);
        log.info(" - Transformed DocumentType : " + transformedDocumentType.getFriendlyName());

        
        // Schema validate document...
        log.info(" - schema : " + transformedDocumentType.getSchemaPath());
        SchemaValidator schemaValidator = new SchemaValidator();
        XmlSchema schema = new XmlSchema(transformedDocumentType.getSchemaPath());
        schemaValidator.schemaValidateXmlDocument(transformed, schema);
        log.info(" - schema validate ok");
        

        // Schematron validate document
        log.info(" - schmeatron stylesheet : " + transformedDocumentType.getSchematronValidationConfig().getSchematronDocumentPath() );
        SchematronValidationConfig schematronValidationConfig = transformedDocumentType.getSchematronValidationConfig();
        SchematronValidator schematronValidator = new SchematronValidator(schematronValidationConfig);
        schematronValidator.schematronValidateXmlDocument(transformed);
        log.info(" - schematron validate ok");
    }

}
